import React, { useEffect, useState } from 'react'
import { BinderBar } from './components/binderbar';

import { Overlays } from './components/overlays';
import './App.css'
//import { NavBar } from './components/navbar'
import { useStatorContext } from './statorobject';
import { OpenBinders} from './components/openbinders'

import { ContextMenu } from './components/contextmenu';
import { TagSidebar, SidebarViewToggleButton} from './components/tagsidebar';
import { HamburgerMenu } from './components/hamburgermenu';
import { NagScreen } from './components/nagscreen';
import { getActivationStator } from './activation';
import { getUpdaterStator } from './updaterstator';
import { Notifications } from './components/notifications';
import {BuildInfo, getBuildInfo} from "./ipcbindings";
import {UploadController} from "./components/uploadcontroller";
import {localNodeConfig} from "./localnodeconfig"

import './components/navbar.css';
import { EulaScreen } from './components/eula';
import {ObjectPropertiesSidebar} from "./components/objectpropertiessidebar";


//I gave up on trying to make rollup work correctly, just put electron-main dependencies here:
//import { v4 } from 'uuid';

//Layout is as such
//--------- notif area
//|########
//|########
//|########
//^   ^
//|   L dynamicContentArea
//L mainsidebar

//Dynamic content area is broken up into two rows vertically
// navbar on top, content area on bottom.




function App()
{
  const appContext = useStatorContext();
  const [configLoaded, setConfigLoaded] = useState<boolean>(false);
  const [eulaAgreed, setEulaAgreed] = useState<boolean>(false);

  //activateWithKey("6AF6D30F-140B499E-A2E8EF92-D1A4D3E2");

  /*
  useEffect(() => {
    return getRootStator(appContext).mount();
  }, [1]);

   */
  useEffect(() => {
    getBuildInfo().then((val: BuildInfo) => {
      document.title = val.appTitle;
    });
  }, [1]);

  useEffect(() => {
    if(localNodeConfig.checkLoadInitiated()) return;

    (async () => {
      await localNodeConfig.load();
      setEulaAgreed(await localNodeConfig.checkEulaAgreement());
      setConfigLoaded(true);
    })();
  }, []);


  /*
  //Initialize binder list
  useEffect(() =>
  {
    allBindersEndpoint()
      .then((freshBinders: Array<Binder>) =>
      {
        binderListStator.syncToBinderList(freshBinders);

        //Normally it's the last in the list we want the first here
        if (freshBinders.length)
          binderListStator.setSelectedBinder(freshBinders[0].id);
      })
      .catch((msg: string) =>
      {
        console.log("Got error while trying to query binders: " + msg);
      });

    //No unregister, cause this is a lifetime object and should only run once
    //except for vite, dumbass...
  }, []);
  */

  /*
  useEffect(() => {
    const id = setInterval(() => {
      const binderListStator = getBinderListStator(appContext);
      binderListStator.flushBinders();
    }, 1000);

    return () => {
      clearInterval(id);
    };
  }, [appContext]);
  */

  useEffect(() =>
  {
    //Very important this is done somewhere...
    const activationStator = getActivationStator(appContext);
    activationStator.initializeActivation();

    //We want the updater to follow activation, cause otherwise
    // it'll say something wrong like it's not valid for your key, so
    // we need to authorize first and then the updater message will be gud.
    const updateStator = getUpdaterStator(appContext);
    activationStator.registerCallback((gen: number) =>
    {
      updateStator.checkForUpdates();
    });

  }, [appContext]);


  const agreedammit = async (e) => {
    e.preventDefault();

    console.log("Agree! but skipped");

    await localNodeConfig.setEulaAgreed();
    setEulaAgreed(await localNodeConfig.checkEulaAgreement());
  }

  console.log("Eula agreed: " + eulaAgreed);

  if(eulaAgreed && configLoaded) {
    return (<div className="app-root">
      <div className="second-level">
        <div className="left-sidebar">
          <HamburgerMenu
            appContext={appContext}
            buttonStyleClass="nav-button binder-bar-ham-burber-bubbon" />
          <BinderBar appContext={appContext} />
          <SidebarViewToggleButton appContext={appContext} />
        </div>

        <TagSidebar appContext={appContext} />

        <div className="third-level">
          <OpenBinders appContext={appContext} />
          <Notifications appContext={appContext} />
        </div>

      </div>
      <Overlays appContext={appContext} />

      <NagScreen appContext={appContext} />

      <UploadController appContext={appContext} />

      <ContextMenu appContext={appContext} />
    </div>);
  } else if(configLoaded && !eulaAgreed) {
    return (
      <EulaScreen agreeCallback={agreedammit}/>
    )
  } else {
      return (<div className="app-root"></div>);
  }

}

export default App

/*
function App() {
  const [count, setCount] = useState(.01)


  return (
    <div className="App">
      <header className="App-header">
        <img src={logo} className="App-logo" alt="logo" />
        <p>Hello Vite + React!</p>
        <p>
          <button type="button" onClick={() => setCount((count) => count * 2 )}>
            count is: {count}
          </button>
        </p>
        <p>
          Edit <code>App.tsx</code> and save to test HMR updates.
        </p>
        <p>
          <a
            className="App-link"
            href="https://reactjs.org"
            target="_blank"
            rel="noopener noreferrer"
          >
            Learn React
          </a>
          {' | '}
          <a
            className="App-link"
            href="https://vitejs.dev/guide/features.html"
            target="_blank"
            rel="noopener noreferrer"
          >
            Vite Docs
          </a>
        </p>
      </header>
    </div>
  )
}

*/

