import { Activation } from './activation';
import { Binder } from './binder';
import {backendEntrypoint} from './ipcbindings'

type BinderConnectionId = string;
type TagId = string;
type ObjectId = string;


class BinderTag
{
  name: string = "";
  id: TagId = "";
  children: Array<BinderTag> = [];

  /*

  clone() {
    const result = new BinderTag();

    result.name = this.name;
    result.id = this.id;

    for(const child of this.children)
      result.children.push(child.clone());

    return result;
  }
  */
}

class IndexedObject {
  id: ObjectId = "";
  index: number = 0;
}

class BinderObject
{
  id: ObjectId = "";
  b2bsum: string = "";
  index: number = 0;
  mime: string = "";

  objectOnDiskPath: string = "";
  thumbnailOnDiskPath: string = "";

  //Since object -> tag is a generally a smaller set
  // than tag -> object, we store the relationship here, to avoid
  // loading potentially hundreds of thousands of objects per tag.
  //Each object has to be loaded, period, and it's tags are already loaded,
  // so yeh.

  tags = new Set<TagId>();

  /*
  clone() {
    const result = new BinderObject();
    result.id = this.id;
    result.imageDisplayURI = this.imageDisplayURI;
    result.videoDisplayURI = this.videoDisplayURI;

    return result;
  }

  */

}


// we can do caching in here
// so like, we can fetch object lists from the database, but then
// if we need them again just memoize, and if we need the intersection
// of two, again, just do that here.
//In fact maybe just don't even have the multi-tag select in the backend at all.
// the "objectsForTagsEndpoint" below accepts a list, but the backend need not.


//Uh oh, need a "binderId"... Which we don't have in here oopsies.

//const binderApiUrl = "http://localhost:18080/api/binder/";

interface CommonResponse
{
    statusCode: number;
    userErrorMessage: string;
}

interface ObjectsForTagResponse extends CommonResponse
{
  objects: Array<BinderObject>;
}




async function commonFetch<Type extends CommonResponse>(endpoint: string, requestBody: any, abortSig?: AbortSignal): Promise<Type>
{
    /*
    const authKey = await getAuthKey();

    const options: RequestInit = {
        method: "POST",
        body: requestBody,
        headers: {"Content-Type": "application/json", "auth-key": authKey}
    };

    if(abortSig)
        options.signal = abortSig;

    const request = new Request(binderApiUrl + endpoint, options);
    const response = await fetch(request);


     */

    const startTagRequest = Date.now();
    const response = await backendEntrypoint(endpoint, requestBody);
    const tagRequestTime = Date.now() - startTagRequest;
    console.log("request time (" + endpoint + "): " + tagRequestTime);

    let result: Type; //LET. JASON. TYPE.

    try
    {
        const startTagRequest = Date.now();
        result = JSON.parse(response);
        const tagRequestTime = Date.now() - startTagRequest;
        console.log("parse time (" + endpoint + "): " + tagRequestTime);
    }
    catch
    {
        console.log("Response not json!");
        console.log(response);

        throw "failed to process request";
    }

    if(result.statusCode != 200)
    {
        console.log("Statuscode not 200! is: " + result.statusCode);
        console.log("server message is: " + result.userErrorMessage);
        throw result.userErrorMessage; //doesn't fetch throw?
    }

    return result;
}

interface ActivationStatusResponse extends CommonResponse
{
  activation: any;
};

async function queryActivationStatus(): Promise<Activation>
{
  const requestBody = ({});

  const respbody = (await commonFetch<ActivationStatusResponse>("activationstatus", {})).activation;

  return respbody;
}

async function saveActivation(payload: Activation)
{
  await commonFetch("activate", (payload));
}

interface ConnectToBinderResponse extends CommonResponse
{
  result: Binder;
}


async function createBinderEndpoint(name: string, directory: string): Promise<Binder>
{
  const requestBody = ({
    "clusterParentDirectory": directory,
    "clusterName": name
  });

  return (await commonFetch<ConnectToBinderResponse>("createbindercluster", requestBody)).result;
}

async function disconnectFromBinderEndpoint(binderId: BinderConnectionId)
{
  throw new Error("Don't use this funciton!!!");
  const requestBody = ({
    binderId: binderId
  });

  await commonFetch<CommonResponse>("disconnectfrombinder", requestBody);

  return;
}

async function connectToExistingEndpoint(binderPath: string): Promise<Binder>
{
  const requestBody = ({
    "clusterPath": binderPath,
  });

  return (await commonFetch<ConnectToBinderResponse>("connecttoexisting", requestBody)).result;
}

interface AllBindersResponse extends CommonResponse
{
  binders: Array<Binder>;
}

async function allBindersEndpoint(): Promise<Array<Binder>>
{
  return (await commonFetch<AllBindersResponse>("allbinders", {})).binders;
}

interface AllTagsResponse extends CommonResponse
{
  tags: Array<BinderTag>;
}

async function allTagsEndpoint(theBinderId: BinderConnectionId): Promise<Array<BinderTag>>
{
  const requestBody = ({"binderId": theBinderId})
  return (await commonFetch<AllTagsResponse>("alltags", requestBody)).tags

 /*
  if(theBinderId == ":3c")
  {
    return [
      { name: "whee", id: "1234", children: [] },
      { name: "hee", id: "234", children: [] },
      { name: "whee", id: "4", children: [{ name: "wheeeeeeeeekkkkkkkkkkkkkkkkk", id: "5", children: [{ name: ":3", id: "5555", children: [] }] }] },
      { name: "blee!!", id: "34", children: [] },
    ];
  }
  if(theBinderId == ":fjidfj3c")
  {
    return [
      { name: "OwO", id: "fjf38383390278j", children: [] },
      { name: "OwO 2", id: "fjf38383390278j88", children: [] },
      { name: "Owo", id: "fjf38383390278jjf", children: [] }
    ]
  }
  if(theBinderId == ":fjidfj3cjfuhudhf")
  {
    //This one uses the same ids as above to check against spooky action at a distance
    return ([
      { name: "uwu", id: "fjf38383390278j", children: [] },
      { name: "uwu 2", id: "fjf38383390278j88", children: [] },
      { name: "uwu 3", id: "fjf38383390278jjf", children: [] }
    ]);
  }

  return [];
//   */
}



async function objectsForTagEndpoint(theBinderId: BinderConnectionId,
                                     tags: Array<TagId>,
                                     sig: AbortSignal)
{
  const requestBody = (
      {
        binderId: theBinderId,
        tags: tags //the problem is the api wants a single tag but we have all these damn tags...
        //The million dollar question is, do we do the intersection here or on the backend?
      })

  return (await commonFetch<ObjectsForTagResponse>("objectsontags", requestBody, sig)).objects;

  /*

  const result = new Array<BinderObject>();

  const tagSet1 = new Set<TagId>(["1234"]);
  const tagSet2 = new Set<TagId>(["1234"]);
  const tagSet3 = new Set<TagId>(["1234"]);


  //779 objects
  let alternator: number = 0;
  for(let i: number = 0; i != 790; ++i) {
    let tagSet = new Set<TagId>();

    if(alternator == 0) tagSet = new Set(tagSet1);
    else if(alternator == 1) tagSet = new Set(tagSet2);
    else if(alternator == 2) tagSet = new Set(tagSet3);
    else throw Error("fuck it");

    if(alternator == 2) {
      alternator = 0;
    } else { alternator = alternator + 1; }


    result.push({id: i.toString(), imageDisplayURI: "", videoDisplayURI: "", tags: tagSet})
  }

  return result;
   */
}

//Uses common response

//Uses common response
async function internalModifyTagRelationship(endpoint: string,
                                             theBinderId: BinderConnectionId,
                                             theTag: TagId,
                                             theObjects: Array<ObjectId>)
{
  const requestBody = (
      {
        binderId: theBinderId,
        tagId: theTag,
        objects: theObjects,
      }
  );

  return (await commonFetch<CommonResponse>(endpoint, requestBody));
}

async function addTagToObjects(theBinderId: BinderConnectionId,
                               theTag: TagId,
                               theObjects: Array<ObjectId>)
{
  return (await internalModifyTagRelationship("addtagtoobjects",
                                              theBinderId,
                                              theTag,
                                              theObjects));
}

//removetagfromobjects
async function removeTagFromObjects(theBinderId: BinderConnectionId,
                                    theTag: TagId,
                                    theObjects: Array<ObjectId>)
{
  return (await internalModifyTagRelationship("removetagfromobjects",
      theBinderId,
      theTag,
      theObjects));
}


interface NewTagResponse extends CommonResponse
{
  tagId: TagId;
}

async function createTagEndpoint(theBinderId: BinderConnectionId,
                                 theName: string,
                                 tagParent: TagId): Promise<TagId>
{
  const requestBody = (
      {
        binderId: theBinderId,
        tagName: theName,
        tagParentId: tagParent
      });

  return (await commonFetch<NewTagResponse>("createtag", requestBody)).tagId;
}

async function renameTagEndpoint(theBinderId: BinderConnectionId,
                                 tagId: TagId,
                                 theNewName: string)
{
  const requestBody = (
      {
        binderId: theBinderId,
        tagId: tagId,
        newName: theNewName,
      }
  );

  return (await commonFetch<CommonResponse>("settagname", requestBody));
}

//settagparent


async function setTagParent(theBinderId: BinderConnectionId,
                            tag: TagId,
                            parentToBe: TagId)
{
    const requestBody = (
        {
            binderId: theBinderId,
            tagId: tag,
            newParentTagId: parentToBe
        }
    )

    return (await commonFetch<CommonResponse>("settagparent", requestBody))
}

export enum Position
{
  Before,
  After,
}

async function repositionTag(theBinderId: BinderConnectionId,
                             tag: TagId,
                             tagOther: TagId,
                             thePosition: Position)
{
  const posString = thePosition == Position.Before ? "before" : "after";

  const requestBody = (
    {
      binderId: theBinderId,
      tagId: tag,
      otherTagId: tagOther,
      position: posString,
    }
  )

  return (await commonFetch<CommonResponse>("repositiontag", requestBody));
}

async function deleteTag(theBinderId: BinderConnectionId, tag: TagId)
{
  const requestBody = (
    {
      binderId: theBinderId,
      tagId: tag
    }
  );

  return (await commonFetch<CommonResponse>("deletetag", requestBody));

}

interface CreateObjectResponse extends CommonResponse
{
  skipped: Array<BinderObject>;
  forcedDuplicates: Array<BinderObject>;
  taggedDuplicates: Array<BinderObject>;
  objects: Array<BinderObject>;
}

async function createObject(theBinderId: BinderConnectionId,
                            baseTag: TagId,
                            additionalTags: Array<TagId>,
                            theFiles: Array<string>)
{
  const requestBody = (
      {
        binderId: theBinderId,
        duplicateBehavior: "tagDuplicate",
        baseTag: baseTag,
        tags: additionalTags,
        paths: theFiles
      });

  return await commonFetch<CreateObjectResponse>("addfiles", requestBody);
}

interface ExportObjectResponse extends CommonResponse
{

}

async function exportObject(theBinderId: BinderConnectionId,
                            dest: string,
                            objectsByB2Sum: Array<string>)
{
    const requestBody = (
        {
            binderId: theBinderId,
            dest: dest,
            objectsB2: objectsByB2Sum,
        });

    return await commonFetch<ExportObjectResponse>("exportobjects", requestBody);
}

async function generateThumbnailForObject(theBinderId: BinderConnectionId,
                                          b2sum: string)
{
    const requestBody = (
        {
            binderId: theBinderId,
            b2sum: b2sum,
        });

    return await commonFetch<CommonResponse>("generatestandardobjectthumbnail", requestBody);
}

async function syncCluster(theBinderId: BinderConnectionId)
{
    const requestBody = (
        {
            binderId: theBinderId,
        }
    );

    return await commonFetch<CommonResponse>("synccluster", requestBody);
}

function getThumbnailUri(obj: ObjectId) {
  console.warn("REimpliment getThumbanilUrls, dubmadss");
  return "";
  //*if electron TODO
  //return "file:///" + obj.thumbnailOnDiskPath;
  //*else
  //return binderApiUrl + "standardobjectthumbnail" + "/" + theBinderId + "/" + b2bsum;
}

function getObjectUri(obj: ObjectId) {
  //*if electron TODO
  console.warn("REimpliment getObjectURI, dubmadss");
  return "";
  //return "file:///" + obj.objectOnDiskPath;
  //*else
//    return binderApiUrl + "image" + "/" + theBinderId + "/" + b2bsum;
}


export {
  Binder,
  BinderTag,
  BinderObject,

    getObjectUri,
    getThumbnailUri,

  queryActivationStatus,
  saveActivation,
  createBinderEndpoint,
  disconnectFromBinderEndpoint,
  connectToExistingEndpoint,
  allBindersEndpoint,
    generateThumbnailForObject,
};

export type {
  BinderConnectionId,
  TagId,
  ObjectId,
  IndexedObject,
};
