import { BinderConnectionId } from "./backend";
import { BinderContext } from "./components/bindercontext";
import { getMetaApi } from "./ipcbindings";
import { BinderIdFilename, localNodeConfig } from "./localnodeconfig";
import { v4 } from 'uuid';


class Binder
{
  name: string = "";
  id: BinderConnectionId = "";
  exists: boolean = false;
  clusterPath: string = ""; //deprecated, same as binderId now
  context: BinderContext;

  constructor(context: BinderContext) {
    this.context = context;
  }

  static async createBinder(location: string, name: string) {
    const path = await getMetaApi().joinPath(location, name);

    if(await localNodeConfig.checkBinderExists(path)) {
        return "Binder already exists!";
        /*
        console.warn("Binder already exists, skipping create for: " + path);
        return;
        */
    }

    await getMetaApi().createDirectories(path);
    const binderIdFilePath = await getMetaApi().joinPath(path, BinderIdFilename);
    await getMetaApi().writeFile(binderIdFilePath, v4(), "utf8");

    await localNodeConfig.registerCluster(path);
    return "";
  }
}

export {Binder}