import React, {useState} from 'react';
import { StatorContext } from '../statorobject';
import { getOverlayStator } from './overlays';
import { getNagStator } from './nagscreen';
import { GenericOverlayWrapper } from './genericoverlay';
import { ActivationStatus, useWatchActivationStator } from '../activation';


interface ActivationOverlayProps
{
    appContext: StatorContext;
}

function ActivationOverlayImpl(props: ActivationOverlayProps)
{
    const activationStator = useWatchActivationStator(props.appContext);
    const standinValue = "Enter License Key";
    const storedKey = activationStator.licenseKey;
    const initialKeyVal = storedKey? storedKey : standinValue;

    const [compKey, setCompKey] = useState(initialKeyVal);

    const onChangeKey = (e: any) =>
    {
        setCompKey(e.target.value);
    };

    const gotoPurchaseOptions = (e: any) =>
    {
        getOverlayStator(props.appContext).closeOverlays();
        getNagStator(props.appContext).forceShow();
    };


    const handleSubmit = (e: any) =>
    {
        e.preventDefault();
        activationStator.activateWithKey(compKey);
    };

    const handleClick = (e: any) =>
    {
        e.preventDefault();
        if(compKey == standinValue)
        {
            setCompKey("");
        }
    };

    const Purchase = () => { return <a href="#" onClick={gotoPurchaseOptions}>purchase a license</a>};

    const statusMessage = (() =>
    {
        if(activationStator == undefined || activationStator.status == ActivationStatus.Pending)
        {
            return <p>Validation in progress, please check again in a bit.</p>
        }

        switch (activationStator.status)
        {
            case ActivationStatus.Unlicensed:
                return (
                    <p>This copy of binder is unlicensed and for evaluation purposes only.
                        Please enter a license key below or explore <a href="#" onClick={gotoPurchaseOptions}>purchase options.</a></p>
                );
            case ActivationStatus.Subscription:
            case ActivationStatus.Perpetual:
                    {
                        return (<div>
                            <h2>Activated!</h2>
                            <p>Email: {activationStator.purchaserEmail}</p>
                            {activationStator.status == ActivationStatus.Perpetual?
                            <p>Complimentary updates until: {(new Date(activationStator.updatesUntil)).toLocaleDateString()}</p>
                            :
                            <></>
                            }
                        </div>);
                    }
            case ActivationStatus.TamperTrip:
                return (<p>Activation tamper system was tripped. Either re-enter your key below or <Purchase/> to continue using Binder.</p>);
                /* This is redundant, and for some reason that's a goddamn error to typescript so w/e.
            case ActivationStatus.Pending:
                return (<p>Activation is pending, please check back in a bit.</p>);
                */
            case ActivationStatus.Refunded:
                return (<p>Your purchase was refunded and Binder is now unlicensed. Please <Purchase/> to continue using Binder</p>);
            case ActivationStatus.ChargedBacked:
                return (<p>A charge back was issued against your purchase and Binder is now unlicensed. Please <Purchase/> to continue using Binder</p>);
            case ActivationStatus.ActivationError:
                return <p>There was an error activating, please try again later.</p>
            case ActivationStatus.InvalidKey:
                return <p>Your key is invalid. Try again or <a href="#" onClick={gotoPurchaseOptions}>explore purchase options.</a></p>
            case ActivationStatus.ExpiredSub:
                return <p>Your subscription has expired. <a href="#" onClick={gotoPurchaseOptions}>Explore purchase options.</a></p>
        }


    })();

    return (
        <div>
            <h1>Activation</h1>
            {statusMessage}
            <form onSubmit={handleSubmit} spellCheck={false}>
                <input type="text" onClick={handleClick} name={"licenseKey"} value={compKey} onChange={onChangeKey} autoComplete="off" />
                <input type="submit" className={"generic-overlay-form-button"} value={"Activate"} disabled={false}/>
            </form>
    </div>);
}

function ActivationOverlay(props: ActivationOverlayProps)
{
    return (<GenericOverlayWrapper
        appContext={props.appContext}
        startTab={0}
        tabs={[{
            title: "Activate", /* Unused cause one tab but good practice nonethelss */
            nestedComponent: ActivationOverlayImpl,
            nestedProps: props,
        }]}
    />);
}

export { ActivationOverlay }