import React from 'react'
import { Binder } from '../backend'
import { StatorContext } from '../statorobject';
import { getBinderListStator, useWatchBinderListStator } from './openbinders';
import './binderbar.css'
import './hamburgermenu.css'
import './vibrantactionbutton.css'
import { getOverlayStator, OverlayStatorStates } from './overlays';
import { ContextMenuData, getContextMenuStator } from './contextmenu';
import { localNodeConfig } from '../localnodeconfig';

interface BinderBarProps
{
    appContext: StatorContext;
};

interface BinderButtonProps
{
    myBinder: Binder;
    current: Boolean;
    appContext: StatorContext;
};

function BinderButton(props: BinderButtonProps)
{
    const contextMenuHandler = (e: React.MouseEvent) => {
        const menu: ContextMenuData = {
            spawnX: e.pageX,
            spawnY: e.pageY,
            contextMenuSections: [ {
                        contextMenuItems: [ {
                                    title: "Disconnect from: " + props.myBinder.name,
                                    keyBinding: "",
                                    action: () => {
                                        localNodeConfig.removecluster(props.myBinder.id);
                                        //disconnectFromBinderEndpoint(props.myBinder.id);
                                        //getBinderListStator(props.appContext).removeBinderById(props.myBinder.id);
                                    }
                                },
                            ]
                    }
                ]
        }


        e.preventDefault();
        getContextMenuStator(props.appContext).spawn(menu);
    };

    const selectThisBinderHandler = (e: React.MouseEvent) =>
    {
        e.preventDefault();
        getBinderListStator(props.appContext).setSelectedBinder(props.myBinder.id);
    }

    return (
        <div className={"binder-bar-button-container"}
        onContextMenu={contextMenuHandler}
        onClick={selectThisBinderHandler}
        >
            <div
                className={`binder-bar-button ${props.current ? "binder-bar-button-current" : ""}`}
                >
            </div>
        </div>
    );
}

function BinderBar(props: BinderBarProps)
{
    //todo Need "collapse taglist"  button at the bottom
    // but only if binderlist.length != 0

    const binderListStator = useWatchBinderListStator(props.appContext);

    const binderButtons = binderListStator.getOpenBinderList().map((pair) => {
        return (
            <BinderButton
                key={pair.binder.id}
                myBinder={pair.binder}
                current = {binderListStator.checkSelected(pair.binder.id)}
                appContext={props.appContext}/>
        );
    })

    const launchNewBinderOverlay = () => 
    {
        const overlayStator = getOverlayStator(props.appContext);
        overlayStator.set(OverlayStatorStates.showNewBinder);
    }

    return (

        <div className="binder-bar">
            {binderButtons}
            <div className={"binder-bar-button-container"}
                onClick={launchNewBinderOverlay}
            >

                <div className="binder-bar-button binder-bar-new-binder-button">
                    <img className="binder-bar-new-binder-button-icon"
                        src={"static/plus-button.svg"}
                        draggable={false}
                    />

                </div>
            </div>
        </div>
    );
}

export {BinderBar};