import {BinderConnectionId, IndexedObject, ObjectId, Position, TagId} from "../backend";
import {TagAction} from "./taglistitem";
import {
    alwaysTriggerFilter,
    differenceFilter,
    Observable,
    ObservableCounter,
    ObservableCounterMap,
    ObservableMap,
    ObservableValue,
    zeroOrManyFilter
} from "../observermanager";
import {TagDatabase, reserved} from "../tagdatabase";
import { UploadControllerStator } from "./uploadcontroller";
import { FileTree } from "../filetree";
import { getMetaApi, openSelectDirectory } from "../ipcbindings";

function tagActionFilter(newValAsPassed: TagAction | undefined, oldValAsPassed: TagAction | undefined): boolean {
    const newVal = newValAsPassed == undefined ? TagAction.Add : newValAsPassed;
    const oldVal = oldValAsPassed == undefined ? TagAction.Add : oldValAsPassed;

    //In a selection, it's implied that you show the add button if you know nothing else.
    //We only want to update observers that need something different from that.
    //That way, we only have to call all observing tag items once when a selection starts
    // and then only incrementally call the others as the selection changes.
    //So if the value is undefined, and it's set to add well, we ignore it.
    // It gets set to add if you remove all selected tags for example,
    // and it needs that update pushed back to it...

    //honestly though, we should only ever pass undefined into here...

    return newVal != oldVal;
}


class ParentsWhoseChildrenHaveSomethingToSayContainer {
    readonly counter = new ObservableCounterMap<TagId>(zeroOrManyFilter);
    readonly alwaysCounter = new ObservableCounterMap<TagId>(alwaysTriggerFilter);
    readonly somethingToSayDeps = new Map<TagId, Set<TagId>>();
/*
    purgeReference(db: TagDatabase, tagId: TagId) {
        //The problem with this function........
        //Is that it's not recursive!!!
        //Why aren't we using the removeChild functions???
        //uhh....... they need parental relationships which are gone..
        // if it's deleted now...
        //But wait the relationships are broken *first* then we can get rid of it
        // ahead of the child delete right? Like if.. yeah


        //What if... ugh... idk... We recursively call purge?
        throw new Error("This isn't good here, nope");
        for(const [parent, depsSet] of this.somethingToSayDeps) {
            if(parent == tagId) {
                //delete the whole entry, child deps are irrelevant now
                this.counter.set(parent, undefined);
                this.somethingToSayDeps.delete(parent);
            } else if(tagId in depsSet) {
                //Okaaaay now if there are child deps we do the thing
                depsSet.delete(tagId);

                if(depsSet.size == 0) {
                    //nothing left to say :3
                    this.counter.set(parent, undefined);
                    this.somethingToSayDeps.delete(parent);
                } else {
                    this.counter.set(parent, depsSet.size);
                }
            } //else who cares lol
        }

        //If everything is synced, then there shouldn't be anything in the counter referencing this tag
        // anymore anyway
        if(this.counter.has(tagId)) {
            throw new Error("Inconsistent counter state");
        }
    }
    */

    getChildren(parent: TagId): Array<TagId> {
        const result = new Array<TagId>();
        const theSet = this.somethingToSayDeps.get(parent);

        if(theSet) {
            let i = 0;
            for (const c of theSet) {
                ++i;
                result.push(c);
            }
        }

        return result;
    }

    
    addChild(db: TagDatabase, tagId: TagId) {
        const parent = db.getTagParent(tagId);

        if(!parent || parent == reserved.tagIds.root) {
            return;
        }

        const myDeps: Set<TagId> = (() => {
            let cont = this.somethingToSayDeps.get(parent);

            if(!cont) {
                cont = new Set<TagId>();
                this.somethingToSayDeps.set(parent, cont);
            }

            return cont;
        })();
        

        /*
        if(tagId in myDeps) {
            //Assume state of parents is correct already
            //return;
            //actually fucking don'tn't because if we're in a reparent scenario, the only thing
            // we need to do is update parents...
        }
        */

        //add mode!! :3
        myDeps.add(tagId);

        this.counter.set(parent, myDeps.size); //prevent double add cause we don't skip tags already in anymore
        this.alwaysCounter.set(parent, myDeps.size);
        this.addChild(db, parent);
    }

    removeChild(db: TagDatabase, childId: TagId, parent: TagId | undefined = undefined) {
        if (!parent)
            parent = db.getTagParent(childId);

        if(!parent || parent == reserved.tagIds.root) {
            return;
        }

        const myDeps = this.somethingToSayDeps.get(parent);

        if(!myDeps || !myDeps.has(childId)) {
            return;
        }

        myDeps.delete(childId);

        const depsSize = myDeps.size == 0 ? undefined : myDeps.size;

        if(!depsSize) {
            this.somethingToSayDeps.delete(parent);
        }

        //console.log("Depsize for child: " + depsSize);
        this.counter.set(parent, depsSize);
        this.alwaysCounter.set(parent, depsSize);
        this.removeChild(db, parent);
    }
}


enum ContextStatus {
    LoadingTags, //initial state
    LoadingTagSelection, //interim between selections
    Ready, //self explanatory so why am I typing this comment
    Searching,
}


class BinderContext {
    private binderId;
    readonly db: TagDatabase;

    readonly status = new ObservableValue<ContextStatus>(differenceFilter);

    readonly currentOpenObject = new ObservableValue(differenceFilter);

    readonly objectsSelectedOrNotSignal = new ObservableCounter(zeroOrManyFilter); //for the action observers
    readonly exactSelectedObjectCounter = new ObservableCounter(differenceFilter); //for the selection counter

    readonly searchQueryString = new ObservableValue<string | undefined>(differenceFilter);
    //readonly tagsToShowWhileFiltered = new ObservableValue<Set<TagId>>(differenceFilter);
    readonly objectQueue = new ObservableValue<Array<IndexedObject>>(differenceFilter);


    readonly tagSelectionChangeWatcher = new Observable();
    readonly tagActionObservers = new ObservableMap<TagId, TagAction | undefined>(tagActionFilter);
    readonly objectSelectionObservers = new ObservableMap<ObjectId, boolean | undefined>(differenceFilter);

    //Should these only be set from the taglistitem itself? hmm...
    readonly tagSelectionSentinel = new ObservableMap<TagId, boolean | undefined>(differenceFilter);

    //readonly selectedTags = new ObservableValue(differenceFilter);

    private parentsWhoseChildrenHaveSomethingToSay = new ParentsWhoseChildrenHaveSomethingToSayContainer();
    private parentsWhoseChildrenHaveSomethingToSayGreenEdition = new ParentsWhoseChildrenHaveSomethingToSayContainer();

    //How can we *only* look at the ones not modified? Without iterating the whole set?
    //that would be nice but this might be good enough.
    //We only grow this per tag *on selected objects* everything else is w/e
    private selectedObjectCountByTag = new Map<TagId, number>();


    //Internally, for efficiency we can just use this directly, but the
    // accessor duplicates it to avoid spooky bugs at a distance
    private selectedTags = new Set<TagId>(); //DON'T alter this directly outside of setTagSelection
    private selectedObjects = new Set<ObjectId>();
    private objectQueueWorkingCopy = new Set<ObjectId>();

    private lastSingleSelectIndex = 0;

    private definitelySetRowWidth = 1;

    private rowWidthBeforeOpen = 1;
    private objectQueueBeforeOpen = new Array<IndexedObject>();

    constructor(path: BinderConnectionId) {
        this.status.set(ContextStatus.LoadingTags);

        this.binderId = path;
        this.db = new TagDatabase(path, () => {
            this.status.set(ContextStatus.Ready);
        });

        //callback if a tag is delete
        //callback if an object is deleted
        //callback if an object is added to a selected tag
        //callback if an object is removed from a selected tag
        //And reparenting, already done :3

        this.db.registerTagDeleteObserver((tagId: TagId) => {
            //Tags are deleted depth first, so the root child will be removed first so there's no need
            // to worry about going *down* the tree.
            this.parentsWhoseChildrenHaveSomethingToSay.removeChild(this.db, tagId);
            this.parentsWhoseChildrenHaveSomethingToSayGreenEdition.removeChild(this.db, tagId);

            //and TagDatabase deletes the tag before the relationships, and that's when this callback fires

            //This should clear out tag action observers (by letting them get to undefined,
            // they won't delete if they have subscribers, but the taglistitem should also catch a delete so)
            this.setTagSelection(tagId, false);

            if(this.selectedTags.size == 0) {
                //then we want to select the next closest tag

                const parent = this.db.getTagParent(tagId);
                if(this.db.checkLimitTagChildrenToObjectSelection(parent)) {
                    //then we don't actually LOL
                    return;
                }
                const tagList: TagId[] = this.db.getSortedTags(parent);

                let closestMatch: TagId | undefined = undefined;
                let reachedGoal = false;

                for (const t of tagList) {
                    if (t == tagId) {
                        reachedGoal = true;
                        continue;
                    }

                    if (!closestMatch || (closestMatch && !reachedGoal)) {
                        closestMatch = t;
                    }
                }

                //You know what... What if tag deletes aren't in order......
                // like... in a sync that's coming in out of order......
                //Is order guaranteed? It might be, but it wasn't a design requirement...

                //What if I delete a parent in one binder that gets synced in first, without deleting
                // the child on another binder? Guh...


                //if the parent still exists, then it get unmarked
                // if it doesn't exist, then it doesn't matter.
                // It's not that complicated bro

                //If the parent is invalid, then... it's an orphan and that's it's own thing?
                //hmm... don't wanna set the parent cause the orphan status could be transient...
                ///ughhhh
                //Well, proper orphaned tags are different than like, tags that are in deleted
                // subtrees, because they were not explicitly deleted, just kinda implicitly...
                //But... I don't want to properly reparent them to an "orphan" tag cause that
                // would overwrite the new parent if it's simply an inconsistent state.
                //So maybe a temporary association into an "orphaned" tag? That's not a proper
                // parent/child relationship but... Gosh then that makes it even more complicated...
                //Cause then, this whole class won't work right...
                //Maybe backdate it? IDKKKKKK
                //If it's just an inconsistent state, which is the only thing it should ever be...
                //that's not true it could very well be normal.
                //Maybe just, return the highest valid parent? Root if no other?
                //Not the *true* parent but it acts like it? And then, when the reparent comes through,
                // we can just treat it as normal reparenting.
                //Of course, if you reposition it within the pretend parent, then it should properly reparent...

                //If... the parent is weird like that, should we also trigger a reparent event though too? ughh
                //Maybe just reparent it, if it's that inconsistent... The reparent will sync back to
                // wherever else anyway, and it's a bizzaro edge case...


                //Okay I'm overthinking this... if the parent doesn't exist just deselect get over it



                if (!closestMatch) {
                    const grandParent = this.db.getTagParent(parent); //should this check for orphans?
                    if (grandParent != reserved.tagIds.root) {
                        closestMatch = grandParent;
                    }
                }

                if(closestMatch && this.db.checkTagExists(closestMatch)) {
                    this.setTagSelection(closestMatch, true);
                }//else we just deselect all and get over it w/e
            }
        });

        this.db.registerObjectDeleteObserver((objId: ObjectId) => {
            if(!this.objectQueueWorkingCopy.has(objId)) { //if it's not in the queue then we don't reference it at all
                return;
            }

            this.setSingleObjectSelection(objId, false);
            this.objectQueueWorkingCopy.delete(objId);
            this.publishObjectQueue();

            //Of course, we don't have the ability to delete objects anyway, but....
        });

        this.db.registerObjectTagLinkChangeObserver((tagId: TagId, objId: ObjectId, rel: boolean) => {
            if(this.selectedObjects.has(objId)) {
                let count;

                //Remember, this doesn't change the *selection*, all we're concerned about is how
                // it affects the tag actions <--> selection relationship... So we can
                // leave those other counters alone. We worry about whether it should
                // stay in the queue in the next section of this function

                if(rel) { 
                    count = this.incrementSelectedObjectsByTag(tagId);
                } else {
                    count = this.decrementSelectedObjectsByTag(tagId);
                }

                this.syncTagActionObserver(tagId, count);
            }



            //So what this condition does is special case removing stuff from the queue live
            // It will need to be updated accordingly if we have a "show parents but now children"
            // thing...
            //Only if rel == false, because otherwise it's a tag add, and can't
            // have invalidated the selected tags intersection

            //Of course one day this will be more complicated when we need a "negative" tag selection
            // or "red" tag selection that indicates anything with that tag should NOT be in the queue
            // ... but we're not there yet... and it should be relatively straightforward
            if(!rel && this.objectQueueWorkingCopy.has(objId)) {
                //Less efficient theoretically but lets be real here.
                this.reloadObjectQueueForTagSelection();
            } //else then it's not even a consideration :3 because we don't add objects to the queue in this condition

            const parentChain = this.db.getTagParentChain(tagId);
            parentChain.push(tagId);
            for (const ancestor of parentChain) {
                if (this.selectedTags.has(ancestor)) {
                    this.reloadObjectQueueForTagSelection();
                    break;
                }
            }

            //I... Kind of think that should do it?
        });

        this.db.registerTagParentChangeObserver((parent: string, child: string, rel: boolean) => {

            if(!rel) { //child has been removed from parent
                this.parentsWhoseChildrenHaveSomethingToSay.removeChild(this.db, child, parent);

                //the child could be a parent of the tag in selectedCountByTag...
                //Removing it should be fine, it's a no-op if it's not there.
                this.parentsWhoseChildrenHaveSomethingToSayGreenEdition.removeChild(this.db, child, parent);
            } else {
                const childAction = this.tagActionObservers.get(child);
                if(childAction == TagAction.Remove || childAction == TagAction.AddRemove) {
                    this.parentsWhoseChildrenHaveSomethingToSay.addChild(this.db, child);
                }

                //This is trickier, we only care about a reparent of a tag, whose children have something to say
                // or is selected. But convieniently... It should already be in parentswhohavesomethingtosaygreenedition
                // if the latter is the case and therefore...
                //console.log("Reparent with child :3: " + child);
                if (this.tagSelectionSentinel.get(child) || this.parentsWhoseChildrenHaveSomethingToSayGreenEdition.counter.hasCount(child)) {
                    //Of course, it's already there. But the fun part is where it goes through its parents recursively
                    //console.log("Parenting condition where child is parent with something to say or selected child");
                    this.parentsWhoseChildrenHaveSomethingToSayGreenEdition.addChild(this.db, child);

                    //seems weird, but the thing is, this is a *reparent* scenario. So the parents *aren't* marked
                    // as having green child.... And this call adds them.
                    //But and it feels weird, but we know that if it's in that container then we want to "infect"
                    // the new parents, but only if it has the "disease".
                } else {
                    //console.log("didn't do anything...");
                }

            }

            //added or removed, if one of the parents in the chain is selected...
            //wait I have a get parent chain function, why didn't I use that in the below one...
            const parentChain = this.db.getTagParentChain(parent);
            parentChain.push(parent); //just to make it easy to go over all of them

            for (const candidate of parentChain) {
                if (this.selectedTags.has(candidate)) {
                    this.reloadObjectQueueForTagSelection();
                    return;
                }
            }
        });

        this.db.registerTagChildRepositionObs((changedChildOrderTag: TagId, unsub: () => void) => {
            let candidate = changedChildOrderTag;
            while (candidate != reserved.tagIds.root) {
                if (this.selectedTags.has(candidate)) {
                    this.reloadObjectQueueForTagSelection();
                    return;
                }

                const nextCandidate = this.db.getTagParent(candidate);

                if(!nextCandidate && candidate) {
                    const exists = this.db.checkTagExists(candidate);
                    console.warn("no parent for tag: " + candidate + ", with name: " + this.db.getTagName(candidate) + ", exists? " + exists);
                    break;
                } else if(!candidate) {
                    console.log("candidate child changed order tag missing... Deleted?");
                    break;
                }

                candidate = nextCandidate;
            }
        });
    }

    openViewer(obj: IndexedObject | undefined) {
        this.rowWidthBeforeOpen = this.definitelySetRowWidth;

        const queueBeforeOpen = this.objectQueue.get();

        if(!queueBeforeOpen) {
            return;
        }

        if(!obj) {
            obj = queueBeforeOpen.at(this.lastSingleSelectIndex);

            if(!obj) {
                console.warn("Still can't find an object to open aborting");
                return;
            }
        }

        this.objectQueueBeforeOpen = queueBeforeOpen;

        this.handoffSingleObjectSelection(obj);

        this.currentOpenObject.set(obj);
    }

    areTagsSelected() {
        return this.selectedTags.size != 0;
    }

    private selectByOffset(offset: number) {
        const objectViewerOpen = this.currentOpenObject.get() ? true : false;

        let myQueue;
        if(objectViewerOpen) {
            myQueue = this.objectQueueBeforeOpen;
        } else {
            myQueue = this.objectQueue.get();
        }

        if(!myQueue) {
            console.warn("Object queue non-existent in select by offset");
            return;
        }


        let nextIndex = this.lastSingleSelectIndex + offset;

        if(nextIndex >= myQueue.length) {
            nextIndex = 0;
        }

        if(nextIndex <= -1) {
            nextIndex = myQueue.length - 1;
        }

        if(nextIndex > myQueue.length - 1) {
            throw new Error("Does this make sense?");
        }

        const optionalNextId = myQueue.at(nextIndex);

        if(!optionalNextId) {
            return;
        }

        this.lastSingleSelectIndex = nextIndex;

        this.handoffSingleObjectSelection(optionalNextId);

        if (objectViewerOpen) {
            this.currentOpenObject.set(optionalNextId);
        }
    }

    closeViewer() {
        const objId: IndexedObject = this.currentOpenObject.get()

        if(!objId) return;

        //Close first because it's a no-op if current object is open
        this.currentOpenObject.set(undefined);

        //Now cleanup
        if(!this.objectQueueWorkingCopy.has(objId.id)) {
            this.clearObjectSelection();
        }
    }

    selectNext() {
        this.selectByOffset(1);
    }

    selectPrevious() {
        this.selectByOffset(-1);
    }

    private getCorrectRowWidth() {
        if(this.currentOpenObject.get()) {
            return this.rowWidthBeforeOpen;
        } else {
            return this.definitelySetRowWidth;
        }
    }

    selectPreviousRow() {
        this.selectByOffset(-this.getCorrectRowWidth());
    }

    selectNextRow() {
        this.selectByOffset(this.getCorrectRowWidth());
    }

    setRowWidth(width: number) {
        this.definitelySetRowWidth = width;
    }

    shiftSelect(toIndex: number) {
        const queue = this.objectQueue.get();

        if(!queue) {
            return;
        }

        if(toIndex == this.lastSingleSelectIndex) {
            const obj = queue.at(toIndex);

            if (obj) {
                this.handoffSingleObjectSelection(obj);
            }
        }

        const lowerIndex = toIndex < this.lastSingleSelectIndex ? toIndex : this.lastSingleSelectIndex;
        const upperIndex = toIndex > this.lastSingleSelectIndex ? toIndex : this.lastSingleSelectIndex;

        const deselectable = new Array<IndexedObject>();

        for(let i = 0; i != queue.length; ++i) {
            const obj = queue.at(i);
            if (!obj) {
                console.warn("Bad object queue");
                continue;
            }

            if(i >= lowerIndex && i <= upperIndex) {
                this.setObjectSelectedWithoutAlertingTagObservers(obj.id);
            } else {
                deselectable.push(obj);
            }
        }

        //This prevents the "selected or not" signal from firing if it reaches zero...

        //Although I don't think that's actually possible in a shift select scenario come to think of it
        // because the thing should always be selected....
        for(const obj of deselectable) {
            this.setObjectDeselectedWithoutAlertingTagObservers(obj.id);
        }

        this.syncTagActionObserversToSelectedObjects();
    }

    getSelectedObjects() {
        return [...this.selectedObjects];
    }

    private syncTagActionObserver(tagId: TagId, selectedObjectsWithMe: number) {
        let action: TagAction | undefined;

        if (selectedObjectsWithMe == 0) { //needs to be cheked before the next ask me how i found out
            //already undefined, but also we should remove it from the list because
            // apparently it's not relevant anymore.
            this.selectedObjectCountByTag.delete(tagId);
        } else if (selectedObjectsWithMe == this.selectedObjects.size) {
            //if all the objects that are selected, match the count of the objects with me...
            // then they all have me :3
            action = TagAction.Remove;
        } else {
            action = TagAction.AddRemove;
        }

        //We need to definitely set it. Because it's set by the "any objects selected" thing first, but
        // then when we get involved with individual selections, we have to get in there and reset it,
        // without the help of that callback. So the state we call back to if we're undefined is
        // Add if any are selected and None if not

        if(action == undefined) { //cause it could be zero, lol
            if(this.selectedObjects.size) {
                action = TagAction.Add;
            } else {
                action = undefined; //treat this as Action.None
            }
        }


        //This only updates when it's different, thanks to the filter :3
        //And because of how subscription works, if there's no value there still
        // might be listeners, and if there's no listeners, we should still set the
        // value in case a parent tag is expanded to show it.
        // ~ elegance ~
        // and since it tells us if there is an actual triggerable change,
        // we can tell whether or not to continue processing up the parent chain.
        const difference = this.tagActionObservers.set(tagId, action);

        if (!difference) {
            return;
        }

        //shouldn't this be... the parents?
        //No, it's poorly named so no

        if (action == TagAction.Remove || action == TagAction.AddRemove) {
            this.parentsWhoseChildrenHaveSomethingToSay.addChild(this.db, tagId);
        } else {
            this.parentsWhoseChildrenHaveSomethingToSay.removeChild(this.db, tagId);
        }

    }

    private syncTagActionObserversToSelectedObjects() {
        for(const [tagId, selectedObjectsWithMe] of this.selectedObjectCountByTag) {
            this.syncTagActionObserver(tagId, selectedObjectsWithMe);
        }
    }

    private incrementSelectedObjectsByTag(tagId: TagId) {
        let current = this.selectedObjectCountByTag.get(tagId);

        if (!current) { //should never be zero at this point anyway, only undefined
            current = 0;
        }

        const newVal = current + 1;
        this.selectedObjectCountByTag.set(tagId, newVal);
        return newVal;
    }

    private decrementSelectedObjectsByTag(tagId: TagId) {
        const current = this.selectedObjectCountByTag.get(tagId);
        if (!current) {
            console.warn("Could not find a tag count for deselecting object, this is probably an error");
            return 0;
        }

        if (current < 0) {
            console.warn("Warning! Dependent tag selection went negative, this should never happen");
        }

        if(current == 0) {
            return 0;
        }

        const newVal: number = current - 1;

        this.selectedObjectCountByTag.set(tagId, newVal);
        return newVal;
    }


    private setObjectSelectedWithoutAlertingTagObservers(objId: ObjectId) {
        if(this.selectedObjects.has(objId)) {
            return false;
        }

        this.selectedObjects.add(objId);
        //And it's important that the selected or not signal be fired before
        // the tag action observers, because it overwrites it!
        this.objectsSelectedOrNotSignal.increment(); // this technically, potentially alerts taglistitems...
        this.exactSelectedObjectCounter.increment();
        this.objectSelectionObservers.set(objId, true);
        const tagList: Array<TagId> = this.db.getTagIdsOnObject(objId);

        for(const tagId of tagList) {
            this.incrementSelectedObjectsByTag(tagId);
        }

        return true;
    }

    private setObjectDeselectedWithoutAlertingTagObservers(objId: ObjectId) {
        if(!this.selectedObjects.has(objId)) {
            return false;
        }

        this.selectedObjects.delete(objId);
        //console.log("selected or not count: " + this.objectsSelectedOrNotSignal.get());
        this.objectsSelectedOrNotSignal.decrement();
        //console.log("selected or not count: " + this.objectsSelectedOrNotSignal.get());
        this.exactSelectedObjectCounter.decrement();
        this.objectSelectionObservers.set(objId, undefined); //undefined means it'll clear the entry when no observers :3
        const tagList: Array<TagId> = this.db.getTagIdsOnObject(objId);

        for(const tagId of tagList) {
            this.decrementSelectedObjectsByTag(tagId);
        }

        return true;
    }

    private clearObjectSelectionWithoutAlertingTagObservers() {
        for(const objId of this.selectedObjects) {
            this.setObjectDeselectedWithoutAlertingTagObservers(objId);
        }
    }

    setSingleObjectSelectionWithoutAlertingTagObservers(objId: ObjectId, selected: boolean) {
        if(selected) {
            return this.setObjectSelectedWithoutAlertingTagObservers(objId);
        } else {
            return this.setObjectDeselectedWithoutAlertingTagObservers(objId);
        }
    }


    setSingleObjectSelection(objId: ObjectId, selected: boolean) {
        let didChange = this.setSingleObjectSelectionWithoutAlertingTagObservers(objId, selected);

        if(didChange) {
            this.syncTagActionObserversToSelectedObjects();
        }
    }

    toggleObjectSelection(objId: ObjectId) {
        if(this.selectedObjects.has(objId)) {
            return this.setSingleObjectSelection(objId, false);
        } else {
            return this.setSingleObjectSelection(objId, true);
        }
    }


    handoffSingleObjectSelection(obj: IndexedObject) {
        //the idea here is that we don't want to call allllll of the "anySelectedCounter"
        // observers, which could be... a lot. To change the add context to a non-modifying context in the
        // tag bar. But that would happen if we called clear selection and then set selection, which
        // is a common enough task when selecting.


        //First, bump the selection count to make sure we don't get to zero and trigger a 
        // refresh of the non-actioning tags
        this.setObjectSelectedWithoutAlertingTagObservers(obj.id);

        //Deselect all others :3
        for(const selObjId of this.selectedObjects) {
            if(selObjId == obj.id) {
                continue;
            }

            this.setObjectDeselectedWithoutAlertingTagObservers(selObjId);
        }

        this.lastSingleSelectIndex = obj.index;

        this.syncTagActionObserversToSelectedObjects();
    }

    clearObjectSelection() {
        if(this.currentOpenObject.get()) {
            return;
        }

        this.clearObjectSelectionWithoutAlertingTagObservers();
        this.syncTagActionObserversToSelectedObjects();
    }

    selectAllObjects() {
        if(this.currentOpenObject.get()) {
            return;
        }

        const queue = this.objectQueue.get();
        if(!queue) {
            return;
        }

        for(const objId of queue) {
            this.setSingleObjectSelectionWithoutAlertingTagObservers(objId.id, true);
        }

        this.syncTagActionObserversToSelectedObjects();
    }

    invertObjectSelection() {
        if(this.currentOpenObject.get()) {
            return;
        }

        const queue = this.objectQueue.get();
        if(!queue) {
            return;
        }

        //This extra complexity so that we can push all the selections first,
        // then get to the deselections, to not trigger anyObjectSelected to update all tags
        // only to re-engage them.
        const toSelect = new Array<ObjectId>();
        const toDeselect = new Array<ObjectId>();

        for(const obj of queue) {
            if(this.selectedObjects.has(obj.id)) {
                toDeselect.push(obj.id);
            } else {
                toSelect.push(obj.id);
            }
        }

        for(const objId of toSelect) {
            this.setSingleObjectSelectionWithoutAlertingTagObservers(objId, true);
        }

        for(const objId of toDeselect) {
            this.setSingleObjectSelectionWithoutAlertingTagObservers(objId, false);
        }

        this.syncTagActionObserversToSelectedObjects();
    }

    private publishObjectQueue() {
        //I'm thinking, an observable map that maps to a set that is a *copy* of the queue?
        //That way we preserve a working copy that won't mess up the UI copy...

        const result = new Array<IndexedObject>();

        let index = 0;
        for(const objId of this.objectQueueWorkingCopy) {
            result.push({id: objId, index: index++});
        }

        this.objectQueue.set(result);
        this.status.set(ContextStatus.Ready);
    }

    getSelectedTags() {
        return new Set<TagId>([...this.selectedTags]);
    }

    toggleTagSelection(tagId: TagId) {
        this.setTagSelection(tagId, !this.tagSelectionSentinel.get(tagId));
    }

    setSingleTagSelection(tagId: TagId) {
        if(this.db.checkLimitTagChildrenToObjectSelection(tagId)) {
            //yeah we don'tn't
            return;
        }

        for(const selTag of this.selectedTags) {
            if (selTag == tagId)
                continue;

            this.setTagSelection(selTag, false);
        }

        this.setTagSelection(tagId, true);
    }

    setTagSelection(tagId: TagId, selected: boolean) {
        if(this.db.checkLimitTagChildrenToObjectSelection(tagId)) {
            //yeah we don'tn't
            return;
        }

        if(this.searchQueryString.get() && selected) {
            //we only do this if we're trying to select a tag, otherwise it could be the
            // search query function trying to clear out the selection
            this.searchQueryString.set(undefined);
        }
        
        if(!selected && !this.selectedTags.has(tagId)) {
            return;
        }


        const currentSelection = this.tagSelectionSentinel.get(tagId);
        if(currentSelection == selected) {
            console.log("Skipping cause selection already matches");
            return;
        }


        // I think we should always clear it, probably. It's non-obvious how to handle object selection
        // in a multi-tag selection-changing scenario. Preserve the selection while hidden? Unselect those but
        // if they show back up they're not selected anymore? Honestly just not having a selection I think
        // is more intuitive. (cause it also eliminates the anxiety of "if I tag these selected, what about the others
        // left selected?" problem).
        this.closeViewer();



        if(selected) {
            this.selectedTags.add(tagId);
            this.tagSelectionSentinel.set(tagId, true);
            //console.log("Add child of something to say green ed: " + tagId);
            //console.log("(parent): " + this.db.getTagParent(tagId));
            this.parentsWhoseChildrenHaveSomethingToSayGreenEdition.addChild(this.db, tagId);
        } else {
            this.selectedTags.delete(tagId);
            this.tagSelectionSentinel.set(tagId, undefined); //undefined so it is removed if no observers and no value
            //console.log("REMOVE child of something to say green ed: " + tagId);
            //console.log("(parent): " + this.db.getTagParent(tagId));

            this.parentsWhoseChildrenHaveSomethingToSayGreenEdition.removeChild(this.db, tagId);
        }


        //Now we just work with selectedTags to populate the queue :)
        this.tagSelectionChangeWatcher.trigger();
        this.reloadObjectQueueForTagSelection();
    }

    setSearchQuery(query: string | undefined) {
        if(!query) {
            this.searchQueryString.set(undefined);
            this.reloadObjectQueueForTagSelection();
            return;
        }


        for(const selectedTag of this.selectedTags) {
            this.setTagSelection(selectedTag, false);
        }

        //This happens after so we don't trigger the warning from reloadObjectQueueForTagSelection
        this.status.set(ContextStatus.Searching);
        this.searchQueryString.set(query);

        const matchedTagGroups = this.db.keywordSeachTags(query);

        this.closeViewer();
        this.clearObjectSelection();

        const result = new Set<ObjectId>();
        //const relevantTags = new Set<TagId>();

        //don't worry about the comlexity of this function just don't worry about it
        // IT DOESN'T MATTER DON'T WORRY ABOUT IT
        for (const [, tagSet] of matchedTagGroups) {
            for (const tagId of tagSet) {
                //NON INCLUSIVE ver
                const objsForTag = this.db.getObjectIdsWithTag(tagId);

                for (const objId of objsForTag) {
                    result.add(objId);
                }
            }
        }


        for (const objId of result) {
            const myTags = new Set<TagId>(this.db.getTagIdsOnObject(objId));

            //you have to have one tag in each group.
            for (const [, matchedTags] of matchedTagGroups) {
                let matchedOne = false;
                for (const aTagOfMine of myTags) {
                    if (matchedTags.has(aTagOfMine)) {
                        matchedOne = true;
                        break;
                    }
                }

                if (!matchedOne) {
                    result.delete(objId);
                    break;
                }
            }
        }

        //this.tagsToShowWhileFiltered.set(relevantTags);
        this.objectQueueWorkingCopy = result;
        this.publishObjectQueue();
    }

    reloadObjectQueueForTagSelection() {
        if (this.searchQueryString.get()) {
            //let setSearchQuery manage it
            console.warn("Reload object queue called in query mode");
            return;
        }

        const resultQueue = new Set<ObjectId>();
        let initialized = false;

        //In theory, it would be faster to only consider the tags that have changed... But is that worth it?
        //Actually might be simpler too, because then it's only two intersecting sets at a time...

        for(const tagId of this.selectedTags) {
            const myObjects = this.db.getObjectsWithTagInclusive(tagId);

            //Initialize result queue with first tag's objects.
            //Once initialized, then intersect it with all the other tag's objects

            if(!initialized) {
                for(const objId of myObjects) {
                    resultQueue.add(objId);
                }
            } else {
                for(const objId of resultQueue) {
                    if(!myObjects.has(objId)) {
                        resultQueue.delete(objId);
                    }
                }
            }

            if(!resultQueue.size) {
                break;
            }

            initialized = true;
        }

        if (!this.currentOpenObject.get()) {
            let needSync = false;

            for (const objId of this.selectedObjects) {
                if (!resultQueue.has(objId)) {
                    if (this.setObjectDeselectedWithoutAlertingTagObservers(objId)) {
                        needSync = true;
                    }
                }
            }

            if (needSync) {
                this.syncTagActionObserversToSelectedObjects();
            }
        }


        this.objectQueueWorkingCopy = resultQueue;
        this.publishObjectQueue();

        if(this.selectedTags.size == 1) {
            for(const tagId of this.selectedTags) {
                const tagParent = this.db.getTagParent(tagId);
                if (this.db.checkLimitTagChildrenToObjectSelection(tagParent)) {
                    this.selectAllObjects();
                    break;
                }
            }
        }
    }

    async createNewTagAndAddSelected(parent: TagId = reserved.tagIds.root, repositionAfter: TagId | undefined = undefined) {
        const selectedObjects = [...this.selectedObjects];
        const tagId = this.db.createTagWithoutFlush(reserved.tagNames.newTag, parent);

        if(repositionAfter) {
            this.db.repositionTagWithoutFlush(tagId, repositionAfter, Position.After);
        }

        //This is a performance optimization. Otherwise it does a lot of unnecessary shit and hangs the app
        // if it's in the 10s of thousands...
        this.clearObjectSelection();

        //this one flushes but honestly it's making me feel like maybe flushing should be
        // something the callee determines is appropriate... idk...
        //Or maybe this function itself should just be in the tag database...
        //Problem is, it's multi-step and involves us... So callbacks would be necessary... ugh
        await this.db.addTagToObjects(tagId, selectedObjects);

        //do this AFTER so that if it's tonnes, we don't keep recompiling the object queue...
        this.setSingleTagSelection(tagId);



        return tagId;
    }

    async deleteTag(tagId: TagId) {
        const selectedObjects = [...this.selectedObjects];
        this.setTagSelection(tagId, false); //to avoid it updating the queue a ton
        this.clearObjectSelection();
        await this.db.removeTagFromObjects(tagId, selectedObjects);
        await this.db.deleteTag(tagId); // < this flushes
    }


    async handleUpload(fileTree: FileTree, controller: UploadControllerStator) {
        if (this.selectedTags.size > 1) {
            console.log("Upload with multiple tags selected is a no-op");
            //Cause otherwise how do we build folder trees in? Both tags? First tag? etc...

            controller.errorOut("Can't upload to multiple tags at once, sorry :<");
            return;
        }

        //we DON'T WANT to upload to these tags...
        for(const selTag of this.selectedTags) {
            const parent = this.db.getTagParent(selTag);
            if(this.db.checkLimitTagChildrenToObjectSelection(parent)) {
                this.setTagSelection(selTag, false);
            }
        }

        //We only want to create a new base tag if we got files in the base selection,
        // if we're just uploading a folder or folders, the folder names should be the
        // tag names
        if (this.selectedTags.size == 0 && fileTree.files.length) {
            console.log("No tags selected creating");

            const tagId = await this.db.createTagEndpoint(reserved.tagNames.newTag, reserved.tagIds.root);
            this.setSingleTagSelection(tagId);
        }

        let baseTag: TagId | undefined;

        if (this.selectedTags.size > 1) {
            //in case it has changed
            throw new Error("Too many tags selected");
        }

        for (const s of this.selectedTags) {
            baseTag = s;
        }


        //This opens the dialog window
        controller.setCount(fileTree.howManyFileUgot());


        const recursivelyAddNtagDirectory = async (tree: FileTree, controller: UploadControllerStator, baseTag: TagId | undefined) => {
            for (const f of tree.files) {
                controller.nextFileName(f.name);

                try {
                    if (!baseTag)
                        throw new Error("No tag selected in try block");

                    await this.db.uploadFile(f, [baseTag]);

                    controller.succ();

                } catch {
                    controller.fail("An error occurred");
                }

                if (controller.checkIsCancelled()) {
                    console.log("Controller cancelled");
                    break;
                }
            }

            for (const [name, child] of tree.children) {
                const parent = baseTag ? baseTag : reserved.tagIds.root;
                const childTag: TagId = await this.db.createTagEndpoint(name, parent);
                if(!this.selectedTags.size) {
                    this.setSingleTagSelection(childTag);
                }
                await recursivelyAddNtagDirectory(child, controller, childTag);
            }

        }

        await recursivelyAddNtagDirectory(fileTree, controller, baseTag);

        controller.finish();
    }

    async exportSelected() {
        //ALRIGHT todo is to modify the upload controller to handle this as well..
        // and honestly it could be cleaner, it needs to double check the file extension based on
        // mime, etc... but I might just... skip this for now...
        const selectedActual = this.selectedObjects;

        if(!selectedActual) {
            return;
        }

        const uploadables = [...selectedActual];

        const destList: string[] | undefined = await openSelectDirectory();

        if (!destList) return;

        if(destList.length != 1) {
            console.log("too many directories, can only export to one");
            return;
        }

        const destDir = destList[0];


        const takenFilenames = new Set<string>();

        for(const obj of uploadables) {
            const srcPathPair = (await this.db.getObjectPath(obj)); 

            if(!srcPathPair) {
                continue;
            }

            const srcPath = srcPathPair.path;

            const baseFilename = this.db.pickFilename(obj);


            let enumerator = 0;
            let filename = baseFilename;

            while(true) {
                ++enumerator;

                if(takenFilenames.has(filename)) {
                    filename = `[${enumerator}]${baseFilename}`
                    continue;
                }

                const destPath = await getMetaApi().joinPath(destDir, filename);
                const existsTho = await getMetaApi().checkPathExists(destPath);

                //either it already exists, or it's about to
                takenFilenames.add(filename);

                if(existsTho) {
                    continue;
                }

                await getMetaApi().copyPath(srcPath, destPath);
                break;
            }
        }
    }

    getNoisyChildrenOfParentsWhoAretired(parent: TagId) {
        return this.parentsWhoseChildrenHaveSomethingToSay.getChildren(parent);
    }

    getParentsWhoseChildrenHaveSomethingToSayCounter() {
        return this.parentsWhoseChildrenHaveSomethingToSay.counter;
    }

    getParentsWhoseChildrenHaveSomethingToSayAlwaysCounter() {
        return this.parentsWhoseChildrenHaveSomethingToSay.alwaysCounter;
    }

    getParentsWhoseChildrenHaveSomethingToSayGreenEditionCounter() {
        return this.parentsWhoseChildrenHaveSomethingToSayGreenEdition.counter;
    }
}

/*
const mapStoreInternal = new Map<string, BinderContext>();

function getBinderContext(binderPath: BinderConnectionId): BinderContext {
    let result = mapStoreInternal.get(binderPath);

    if(!result) {
        result = new BinderContext(binderPath);
        mapStoreInternal.set(binderPath, result);
    }

    return result;
}

function destroyBinderContext(nodeId: string) {
    mapStoreInternal.delete(nodeId);
}
*/

export { BinderContext, ContextStatus }