import React, { useLayoutEffect, useState} from 'react'
import './contextmenu.css'
import { useCloseableOverlay } from './useCloseableOverlay';

import { StatorObjectBase, StatorContext } from '../statorobject';

interface ContextMenuItem
{
    title: string;
    keyBinding: string;
    action: () => void;
}

interface ContextMenuSection
{
    contextMenuItems: Array<ContextMenuItem>;
}

export class ContextMenuData
{
    spawnX: number = 0;
    spawnY: number = 0;

    contextMenuSections = new Array<ContextMenuSection>();
}


class ContextMenuStator extends StatorObjectBase
{
    show = false;

    private myContextMenuData = new ContextMenuData();

    spawn(data: ContextMenuData)
    {
        //spawnContextMenu(data);
        //return;
        this.myContextMenuData = data;
        this.show = true;
        this.publishChanges();
    }

    close()
    {
        this.show = false;
        this.publishChanges();
    }

    getData()
    {
        return this.myContextMenuData
    }
}

const ContextMenuStatorKey = "ContextMenuStator";

function getContextMenuStator(context: StatorContext)
{
    return context.getStatorLegacy(ContextMenuStatorKey, new ContextMenuStator(context)) as ContextMenuStator;
}


interface ContextMenuProps
{
    appContext: StatorContext;
}

interface Dimens
{
    width: number;
    height: number;
}

function ContextMenu(props: ContextMenuProps)
{
    const myStator = props.appContext.useWatchStatorLegacy(ContextMenuStatorKey, new ContextMenuStator(props.appContext));
    const myData = myStator.getData();

    //Make sure to display none until we get a ref to set the location correctly
    const [myDimens, setMyDimens] = useState<Dimens>({width: 0, height: 0});

    useLayoutEffect(() =>
    {
        if(!myRef.current)
        {
            return;
        }

        const width = myRef.current.getBoundingClientRect().width;
        const height = myRef.current.getBoundingClientRect().height;


        setMyDimens({
            width: width,
            height: height
        });
    }, [myData.spawnX, myData.spawnY, myData.contextMenuSections]);

    const closeHandler = () =>
    {
        myStator.close();
    }

    const myRef = useCloseableOverlay(closeHandler);

    if(!myStator.show)
        return null;

    //This gets a warning because of keys in a list but they're not possible to rearrange...
    //It's bitching because of the <hr> tags...
    //But no option to supress that warning... Thanks react
    let idIndex = 0;
    const content = myData.contextMenuSections.map((section: ContextMenuSection, index: number) =>
    {
        const result = section.contextMenuItems.map((item: ContextMenuItem) =>
        {
            const decoratedAction = () =>
            {
                closeHandler();
                item.action();
            }
            return (
                <button className="menu-item"
                    onClick={decoratedAction}
                    key={idIndex++}
                >
                    {item.title}
                </button>
            );
        });

        const sectionBreak = <hr key={idIndex++}/>;

        if(index != 0)
            return [sectionBreak, ...result];

        return result;
    });

    //console.log("Spawn x, y: " + props.spawnX + ", " + props.spawnY);


    //Gotta do it this way cause the ref is invalid at first


    let myStyle: React.CSSProperties = {};
    const upperBoundX = myData.spawnX + myDimens.width;
    if (upperBoundX > window.innerWidth)
        myStyle.left = myData.spawnX - (upperBoundX - window.innerWidth);
    else
        myStyle.left = myData.spawnX;

    //if(!myData.spawnX)
    //    console.log("NOT!");

    const upperBoundY = myData.spawnY + myDimens.height;
    //console.log("Spawny: " + myData.spawnY);
    //console.log("Height: " + height);
    //console.log("Uppder bound y: " + upperBoundY + ", window.innerHeight: " + window.innerHeight);
    if (upperBoundY > window.innerHeight)
        myStyle.top = myData.spawnY - (upperBoundY - window.innerHeight);
    else
        myStyle.top = myData.spawnY;

    return (
            <div className={`hamburger-menu menu`}
            ref={myRef}
            style={myStyle}
            >
                {content}
            </div>
    );
}

export {ContextMenu, getContextMenuStator };