import React, {useState, useEffect} from 'react';
import {getMetaApi} from "../ipcbindings";
import './eula.css'

//The part before the dash is the license with it's version, after the dash
// is any revisions made to the changeover date, changeover license, etc.

interface EulaScreenProps {
    agreeCallback: (e: any) => void;
}

async function takeLicenseChecksum() {
    const lpath = await getMetaApi().getLicensePath();
    return await getMetaApi().takeChecksumOfFile(lpath, "blake2b");
}

async function readLicense(): Promise<string> {
    const lpath = await getMetaApi().getLicensePath();
    return (await getMetaApi().readUtf8File(lpath)).data;
}


interface FormatIntoParagraphsProps {
    text: string;
}

function FormatTextFileIntoParagraphsWithHeader(props: FormatIntoParagraphsProps) {
    return (<>
        {props.text.split('\n').map((paragraph: string, index: number) => {
            if(paragraph == "") {
                return;
                
            } else if (index == 0) {
                return <h1 key={index}>{paragraph}</h1>
            } else {
                return <p key={index}>{paragraph}</p>
            }
        })}
    </>)
}

function EulaScreen(props: EulaScreenProps) {
    const [loadingLicense, setLoadingLicense] = useState(true);
    const [licenseText, setLicenseText] = useState<string>("");

    useEffect(() => {
        (async () => {
            setLicenseText(await readLicense());
            setLoadingLicense(false);
        })();
    }, [])
    

    if(loadingLicense) {
        return (
            <div className="eula-screen">
                <div className="eulaText">
                    <p>Loading license please wait...</p>
                </div>
            </div>
        )

    } else {
        return (
            <div className="eula-screen">
                <div className="eulaText">
                    <FormatTextFileIntoParagraphsWithHeader text={licenseText}/>
                </div>
                <form onSubmit={props.agreeCallback} spellCheck={false}>
                    <input type="submit" className={"generic-overlay-form-button"} value={"Agree"} disabled={false} />
                </form>
            </div>
        )
    }
}

export {EulaScreen, takeLicenseChecksum, readLicense, FormatTextFileIntoParagraphsWithHeader}