import React, {useState} from 'react';
import { StatorContext } from '../statorobject';
import { getOverlayStator } from './overlays';
import { useCloseableOverlay } from './useCloseableOverlay';
import "./genericoverlay.css"

export interface GenericOverlayTab
{
    title: string;
    nestedComponent: any;
    nestedProps: any;
}

export interface GenericOverlayProps
{
    startTab: number;
    tabs: Array<GenericOverlayTab>;
    appContext: StatorContext;
}

interface TabButtonProps
{
    text: String;
    index: number;
    isCurrent: boolean;
    tabChanger: (i: number) => void;
}

function TabButton(props: TabButtonProps)
{
    return(
        <button className={`generic-overlay-tab-button ${props.isCurrent ? "generic-overlay-tab-button-current" : ""}`}
                onClick={() => { props.tabChanger(props.index) }}>
            {props.text}
        </button>
    )
}

function GenericOverlayWrapper(props: GenericOverlayProps)
{
    const [currentTab, setCurrentTab] = useState(props.startTab);

    const closeMeCallback = () =>
    {
        getOverlayStator(props.appContext).closeOverlays();
    };
    const myRef = useCloseableOverlay(closeMeCallback);

    const tabsJsx = (() =>
    {
        if(props.tabs.length == 1)
            return <></>;
        
        const tabs = props.tabs.map((t: GenericOverlayTab, index: number) =>
        {
            return ( <TabButton
                text={t.title}
                index={index}
                isCurrent={index == currentTab}
                tabChanger={setCurrentTab}

                key={t.title}
            />);
        });

        return <div className={"generic-overlay-tabbar"}>{tabs}</div>
    })();



    const Component = props.tabs[currentTab].nestedComponent;
    const ChildProps = props.tabs[currentTab].nestedProps;

    return (
        <div ref={myRef} className="center-overlay">
            <div className={"generic-overlay-container"}>
                {tabsJsx}
                <div className={"generic-overlay-content-container"}>
                    <div className={"generic-overlay-inner"}>
                        <Component {...ChildProps} />
                    </div>
                </div>
            </div>
        </div>
    );
}

export { GenericOverlayWrapper }