import React from 'react'
import { ContextMenuData, getContextMenuStator } from './contextmenu';
import "./hamburgermenu.css"
import { StatorContext } from '../statorobject';
import { getBinderListStator } from './openbinders';
import { getOverlayStator, OverlayStatorStates } from './overlays';
import {justDie} from '../ipcbindings';


interface HamburgerMenuProps
{
    buttonStyleClass: string;
    appContext: StatorContext;
}

function HamburgerMenu(props: HamburgerMenuProps)
{
    const overlayStator = getOverlayStator(props.appContext);
    const clickHandler = (e: React.MouseEvent) =>
    {
        const menuProps: ContextMenuData =
        {
            spawnX: 28,
            spawnY: 16,
            contextMenuSections:
                [
                    {
                        contextMenuItems:
                            [
                                {
                                    title: "Sync Selected Binder",
                                    keyBinding: "F6",
                                    action: () => {
                                        const binder = getBinderListStator(props.appContext).getCurrentBinder();
                                        binder?.context.db.syncCluster();
                                    }
                                },
                            ]
                    },
                    /*
                    {
                        contextMenuItems:
                            [
                                {
                                    title: "Undo",
                                    keyBinding: "Ctrl-Z",
                                    action: () => { }
                                },
                                {
                                    title: "Redo",
                                    keyBinding: "Ctrl-Y",
                                    action: () => { }
                                },
                            ]
                    },
                    */
                    {
                        contextMenuItems:
                        [
                            {
                                title: "Select All",
                                keyBinding: "Ctrl-A",
                                action: () =>
                                { 
                                    getBinderListStator(props.appContext).getCurrentBinder()?.context.selectAllObjects();
                                }
                            },
                            {
                                title: "Invert Selection",
                                keyBinding: "Ctrl-Alt-A",
                                action: () => 
                                { 
                                    getBinderListStator(props.appContext).getCurrentBinder()?.context.invertObjectSelection();
                                }
                            },
                        ]
                    },
                    {
                        contextMenuItems:
                            [

                                {
                                    title: "Export Selected Objects",
                                    keyBinding: "Ctrl-S",
                                    action: () =>
                                    {

                                        getBinderListStator(props.appContext).getCurrentBinder()?.context.exportSelected();
                                        //getMapper(getBinderListStator(props.appContext).getSelectedBinder())?.exportSelection();
                                    }
                                }
                            ]
                    },
                    {
                        contextMenuItems:
                            [
                                {
                                    title: "Activation", 
                                    keyBinding: "",
                                    action: () => { getOverlayStator(props.appContext).set(OverlayStatorStates.showActivation); } // :)
                                },
                                {
                                    title: "Updates",
                                    keyBinding: "",
                                    action: () => { getOverlayStator(props.appContext).set(OverlayStatorStates.showUpdater); }
                                }
                            ]
                    },
                    {
                        contextMenuItems:
                            [
                                {
                                    title: "About",
                                    keyBinding: "",
                                    action: () => { overlayStator.set(OverlayStatorStates.showAbout) }
                                },
                            ]
                    },
                    {
                        contextMenuItems:
                            [
                                {
                                    title: "Quit",
                                    keyBinding: "",
                                    action: () => { justDie() }
                                },
                            ]
                    },
                ],
        }

        getContextMenuStator(props.appContext).spawn(menuProps);
    };

    return (
            <img
            className={`hamburber ${props.buttonStyleClass}`}
                onClick={clickHandler}
                src={"static/kebaab.svg"}
                draggable={false}
                />
    );
}

export { HamburgerMenu };