import React, { useState, useEffect } from "react";
import { ActivationStatus, getActivationStator} from '../activation';
import { StatorContext, StatorObjectBase} from "../statorobject";
import { getBinderVersion, getUpdaterStator } from "../updaterstator";
import './nagscreen.css';
import { getOverlayStator, OverlayStatorStates } from "./overlays";
import { getMetaApi } from "../ipcbindings";

/*
class Periodic
{
  interval: number = 0;
  actuator = (): boolean => { return false; }; //returns true on continue
}
*/

class NagStator extends StatorObjectBase
{
    private neverNag = true;
    private timerId: number = 0;
    private interval = (1000 * 60 * 45); //Every forty five minutes

    private showNaggy = false;

    constructor(context: StatorContext)
    {
        super(context);

        //init :3
        this.triggerNag();
    }

    private stopTimer()
    {
        window.clearTimeout(this.timerId);
    }

    private startTheClock()
    {
        this.timerId = window.setTimeout(() => { this.triggerNag(); }, this.interval)
    }

    private pumpActivation()
    {
        this.neverNag = !getActivationStator(this.getContext()).shouldNag();
    }

    forceShow()
    {
        this.showNaggy = true;
        this.publishChanges();
    }

    private triggerNag()
    {
        this.pumpActivation();
        //If the activation page is up, don't nag, that's heckin' annoying. Just restart the clock

        //It's okay if it pops up some variable time after it's closed, it doesn't matter, I just don't want it
        // to pop up while someone is trying to enter their key

        //Now it's just any overlay, cause... Idk... It's annoying. What are we nagging over, a help screen? About page?
        // meh.

        //EHHHH I mean... The only reason not to annoy is to annoy someone that already purchased, otherwise it's like... par for the course

        ///ehhhhh it's annoying turn it off if an overlay is open
        if(this.neverNag || getOverlayStator(this.getContext()).myShowState != OverlayStatorStates.showNone)
        {
            this.startTheClock();
            return;
        }

        this.forceShow();
    }

    shutUp()
    {
        this.showNaggy = false;
        //Check activation status now tho?
        //Nah, it's checked in triggerNag, and we wait to check periodically even if activated.
        this.startTheClock();
        this.publishChanges();
    }

    shouldShowNaggy(): boolean
    {
        return this.showNaggy;
    }
};

const NagStatorKey = "NagStator";

function useWatchNagStator(appContext: StatorContext)
{
    return appContext.useWatchStatorLegacy(NagStatorKey, new NagStator(appContext)) as NagStator;
}

function getNagStator(appContext: StatorContext)
{
    return appContext.getStatorLegacy(NagStatorKey, new NagStator(appContext)) as NagStator;
}


interface NagScreenProps
{
  appContext: StatorContext;
};

function NagScreen(props: NagScreenProps)
{
    const [pricingUrl, setPricingUrl] = useState("");
    const stator = useWatchNagStator(props.appContext);

    useEffect(() => {
        (async () => {
            const baseUrl = await getMetaApi().getServerEndpoint();
            const binderVer = await getBinderVersion();
            const endpoint = baseUrl + "/api/binder/v1/pricing/" + binderVer;
            setPricingUrl(endpoint);
        })();
    }, []);

    const openActivationPage = () =>
    {
        stator.shutUp();
        getOverlayStator(props.appContext).set(OverlayStatorStates.showActivation);
    };

    if(!stator.shouldShowNaggy())
        return <></>;

    const appContext = props.appContext;

    const CloseButton = (props: { val: string }) =>
    {
        return (
        <a href="#" onClick={() =>
        {
            stator.shutUp();

            //Check for updates cause otherwise we blasted past them if we're unlicensed.
            getUpdaterStator(appContext).checkForUpdates();
        }}>
            {props.val}
        </a>);
    }

    //IF tamper trip... Just go straight to activation page?
    let tamperTrip = false;

    const licenseStatus = (() =>
    {
        const myActivation = getActivationStator(props.appContext);

        if(!myActivation)
        {
            return (<p><CloseButton val="close"/></p>);
        }

        switch (myActivation.status)
        {
            case ActivationStatus.Unlicensed:
                return (<p>This copy of Binder is unlicensed and for evaluation purposes only, please purchase a license if you are done evaluating or <CloseButton val="continue your evaluation." /></p>);
            case ActivationStatus.Subscription:
            case ActivationStatus.Perpetual:
                return <p>This copy of Binder is already licensed, <CloseButton val={"close this."} /></p>
            case ActivationStatus.TamperTrip:
                tamperTrip = true;
                return <p>Activation tamper detection system was tripped and you need to re-activate</p>
            case ActivationStatus.Pending:
                //This could happen if the thing is opened from the activation page, otherwise it should never
                // automatically nag if pending (don't annoy someone unless you're ready for their attention)
                return <p>Activation is pending, <CloseButton val={"click here to dismiss this dialog."} /></p>
            case ActivationStatus.Refunded:
                return <p>Your subscription was refunded and you will need a new license to continue using Binder.</p>;
            case ActivationStatus.ChargedBacked:
                return <p>A charge back was issued against your subscription and you will need a new license to continue using Binder.</p>;
            case ActivationStatus.ActivationError:
                return (<p>There was an error activating, please try again later. <CloseButton val={"Close"}/></p>)

            case ActivationStatus.InvalidKey:
                return (<p>
                    Your key is invalid. <a href="#" onClick={openActivationPage}>Resolve.</a>
                </p>);
            case ActivationStatus.ExpiredSub:
                return <p>Your subscription has expired.</p>;

            default:
                return (<CloseButton val={"Close"} />);
        }
    })();


    //Tamper trip status, unlicensed status, activated and no further action needed (cause we can get to this page
    // from the activation page which is always available)


    //Todo: onError for the iframe, the case being it's not online, just... idk render it without it?

    return (
        <div className={"nag-screen-background"}
            draggable={false}>
            <div className={"sales-thing"}>
                {licenseStatus}

                <iframe className={"tier-frame"} src={pricingUrl}></iframe>
                {
                    tamperTrip ?
                        <div className="sales-thing-footer">
                            <h2><a href="#" onClick={openActivationPage}>Go to activation page to sort this out</a></h2>
                        </div>
                        :
                        (
                            <div className="sales-thing-footer">
                                <p>and/or</p>
                                <h2><a href="#" onClick={openActivationPage}>Enter License Key</a></h2>
                            </div>
                        )
                }

            </div>
    </div>);
}

export {NagScreen, getNagStator}