import React from 'react'
import { StatorContext, StatorObjectBase } from '../statorobject';
import './navbar.css'
import { ViewerState, useWatchObjectViewerOpenStator, useWatchObjectViewerStator } from './objectviewercontroller';
import { useWatchBinderListStator } from './openbinders';
import { justDie } from '../ipcbindings';


class NavBarStator extends StatorObjectBase
{

}

const NavBarStatorKey = "NavBarStator";

function useWatchNavBarStator(appContext: StatorContext)
{
    return appContext.useWatchStatorLegacy(NavBarStatorKey, new NavBarStator(appContext)) as NavBarStator;
}

function getNavBarStator(appContext: StatorContext)
{
    return appContext.getStatorLegacy(NavBarStatorKey, new NavBarStator(appContext)) as NavBarStator;
}

interface ObjectViewerControlsProps
{
    appContext: StatorContext;
}


function AudioVideoControls(props: ObjectViewerControlsProps)
{
    const controller = useWatchObjectViewerStator(props.appContext);

    if (controller.checkPlayingAudioVideo())
        return <NavBarButton src={"static/pause-button.svg"} clickHandler={() => controller.pauseAudioVideo()}/>
    else
        return <NavBarButton src={"static/play-button.svg"} clickHandler={() => controller.playAudioVideo()}/>
}

function ObjectViewerControls(props: ObjectViewerControlsProps)
{
    //The beautiful thing about the usewatch functions, is that
    // they gracefully handle the context object being swapped out.
    const controllerStator = useWatchObjectViewerStator(props.appContext);
    const singleObjectViewOpenStator = useWatchObjectViewerOpenStator(props.appContext);

    if(!singleObjectViewOpenStator.viewerOpen)
    {
        console.log("Skip render object viewer controls");
        return null;
    }
    console.log("Render object viewer controls");


    const msg = controllerStator.theViewerState == ViewerState.loadingWhilePlayingSlideshow ? "Loading next..." : "";


    const slideshowToggle = (() =>
    {
        switch(controllerStator.theViewerState)
        {
            case ViewerState.loadingWhilePlayingSlideshow:
            case ViewerState.playingSlideshow:
                return <NavBarButton src={"static/stop-slideshow-button.svg"}
                                     clickHandler={ () => controllerStator.stopSlideshow() }/>
            default:
                return <NavBarButton src={"static/play-slideshow-button.svg"}
                                     clickHandler={ () => {controllerStator.startOrContinueSlideshow() }}/>
        }
    })();

    return (<div>
        <NavBarButton src={"static/cancel-button.svg"} clickHandler={() =>
            {
                //Kinda jank cause this should be an action basically that other areas can call..
                singleObjectViewOpenStator.close();
                controllerStator.stopSlideshow();
                //REALLY JANK CAUSE THIS SHOULD BE AN ACTION BASICALLY THAT OTHER AREAS CAN CALL..
                controllerStator.pauseAudioVideo();
            }}/>
        {slideshowToggle}
        {controllerStator.checkShowAudioVideoControls() ? <AudioVideoControls {...props}/> : ""}
    </div>);
}

interface NavBarButtonProps
{
    src: string;
    clickHandler: () => void;
}

function NavBarButton(props: NavBarButtonProps)
{
    return <img className="forward-button nav-button"
        src={props.src}
        onClick={(e: any) => { props.clickHandler() }}
        draggable={false}
    />
}

interface NavBarProps
{
    appContext: StatorContext;
}

function NavBar(props: NavBarProps)
{
    const stator = useWatchNavBarStator(props.appContext);
    const binderListStator = useWatchBinderListStator(props.appContext);

    console.log("Re-rendering navbar");
    let progressBar = <div></div>;

    /*
    if(stator.showProgressBar)
    {
        //Multiply all the percentages together
        let percentage = 1;
        let message = "";
        for(const progBar of stator.progressBars)
        {
            percentage = percentage * progBar.getPercentage();
            if (message != "")
                message = message + " and " + progBar.getMessage();
            else
                message = progBar.getMessage();
        }

        const coolKidsPercentage = 100 * percentage;
        const progressBarStyle = { width: coolKidsPercentage + "%" }


        progressBar = (
            <div>
                <div className="nav-bar-progress-background"></div>
                <div className="nav-bar-progress-message"> {message} </div>
                <div style={progressBarStyle} className='nav-bar-progress'></div>
            </div>
        );
    }
     */

    //TODO
    const openBinderContext = undefined;// binderListStator.getCurrentBinderContext();

    if(!openBinderContext)
        console.warn("Bad binder context");

    return (<div className="nav-bar">
        <div className="nav-left-section">
            {openBinderContext ? <ObjectViewerControls appContext={openBinderContext}/>: null}
            {progressBar}
        </div>
        <div className="nav-right-section">

            {/*
                        <img className="back-button nav-button"
                         src={"static/left-arrow.svg"}
                        onClick={prevHandler}/>

                        <img className="forward-button nav-button"
                         src={"static/right-arrow.svg"}
                        onClick={nextHandler}/>

                        */
            }
            <div className={'nav-bar-drag-area'} />
            <NavBarButton src={"static/close-button.svg"}
                clickHandler={() => { justDie() }} />
        </div>
    </div>);
}

export {NavBar, getNavBarStator}