import React, {useState} from 'react';
import { Binder } from '../backend'
import { getOverlayStator } from './overlays';
import './newbinderoverlay.css'
import './centeroverlay.css'
import './vibrantactionbutton.css'
import { StatorContext } from '../statorobject';
import { GenericOverlayWrapper } from './genericoverlay';
import {openSelectDirectory} from "../ipcbindings";
import { localNodeConfig } from '../localnodeconfig';


enum Operation //And we can do shit like this cause it's not C++, lol
{
    NewBinder,
    ConnectToExisting,
};


interface NewBinderOverlayProps
{
    appContext: StatorContext;
}

function CreateNewBinderContent(props: NewBinderOverlayProps)
{
    const [binderNameField, setBinderNameField] = useState<string>("");
    const [binderDirectoryField, setBinderDirectoryField] = useState<string>("");
    const [errString, setErrString]  = useState<string>("");

    const binderNameChange = (e: any) =>
    {
        setBinderNameField(e.target.value);
    };

    const binderDirectoryChange = (e: any) =>
    {
        setBinderDirectoryField(e.target.value);
    }

    const handleSubmit = async (e: any) => {
        e.preventDefault();

        try {
            const msg = await Binder.createBinder(binderDirectoryField, binderNameField);
            setErrString(msg);

            if (!msg) {
                getOverlayStator(props.appContext).closeOverlays();
            }
        } catch(e) {
            setErrString("There was an error");
            console.warn(e);
        }



        /*
        createBinderEndpoint(binderNameField,
            binderDirectoryField).then( (binder: Binder) =>
            {
                getBinderListStator(props.appContext).addBinder(binder);
            }).catch((msg: string) =>
            {
                setErrString(msg);
                console.log(msg);
            })
            */
    }

    const choose = (e: any) =>
    {
        openSelectDirectory().then((result: string[] | undefined) =>
        {
            if(!result || !result[0])
                return;

            setBinderDirectoryField(result[0]);
        });
    };


    let errorContent = <div></div>;

    if(errString)
    {
        errorContent = <div className="new-binder-overlay-error">{errString}</div>
    }

    return <div>
        <form  onSubmit={handleSubmit} spellCheck={false}>
            {errorContent}
                <p>What would you like to call your new binder? (Folder Name)</p>

                <input
                    type="text"
                    name="binderName"
                    value={binderNameField}
                    onChange={binderNameChange}
                    autoComplete="off"/>



            <p> Where do you want it? (Can be local, Dropbox, OneDrive, etc)</p>

            <div className="generic-horizontal-input-cont">
                <input
                    type="text"
                    name="binderDirectory"
                    value={binderDirectoryField}
                    onChange={binderDirectoryChange}
                    autoComplete="off"/>

                <input className="generic-horizontal-input-button" type="button"
                       value={"Choose"} onClick={choose}/>
            </div>



            <input type="submit" className="generic-overlay-form-button" value={"Create"}/>
        </form>
    </div>;
}

function ConnectToExistingContent(props: NewBinderOverlayProps)
{
    const [binderPath, setBinderPath] = useState<string>("");
    const [errString, setErrString]  = useState<string>("");

    const binderPathChange = (e: any) =>
    {
        setBinderPath(e.target.value);
    };


    const handleSubmit = async (e: any) => {
        e.preventDefault();

        try {
            const msg = await localNodeConfig.registerCluster(binderPath);
            setErrString(msg);

            if (!msg) {
                getOverlayStator(props.appContext).closeOverlays();
            }
        } catch(e) {
            setErrString("There was an error");
            console.warn(e);
        }

        /*
        connectToExistingEndpoint(binderPath).then((binder: Binder) =>
        {
            getBinderListStator(props.appContext).addBinder(binder);
            getMapper(binder.id).refreshAndSync();
        }).catch((msg: string) =>
        {
            setErrString(msg);
            console.log(msg);
        });
        */
    }

    const choose = (e: any) =>
    {
        openSelectDirectory().then((result: string[] | undefined) =>
        {
            if(!result || !result[0])
                return;

            setBinderPath(result[0]);
        });
    };

    let errorContent = <div></div>;

    if(errString != "")
    {
        errorContent = <div className="new-binder-overlay-error">{errString}</div>
    }

    return <div>
        <form onSubmit={handleSubmit} spellCheck={false}>
            {errorContent}
                <p>Where is the binder?</p>

            <div className="generic-horizontal-input-cont">
                <input
                    type="text"
                    name="binderName"
                    value={binderPath}
                    onChange={binderPathChange}
                    autoComplete="off"/>

                <input className="generic-horizontal-input-button" type="button"
                       value={"Choose"} onClick={choose}/>

            </div>


            <input type="submit" className={"generic-overlay-form-button"} value={"Connect"}/>

        </form>
    </div>;

}

function NewBinderOverlay(props: NewBinderOverlayProps): JSX.Element
{
    const [currentOperation, setCurrentOperation] = useState<Operation>(Operation.NewBinder);

    return (<GenericOverlayWrapper
        appContext={props.appContext}
        startTab={0} /*Start us out at new binder*/
        tabs={[{
            title: "New",
            nestedComponent: CreateNewBinderContent,
            nestedProps: props,
        },
        {
            title: "Existing",
            nestedComponent: ConnectToExistingContent,
            nestedProps: props,
        }
        ]}
    />);
}


export { NewBinderOverlay }