import React, {useEffect, useState, useRef} from 'react';
import { ObjectThumbnail} from './thumbnails/objectthumbnail';
import { BinderConnectionId, IndexedObject} from '../backend';
import './objectgallery.css';
import { StatorContext } from '../statorobject';
//import {  getNavBarStator } from './navbar'
import {  getUploadControllerStator } from "./uploadcontroller";
import { getBinderListStator } from './openbinders';
import { fileTreeFromDataTransferList } from '../filetree';
import { useWatchObservableValue } from '../observermanager';
import { ContextStatus } from './bindercontext';

const overRenderFactor = 3;
const rerenderThresholdFactor = 1;

interface ObjectGalleryProps {
  theBinderId: BinderConnectionId;
  appContext: StatorContext;
  tagSelectionCount: number;
  viewAreaHeight: number;
  nominalOffset: number;
}

function PageMessage(props: {message: string}) {
    return (<div className="page-message">{props.message}</div>);
}


function ObjectGallery(props: ObjectGalleryProps): JSX.Element {
  const myRef = useRef<HTMLDivElement>(null)

  //WHEN YOU'RE DONE REWRITING DON'T FORGET TO DO THE THING!
  // AND SET THE ROW WIDTH!!
  const [loadState, setLoadState] = useState<number>(ContextStatus.LoadingTags);
  const [objectList, setObjectList] = useState<Array<IndexedObject>>(new Array<IndexedObject>());
  const [viewWidth, setViewWidth] = useState<number>(0);

  const context = getBinderListStator(props.appContext).getBinder(props.theBinderId).context;

  useEffect(() => {
    const queueChangeObsId = context.objectQueue.register(() => {
      const objList = context.objectQueue.get();
      setObjectList(objList ? objList : new Array<IndexedObject>());
    });

    return function () {
      context.objectQueue.unregister(queueChangeObsId);
    };
  }, []);

  useWatchObservableValue(context.status, (stat: number | undefined) => {
    if(!stat) {
      setLoadState(0);
    } else {
      setLoadState(stat);
    }
  });


  useEffect(() => {
    if (!myRef?.current) {
      return;
    }

    const resizeObserver = new ResizeObserver((e: Array<ResizeObserverEntry>) => {
      if (myRef.current) {
        if(context.currentOpenObject.get()) {
          return;
        }

        setViewWidth(myRef.current.offsetWidth);

        //props.doRender isn't updated yet when we need this

        /*
    const rowCount = style.gridTemplateColumns.split(" ").length;

    context.setRowWidth(rowCount);
    */
      }
    });

    resizeObserver.observe(myRef.current);

    return function () {
      //capture a damn copy of the lil fucker
      const currentRef = myRef.current;
      if (!currentRef) {
        console.warn("could not unobserve resize, this is okay if this was a live reload");
        return;
      }

      resizeObserver.unobserve(currentRef);
    }
  }, [myRef]);


  const myDragOverHandler = (e: React.DragEvent<HTMLDivElement>) => {
    e.preventDefault();
  }


  /*
    const myDragLeaveHandler = (e: React.DragEvent<HTMLDivElement>) =>
    {
      //console.log("leave");
      if (!myRef)
        return;

      const current = myRef.current;

      if (!current)
        throw "dfjd";

      const bounds = current.getBoundingClientRect();

      bounds.x += 10;
      bounds.width -= 20;
      bounds.y += 10;
      bounds.height -= 20;

      console.log("page x: " + e.pageX);
      console.log("bounds x + w: " + (bounds.x + bounds.width));
        
        if(e.pageX < bounds.x || e.pageX >= bounds.x + bounds.width ||
            e.pageY < bounds.y || e.pageY >= bounds.y + bounds.height)
        {
            e.preventDefault();
            setInDragOver(false);
        }
    }
    */

  const myDropHandler = async (e: React.DragEvent<HTMLDivElement>) => {
    console.log("Drop");
    if (!e) return;
    if (!e.dataTransfer) return;
    if (!e.dataTransfer.types.includes("Files")) return;
    e.preventDefault();

    //Kind of a crappy way to do this but w/e I'm lazy
    // really need to add the binder context itself to the appcontext...
    const binder = getBinderListStator(props.appContext).getCurrentBinder();

    if (!binder) {
      console.warn("Somehow the binder isn't ready for the drop");
      return;
    }

    const myUploadController = getUploadControllerStator(props.appContext);
    myUploadController.setPreparing();
    const tree = await fileTreeFromDataTransferList(e.dataTransfer.items);
    await binder.context.handleUpload(tree, myUploadController);
  }


  //const style = props.doRender ? {} : {display: "none"};
  let style: React.CSSProperties | undefined;

  let content;
  let objectJSX = new Array<any>();

  if (loadState == ContextStatus.LoadingTagSelection) {
      content = (
        <PageMessage message="Loading..." />
      );
  }
  else if(loadState == ContextStatus.Searching) {
      content = (
        <PageMessage message="Searching..." />
      );
  }
  else if(loadState != ContextStatus.Ready) {
      content = (
        <PageMessage message="..." />
      );
  }
  else if (props.tagSelectionCount == 0 && !context.searchQueryString.get()) {
    content = (
      <PageMessage message="No tag selected" />
    );
  }
  else if (((objectList.length == 0))) {
    if (props.tagSelectionCount == 1) {
      content = (
        <PageMessage message="Drop files here to copy them to this tag" />
      );
    }
    else {
      let msg;
      if(!context.searchQueryString.get()) {
        msg = "No objects have all selected tags";
      } else {
        msg = "Found nothing matching your query";
      }
      content = (
        <PageMessage message={msg}/>
      );
    }
  }
  else {


    const overRenderSize = props.viewAreaHeight * overRenderFactor;
    const overRenderPreceding = overRenderSize / 2;

    const startBound = (() => {
      let result = props.nominalOffset - overRenderPreceding;
      return result < 0 ? 0 : result;
    })();

    //Outerbound is non-inclusive so add extra... Or not it really doesn't matter...
    const outerBound = props.nominalOffset + props.viewAreaHeight + overRenderPreceding;

    const itemBorderTotal = 6 * 2;
    const itemSize = 150;
    const gridSpacing = 24;
    const itemMinSize = itemSize + itemBorderTotal;
    const rowHeight = itemSize + itemBorderTotal + gridSpacing;

    const colCount = Math.trunc((viewWidth - 24) / rowHeight);
    const countHeight = objectList.length ? Math.ceil(objectList.length / colCount) * rowHeight : 0;
    context.setRowWidth(colCount);

    const leftPad = ((viewWidth - (colCount * itemMinSize)) / (colCount + 1));

    //hmmm... lol this ain't it
    let rowIndex = 0;
    for(let y = 0; y < outerBound; y += rowHeight) {
      //console.log("first loop");
      if (y >= startBound) {
        let nextX = leftPad;
        for (let colI = 0; colI != colCount; ++colI) {
          const objIndex = rowIndex + colI;
          const obj = objectList.at(objIndex);

          if(!obj) {
        //    console.log("skipping obj");
            break;
          }

         // console.log("drawing object at: " + x + ", " + y);

          objectJSX.push((<ObjectThumbnail key={obj.id}
            obj={obj}
            binderContext={context}
            appContext={props.appContext}
            theBinderId={props.theBinderId}
            x={nextX}
            y={y + gridSpacing/2} />));

          nextX = nextX + itemMinSize + leftPad;
        }

      }
      rowIndex += colCount;
    }

    style = {height: countHeight, display: "visible"};
  }



  return (
    <div className='object-gallery-container'
      onDragOver={myDragOverHandler}
      onDrop={myDropHandler}
    >
      <div className="object-gallery"
        ref={myRef}
        style={style}
      >
        {objectJSX}
      </div>

      {content}
    </div>);
}


export {
  ObjectGallery,
  overRenderFactor,
  rerenderThresholdFactor
}