import React, {useEffect, useState} from 'react';
import {StatorContext} from "../statorobject";
import {useWatchSidebarStator} from "./sidebarstator";
import {getBinderListStator} from "./openbinders";
import {BinderContext} from "./bindercontext";
import "./objectpropertiessidebar.css"
import {getMetaApi} from "../ipcbindings";

interface ObjectPropertiesSidebarProps {
    appContext: StatorContext;
    hide: boolean;
    objId: string;
    binderContext: BinderContext;
    imageResolution: string | undefined;
}

function ObjectPropertiesSidebar(props: ObjectPropertiesSidebarProps) {
    const [sizeInBytes, setSizeInBytes] = useState<number>();
    const [mimeString, setMimeString] = useState("...");
    const [mainMimeType, setMainMimeType] = useState("");

    useEffect(() => {
        if(!props.objId) return;

        (async () => {
            const path = await props.binderContext.db.getObjectPath(props.objId);
            if(path == "") return;

            const stats = await getMetaApi().readFileStats(path.path);

            if(!stats) return;

            setSizeInBytes(Number(stats.size));

            const mime = await props.binderContext.db.getMimeGenIfNotExists(props.objId);

            setMimeString(mime);
            const splits = mime.split("/");

            if (splits.length > 1) {
                setMainMimeType(splits[0]);
            }
        })();
    }, [props.objId]);

    const sizeString = (() => {
        const sizeInKilobytes = sizeInBytes / 1024;
        const sizeInMegabytes = sizeInKilobytes / 1024;
        const sizeInGigabytes = sizeInMegabytes / 1024;

        if(sizeInKilobytes < 1)
            return sizeInBytes.toFixed(2) + " bytes";
        if(sizeInMegabytes < 1)
            return sizeInKilobytes.toFixed(2) + " kilobytes";
        if(sizeInGigabytes < 1)
            return sizeInMegabytes.toFixed(2) + " megabytes";

        return String(sizeInGigabytes) + " gigabytes";
    })();


    if(props.hide) return <></>
    else return <div className={"object-properties-sidebar"}>
        <h1>Properties</h1>
        <p>File Size: {sizeString}</p>
        <p>Type: {mimeString}</p>
        {mainMimeType == "image" ? <p>Resolution: {props.imageResolution}</p> : ""}
    </div>
}


export {ObjectPropertiesSidebar}