import React, {useState, useEffect, useRef} from 'react'
import { BinderConnectionId, getObjectUri} from "../backend";
import { StatorContext } from '../statorobject';
import "./objectviewer.css"
import { getObjectViewerStator } from './objectviewercontroller';
import { getBinderListStator } from './openbinders';
import {ObjectPropertiesSidebar} from "./objectpropertiessidebar";
import {getSidebarStator, useWatchSidebarStator} from "./sidebarstator";

export interface ObjectViewerProps {
    theBinderId: BinderConnectionId;
    appContext: StatorContext;
}

function VideoPlayerHead(props: ObjectViewerProps)
{
    /* cause it uses b2bsum...
    const myRef = useRef<HTMLVideoElement>(null);
    const stator = getObjectViewerStator(props.appContext);

    //Seek should be a function here, that, after calling seek on the video, passing the current time
    // back to the stator
    useEffect(() =>
    {
        if (myRef && myRef.current)
        {
            stator.setAudioVideo(() => myRef.current?.play(),
                () => myRef.current?.pause(),
                (i: number) => myRef.current?.fastSeek(i),
                myRef.current.duration)
        }
    },[myRef, myRef.current, props.obj.b2bsum]);


    return <video
    ref={myRef}
    className={"object-viewer-content-area"}
    src={getObjectUri(props.obj)}
    />//is it really that simple.
    */
}

interface ViewerImplProps extends ObjectViewerProps {
    objId: string;
    resolutionPropogator: (res: string) => void;
}


function ImageViewer(props: ViewerImplProps) {
    const imageRef = useRef<HTMLImageElement>(null);
    const context = getBinderListStator(props.appContext).getBinder(props.theBinderId).context;
    const [imageUrl, setImageUrl] = useState<string>();

    useEffect(() => {
        (async () => {
            const optionalImageUrl = await context.db.getObjectPath(props.objId);

            if (!optionalImageUrl) {
                return;
            }

            setImageUrl(optionalImageUrl.path);
        })();
    });

    const handleLoad = (e: React.SyntheticEvent) => {
        /*
        if (controller.theViewerState == ViewerState.loadingWhilePlayingSlideshow)
        {
            controller.startOrContinueSlideshow();
        }
        */

        const width = imageRef.current.naturalWidth
        const height = imageRef.current.naturalHeight;

        props.resolutionPropogator("" + width + "x" + height + "px");
    }

    const wrappedURI = imageUrl ? "file:///" + imageUrl : undefined;

    return (
        <img
            ref={imageRef}
        className="object-viewer-content-area"
            src={wrappedURI}
            onLoad={handleLoad}
            draggable={false}
        />
    )
}

function PDFViewer(props: ViewerImplProps) {
    const uri = getObjectUri(props.objId);

    return <iframe id="pdf-js-viewer" className="object-viewer-pdf-frame" src={uri}></iframe>;
}

function PlainTextViewer(props: ViewerImplProps)
{
    //literally copy and pasted this from objecthumbnail... It's basically the
    // same component with different container rendering
  const [text, setText] = useState<string>("");


  const textUri = getObjectUri(props.objId);

  useEffect(() =>
  {
    const request = new Request(textUri);

    fetch(request).then((resp: Response) =>
    {
      resp.text().then((value: string) =>
      {
        setText(value);
      });
    });
  }, [textUri]);

  return (
  <div className="object-viewer-plaintext-background">
          <pre className="object-viewer-plaintext">{text}</pre>
  </div>
 )
}

function ObjectViewer(props: ObjectViewerProps)
{
    const myRef = useRef<HTMLDivElement>(null);
    //const myRef = useRef(null);
    const stator = getObjectViewerStator(props.appContext);
    const context = getBinderListStator(props.appContext).getBinder(props.theBinderId).context;
    const [mime, setMime] = useState<string>();
    const [imageResolution, setImageResolution] = useState<string>("");
    const [objId, setObjId] = useState<string>();
    const [doRender, setDoRender] = useState<boolean>(false);
    //const objectOpenStator = getObjectViewerOpenStator(props.appContext);
    const sidebarStator = useWatchSidebarStator(props.appContext);


    useEffect(() => {
        setImageResolution("");//reset on object change
        if (mime) {
            return;
        }


        (async () => {
            if (!objId) {
                return;
            }

            if (myRef.current && doRender && myRef.current != document.activeElement) {
                //console.log("focusing object viewer");
                myRef.current.focus();
            }

            const queriedMime = await context.db.getMimeGenIfNotExists(objId);
            if (!queriedMime) {
                return;
            }

            setMime(queriedMime);
        })();
    }, [objId]);



    useEffect(() => {
        const currentOpenObjectCont = context.currentOpenObject;

        const sync = () => {
            const newObjId = currentOpenObjectCont.get();

            //clear mime cause it's outdated now, other effect hook
            // will trigger on objId change to update it
            setMime("");

            if (!newObjId) {
                setDoRender(false);
                setObjId("");
            } else {
                setDoRender(true);
                setObjId(newObjId.id);
            }
        }

        const obsId = currentOpenObjectCont.register(sync);

        return () => {
            currentOpenObjectCont.unregister(obsId);
        };
    }, [myRef.current]);


    /*
    //Focus on render so that keyboard events are intuitive
    useEffect(() =>
    {
        if(myRef == null)
            return

        if(myRef.current == null)
            return;

        //typescript going crazy on this w/e
        myRef.current.focus();
    });

     */

    const clickHandler = (e: React.MouseEvent) => {
        e.stopPropagation(); //Otherwise the tagpage whitespace handler is called and we get deselected but still visible
        if(e.detail == 2) {
            context.closeViewer();
        }
    }

    //these really need to be unified... with the tagpage one... It's basically the same function
    function downHandler(e: React.KeyboardEvent<HTMLDivElement>) {
        //console.log("Now my turn!");
        if(!doRender) //invisible so skip
            return;


        if(!e.repeat) //No repeats...
        {
            if(e.key == "ArrowLeft")
            {
                e.preventDefault();
                e.stopPropagation();
                context.selectPrevious();
            }
            if(e.key == "ArrowRight")
            {
                e.preventDefault();
                e.stopPropagation();
                context.selectNext();
            }
            if(e.key == "ArrowUp")
            {
                e.preventDefault();
                e.stopPropagation();
                context.selectPreviousRow();
            }
            if(e.key == "ArrowDown")
            {
                e.preventDefault();
                e.stopPropagation();
                context.selectNextRow();
            }
            if(e.key == " ")//Spacebar because whoever did this think's they're real clever
            {
                e.preventDefault()
                e.stopPropagation();

                if(!e.ctrlKey)
                    context.selectNext();
                else
                    context.selectPrevious();
            }
            if(e.key == "Enter")
            {
                e.preventDefault();
                e.stopPropagation();
                context.closeViewer();
            }
            if(e.key == "Escape")
            {
                e.preventDefault();
                e.stopPropagation();
                context.closeViewer();
            }
        }
    }





    //const checkboardBack = mime == "image/svg+xml";
    const checkboardBack = false;

    const style = doRender ? { } : {display: "none"};

    let myJsx = <></>;


    if (mime && objId) //split up the thing
    {
        const splits = mime.split("/");

        if (splits.length > 1) {
            const mainMimeType = splits[0];
            const secondMimeType = splits[1];


            if (mainMimeType == "image") {
                myJsx = <ImageViewer {...props} objId={objId} resolutionPropogator={setImageResolution}/>
            } else if (mainMimeType == "application") {
                let val = secondMimeType;
                if (secondMimeType == "octet-stream") {
                    val = "unknown";
                }
                myJsx = <p>{val}</p>
            } else {
                myJsx = <p>{mime}</p>
            }
        }
    }

    const backClickHandler = (e: React.MouseEvent) =>
    {
        context.selectPrevious();
        e.stopPropagation();
    };

    const forwardClickHandler = (e: React.MouseEvent) =>
    {
        context.selectNext();
        e.stopPropagation();
    };

    const propertiesToggle = (e: React.MouseEvent) => {
        e.stopPropagation();
        sidebarStator.toggleObjectProperties();
    }


    return (
        <div className={"object-viewer-and-properties-container"}
             style={style}
        >
            <div className={`object-viewer ${checkboardBack ? 'object-viewer-checkerboard' : ''}`}
                 onClick={clickHandler}
                 onKeyDown={downHandler}
                 tabIndex={1}
                 ref={myRef}
            >
                <div className={"object-viewer-container"}>
                    {myJsx}
                </div>
                <img className={'object-viewer-side-control-overlay ' +
                    'object-viewer-controls-left-right-base-style ' +
                    'object-viewer-side-control-overlay-left'}
                     src={'static/rewind-button.svg'}
                     draggable={false}
                     onClick={backClickHandler}
                />
                <img className={'object-viewer-side-control-overlay ' +
                    'object-viewer-controls-left-right-base-style ' +
                    'object-viewer-side-control-overlay-right'}
                     src={'static/fast-forward-button.svg'}
                     draggable={false}
                     onClick={forwardClickHandler}
                />


                <img className={'object-viewer-side-control-overlay object-viewer-side-control-overlay-top-right'}
                     src={'static/kebaab.svg'}
                     draggable={false}
                     onClick={propertiesToggle}
                />

            </div>
            <ObjectPropertiesSidebar appContext={props.appContext}
                                     hide={sidebarStator.objectPropertiesHidden}
                                     objId={objId}
                                     imageResolution={imageResolution}
                                     binderContext={context}/>
        </div>
    )
        ;
}

export {ObjectViewer}

