import { StatorContext, StatorObjectBase } from "../statorobject";


//The idea is that the playbar thing resets the animation whenever
// it renders, and doesn't start until... Uh... mental model fuzzy

export enum ViewerState
{
    chill,
    playingSlideshow,
    playingAudioVideo,
    loadingWhilePlayingSlideshow,
}

export enum ObjectViewerContext
{
    image,
    audiovideo,
    pdf,
    text,
}

export class ObjectViewerStator extends StatorObjectBase
{
    private viewerContext = ObjectViewerContext.image;

    theViewerState: ViewerState = ViewerState.chill;
    slideshowInterval = 3000; //3 seconds is gud :3
    private timerId: any = 0;
    loadPreviousImageCallback = () => { console.warn("Previous image callback not set!!"); };
    loadNextImageCallback = () => { console.warn("Next image callback not setup!!!"); };

    private pauseHook = () => {};
    private playHook = () => { console.warn("Play hook not set") };
    private seekHook = (i: number) => {};
    private audioVideoLength = 0;

    checkShowAudioVideoControls()
    {
        if (this.viewerContext != ObjectViewerContext.audiovideo)
            return false;
        if (this.theViewerState == ViewerState.loadingWhilePlayingSlideshow || this.theViewerState == ViewerState.playingSlideshow)
            return false;

        return true;
    }

    checkPlayingAudioVideo()
    {
        return this.theViewerState == ViewerState.playingAudioVideo;
    }

    setAudioVideo(play: () => void, pause: () => void, seek: (i: number) => void, length: number)
    {
        this.playHook = play;
        this.pauseHook = pause;
        this.seekHook = seek;
        this.audioVideoLength = length;
        this.viewerContext = ObjectViewerContext.audiovideo;
        this.publishChanges();
    }

    private cancelTimer()
    {
        clearTimeout(this.timerId);
    }

    setViewerContext(c: ObjectViewerContext)
    {
        if(this.viewerContext == c)
            return;

        this.viewerContext = c;
        this.publishChanges();
    }

    playAudioVideo()
    {
        this.cancelTimer();
        this.theViewerState = ViewerState.playingAudioVideo;
        this.playHook();
        this.publishChanges();
    }

    pauseAudioVideo()
    {
        this.theViewerState = ViewerState.chill;
        this.pauseHook();
        this.publishChanges();
    }


    rewind()
    {
        this.pauseAudioVideo();
        this.loadPreviousImageCallback();
    }

    fastForward()
    {
        this.pauseAudioVideo();
        this.loadNextImageCallback();
    }

    setSlideshowInterval(i: number)
    {
        this.cancelTimer();
        this.slideshowInterval = i;
        this.publishChanges();
    }

    startOrContinueSlideshow()
    {
        if (this.theViewerState == ViewerState.playingAudioVideo)
            this.pauseAudioVideo();

        const callbacksies = () =>
        {
            console.log("next image plox, waiting for continuation from viewer");
            this.theViewerState = ViewerState.loadingWhilePlayingSlideshow;
            this.loadNextImageCallback();
            this.publishChanges();
        };

        this.timerId = setTimeout(callbacksies, this.slideshowInterval);
        this.theViewerState = ViewerState.playingSlideshow;
        this.publishChanges();
    }

    stopSlideshow()
    {
        this.cancelTimer();
        this.theViewerState = ViewerState.chill;
        this.publishChanges();
    }
}

const ObjectViewerStatorKey = "ObjectViewerStator";


function useWatchObjectViewerStator(appContext: StatorContext)
{
    return appContext.useWatchStatorLegacy(ObjectViewerStatorKey, new ObjectViewerStator(appContext)) as ObjectViewerStator;
}

function getObjectViewerStator(appContext: StatorContext)
{
    return appContext.getStatorLegacy(ObjectViewerStatorKey, new ObjectViewerStator(appContext)) as ObjectViewerStator;
}

export class ObjectViewerOpenStator extends StatorObjectBase
{
    //openBinderObject: BinderObject | undefined;
    viewerOpen = false;

    close()
    {
        if(!this.viewerOpen)
            return;

        this.viewerOpen = false;
        this.publishChanges();
    }

    /*
    open(b: BinderObject)
    {
        if(this.openBinderObject == b && this.viewerOpen)
            return;
        
        this.openBinderObject = b;
        this.viewerOpen = true;
        this.publishChanges();
    }
    */

    open()
    {
        if(this.viewerOpen)
        {
            console.warn("Viewer already open");
            return;
        }
        
        this.viewerOpen = true;
        this.publishChanges();
    }
};

const ObjectViewerOpenStatorKey = "ObjectViewerOpenStator";

function useWatchObjectViewerOpenStator(appContext: StatorContext)
{
    return appContext.useWatchStatorLegacy(ObjectViewerOpenStatorKey, new ObjectViewerOpenStator(appContext)) as ObjectViewerOpenStator;
}

function getObjectViewerOpenStator(appContext: StatorContext)
{
    return appContext.getStatorLegacy(ObjectViewerOpenStatorKey, new ObjectViewerOpenStator(appContext)) as ObjectViewerOpenStator;
}

export {useWatchObjectViewerStator,
    getObjectViewerStator,
    useWatchObjectViewerOpenStator,
    getObjectViewerOpenStator,
}