import React from 'react'
import { StatorObjectBase, StatorContext } from '../statorobject'
import { BinderConnectionId, Binder } from '../backend';
import { TagPage } from "./tagPage";
import { getOverlayStator, OverlayStatorStates } from './overlays';
import { BinderContext } from './bindercontext';
import { localNodeConfig } from '../localnodeconfig';


class BinderContextPair {
  binder: Binder;
  narrowedContext: StatorContext;

  constructor(b: Binder, cont: StatorContext)
  {
    this.binder = b;
    this.narrowedContext = cont;
  }
}


//This is hardly incremental, but it wasn't before either, so fuck it
class BinderListStator extends StatorObjectBase {
  private openBinderList = new Array<BinderContextPair>();
  private selectedBinderId: BinderConnectionId = "";
  private initialized = false;

  constructor(appContext: StatorContext) {
    super(appContext);

    const initialBinderList = localNodeConfig.connectedBinders.get();

    
    ///*
    (async () => {
      if (initialBinderList) {
        await this.syncToBinderList(initialBinderList);
        if (initialBinderList.length)
          this.setSelectedBinder(initialBinderList[0]);
      }
    })();
    //*/


    localNodeConfig.connectedBinders.register(async () => {
      const updatedBinderIdList = localNodeConfig.connectedBinders.get();
      await this.syncToBinderList(updatedBinderIdList);

      if(!this.initialized && updatedBinderIdList) {
        if (updatedBinderIdList.length) {
          this.setSelectedBinder(updatedBinderIdList[0]);
          this.initialized = true;
        }
      }
    });
  }

  /*
  flushBinders() {
    for(const pair of this.openBinderList) {
      pair.binder.context.db.flushIfReady();
    }
  }
  */

  private findPairById(binderId: BinderConnectionId): BinderContextPair | undefined
  {
    /*
    for(const pair of this.openBinderList) {
      if(pair.binder.id == binderId) {
        return pair;
      }
    }
    return undefined;
    */
    return this.openBinderList.find((val: BinderContextPair) => { return val.binder.id == binderId; });
  }

  getBinder(binderId: BinderConnectionId): Binder {
    const pair = this.findPairById(binderId);

    if(!pair)
      throw new Error("Can't get binder that doesn't exist");

    return pair.binder;
  }

  checkSelected(bId: BinderConnectionId) {
    return bId == this.selectedBinderId;
  }

  getCurrentBinder(): Binder | undefined {
    return this.getBinder(this.selectedBinderId);
  }

  private async addBinderNonPublishing(bId: BinderConnectionId) {
   // console.log("Loading binder");
    const b = await localNodeConfig.loadBinder(bId);

    this.openBinderList.push(new BinderContextPair(b, this.getContext()));
    this.selectedBinderId = bId;
  }


  setSelectedBinder(bId: BinderConnectionId) {
    const b = this.findPairById(bId);

    if(!b)
    {
      console.warn("Could not find binder by id!!");
      return;
    }

    this.selectedBinderId = bId;
    this.publishChanges();
  }

  async syncToBinderList(binders: Array<BinderConnectionId> | undefined) {
    if(!binders) {
      return;
    }

    //console.log("Binder list to sync to: " + binders.length);

    const currentIdSet = new Set<BinderConnectionId>();
    let changed = false;
    for(const bId of binders) {
      currentIdSet.add(bId);

      if (this.findPairById(bId)) {
        //console.log("Skipping");
        continue;
      }


      await this.addBinderNonPublishing(bId);
      changed = true;
    }

    this.openBinderList.forEach((pair: BinderContextPair, index: number) => {
      if(!currentIdSet.has(pair.binder.id)) {
        this.openBinderList.splice(index, 1);
        changed = true;


        if(this.openBinderList.length && this.selectedBinderId == pair.binder.id) {
          //select adjacent, preferably earlier index so we're stable
          let newIndex = index;

          if(newIndex + 1 > this.openBinderList.length) {
            newIndex = this.openBinderList.length - 1;
          }

          this.setSelectedBinder(this.openBinderList[newIndex].binder.id);
        }

      }
    });

    if(changed) {
      this.publishChanges();
    }
  }

  /*
  removeBinderById(id: BinderConnectionId)
  {
    //This is contrived cause I want to get the binder next to the
    // one that was closed

    let newOpenBindersList = new Array<BinderContextPair>();

    let skippedOldBinder = false;
    let replacedCurrent = false;
    let lastBinderIdAdded: BinderConnectionId = "";

    for (const oldB of this.openBinderList)
    {
      if (oldB.binder.id != id)
      {
        newOpenBindersList.push(oldB);
        lastBinderIdAdded = oldB.binder.id;
      }
      else
      {
        skippedOldBinder = true;
      }

      if (skippedOldBinder && !replacedCurrent && lastBinderIdAdded)
      {
        this.selectedBinderId =  lastBinderIdAdded;
        replacedCurrent = true;
      }
    }

    this.openBinderList = newOpenBindersList;
    this.publishChanges();
  }
  */

  getOpenBinderList() {
    return [...this.openBinderList];
  }
}

const binderListStatorKey = "BinderListStator";

function useWatchBinderListStator(appContext: StatorContext)
{
  return appContext.useWatchStator(BinderListStator);
 // return appContext.useWatchStatorLegacy(binderListStatorKey, new BinderListStator(appContext)) as BinderListStator;

}

function getBinderListStator(appContext: StatorContext)
{
  return appContext.getStator(BinderListStator);
  //return appContext.getStatorLegacy(binderListStatorKey, new BinderListStator(appContext)) as BinderListStator;
}

interface NoBinderAction
{
  actionTitle: string;
  action: () => void;
}

interface NoBinderProps
{
  doRender: boolean;
  message: string;
  actions: Array<NoBinderAction>;
}

function BinderErrorPage(props: NoBinderProps)
{
  const style = props.doRender ? {} : {display: "none"};

  const actionJsx = props.actions.map((val: NoBinderAction) =>
  {
    return <button key={val.actionTitle} onClick={val.action}>{val.actionTitle}</button>
  });

  return (<div style={style} className="open-binder-area no-binders-container">
    <div className="no-binders-container-message">
      {props.message}
      {actionJsx}
    </div>
    </div>)
}



interface OpenBindersProps {
  appContext: StatorContext;
}

function OpenBinders(props: OpenBindersProps): JSX.Element | null
{
    const binderListStator = useWatchBinderListStator(props.appContext);

    function refresh()
    {
        console.log("reimpliment meee")
    }

    const result = binderListStator.getOpenBinderList().map((pair: BinderContextPair) => {
        if (pair.binder.id == "")
            return "";


        if(!pair.binder.exists)
        {
          return (<BinderErrorPage key={pair.binder.id.toString()}
          doRender={binderListStator.checkSelected(pair.binder.id)}
          message={"Could not find this binder, has it moved?"}
          actions={[
            {
              actionTitle: "Update Location",
              action: () => {console.log("oi! update the location yourself asshole"); refresh()}
            },
            {
              actionTitle: "Disconnect",
              action: () => {
                localNodeConfig.removecluster(pair.binder.id);
              }
            },
            {
              actionTitle: "Refresh",
              action: () => {refresh()}
            },
            ]}/>
          );
        }
        

        return(<OpenBinder key={pair.binder.id}
                           myBinderId={pair.binder.id}
                           binderContext={pair.binder.context} //really wanna merge this into app context so bad...
                           binderSpecificAppContext={pair.narrowedContext}
        />);
    });

    /*
    const noBinderJsx = openBinderList.length != 0 ? '' : (
    <BinderErrorPage selected={true}
    message="Create or connect to a binder to get started"
          actions={[
            {
              actionTitle: "Add Binder",
              action: () => { overlayStator.set(OverlayStatorStates.showNewBinder) }
            },
            ]}/>)
            */
          
    if(result.length == 0)
    {
      const newBinderAction: NoBinderAction =
      {
        action: () => { getOverlayStator(props.appContext).set(OverlayStatorStates.showNewBinder) },
        actionTitle: "Create/Connect",
      };
      return <BinderErrorPage doRender={true} message={"No binders"} actions={[newBinderAction]}/>
    }

    return (<>{result}</>);
}

interface OpenBinderProps
{
  binderContext: BinderContext;
  myBinderId: BinderConnectionId;
  binderSpecificAppContext: StatorContext;
}

function OpenBinder(props: OpenBinderProps)
{
    const binderListStator = useWatchBinderListStator(props.binderSpecificAppContext);

    const selected = binderListStator.checkSelected(props.myBinderId);

    //console.log("Open binder re-render");


  return (
    <div className={`open-binder-area ${selected ? "" : "open-binder-area-hidden"}`}>
      <TagPage
        binderContext={props.binderContext}
        binderId={props.myBinderId}
        doRender={true}
        appContext={props.binderSpecificAppContext}
      />
    </div>
  );
}

//TODO: context pair is a leaky abstraction... TagSidebar needs it, just to read
// the binder list...
export { OpenBinders, getBinderListStator, useWatchBinderListStator, BinderContextPair }