import React, {useState, useEffect, useRef} from 'react'
import {BinderConnectionId, TagId, Position} from '../backend'
import './taglistitem.css'
import { getContextMenuStator, ContextMenuData } from './contextmenu';
import { StatorContext } from '../statorobject';
import { parseTagName } from '../tagnamefunction';
import { BinderContext } from './bindercontext';

enum TagAction
{
  None,
  Add,
  Remove,
  AddRemove,
};

class TagListItemUpdatePayload
{
  name: String = "";
  selected: Boolean = false;
  action = TagAction.None;
  hideFromFilter: Boolean = false;
  childHasSomethingToSay: Boolean = false;
  childIsSelected: Boolean = false;
}

export interface TagListItemProps 
{
  binderContext: BinderContext;
  theBinderId: BinderConnectionId;
  tagId: TagId;
  appContext: StatorContext;
}

interface TagMiniButtonProps
{
  symbol: String;
  doRender: Boolean;
  currentTag: Boolean;
  greenCauseChildSelection: Boolean;
  callback: (e: any) => void;
}

function TagMiniButton(props: TagMiniButtonProps)
{
  if(!props.doRender)
    return null;

  const imgSrc: string = (() =>
  {
    //literally just realized I don't have plus/minus symbols and if we're using
    //imgs we.. Need that... shit

    let result = "";

    if(props.symbol == '+')
      result = "static/plus-button";
    else if(props.symbol == '-')
      result = "static/minus-button";
    else if (props.symbol == "▸")
      result = "static/closed-tag-tree-arrow";
    else if(props.symbol == '⤥')
      result = "static/child-tag-attention-tag-tree-arrow";
    else if (props.symbol == '▾')
      result = "static/open-tag-tree-arrow";
    else if (props.symbol == "0") {
      result = "static/related-protected"
    }
    else
      throw "Could not load svg for symbol: " + props.symbol;

    if(props.symbol != "0") {
      if (props.currentTag || (props.greenCauseChildSelection && props.symbol == '⤥'))
        result += "-black";
    }

    return result + ".svg";
  })();

  const plusMinusStyling = (() => {

    if(props.currentTag)
      return "";

      //God damn this is crufty
    if(props.symbol == '+')
    {
      return "tag-list-mini-button-add";
    }
    else if(props.symbol == '⤥')
    {
      const result =  "tag-list-mini-button-remove";
      if(props.greenCauseChildSelection)
        return result + " vibrant-action-button";
      return result;
    }
    else if(props.symbol == '-')
    {
      return "tag-list-mini-button-remove";
    }
    else if(props.symbol == '0') {
      return "tag-list-mini-button-protecc";
    }
  })();

  return (
      <div>
        <img className={`tag-list-item-button
    tag-list-item-mini-button ${plusMinusStyling}
    ${props.currentTag ? "tag-list-item-mini-button-current-tag" : ""}`}
                onClick={props.callback}
                src={imgSrc}
                draggable={false}
                />
      </div>
  );
}

interface TagTreeListViewProps
{
  binderContext: BinderContext;
  parentTag: TagId;
  theBinderId: BinderConnectionId;
  appContext: StatorContext;
}

function InternalTagList(props: TagTreeListViewProps) {
  const [tagList, setTagList] = useState(new Array<TagId>());

  useEffect(() => {
    const updateTagList = () => {
      //Really there should be a "is reserved?" test and if it is just register with the appropriat ething...
      //really really it should be a flag to test if it's limited or not...
      if(!props.binderContext.db.checkLimitTagChildrenToObjectSelection(props.parentTag)) {
        setTagList(props.binderContext.db.getSortedTags(props.parentTag));
      } else { //reserved

        //limit because we don't have lazy loading (yet, TODO)
        const nList = new Array<TagId>();
        const limit = 500;
        let i = 0;
        for(const tagId of props.binderContext.getNoisyChildrenOfParentsWhoAretired(props.parentTag)) {
          if(i++ == limit) {
            break;
          }

          nList.push(tagId);
        }

        setTagList(nList);
      }
    };

    updateTagList(); //initialize it


    //This might cause a double update, unfortunate but it is what it is...
    //I can't think of a clean way of solving this problem that handles all the edge cases.
    //At the very least it won't trigger a second render, so... yay react?
    const sortTableObsId = props.binderContext.db.registerChildrenOrderChange(props.parentTag, () => {
      //console.log("Updating taglistinternal from sort table");
      updateTagList();
    });
    const childrenRelationShipObsId = props.binderContext.db.registerSpecificChildrenObserver(props.parentTag, () => {
      //console.log("Updating taglistinternal from relationshipchange");
      //const newList = props.binderContext.db.getSortedTags(props.parentTag);
      //console.log("List size: " + newList.length + ", vals: " + newList);
      updateTagList();
      //updateTagList();
    });

    const parentsSomethingSayDeps = props.binderContext.getParentsWhoseChildrenHaveSomethingToSayAlwaysCounter();
    const parentsCounterObsId = parentsSomethingSayDeps.register(props.parentTag, () => {
      updateTagList();
    });

    return (() => {
      props.binderContext.db.removeChildrenOrderChangeObserver(props.parentTag, sortTableObsId);
      props.binderContext.db.removeSpecificChildrenObserver(props.parentTag, childrenRelationShipObsId);
      parentsSomethingSayDeps.unregister(props.parentTag, parentsCounterObsId);
    });
  }, []);

  //Okay sooooooo tagList should be a useState, and a use effect needs to register an observer
  // or two on the TagDatabase for when the list changes :3

  return (
  <div>
    {tagList.map((childId: TagId ) => {
        return (
            <TagListItem key={childId}
                         tagId={childId}
                         theBinderId={props.theBinderId}
                         appContext={props.appContext}
                         binderContext={props.binderContext}
            />
        );
    })}
  </div>);
}

interface TagListProps {
  binderContext: BinderContext;
  theBinderId: BinderConnectionId;
  parentTag: TagId;
  appContext: StatorContext;
}

enum DropZoneState
{
  Above,
  Center,
  Below,
}

interface TagItemDragData
{
  dragType: string;
  tagId: TagId;
}

interface DragTargetData
{
  tagId: TagId;
  openWithChildren: boolean;
}

const TagDropTargetDataType = "drag-target-data";

enum DragTargetState
{
  None,
  Above,
  Center,
  Below,
}

const TagItemDragDataType = "tagitemdragdata"; //Lowercased! Cause it will be lowercased if we don't do it!

function TagList(props: TagListProps) {
  const [dragState, setDragState] = useState<DragTargetState>(DragTargetState.None)
  const [layoverOffset, setLayoverOffset] = useState<number>(0);
  const dropZoneContainerRef = useRef<HTMLDivElement>(null);

  const tagDb = props.binderContext.db;

  //ref={dropZoneContainerRef}
  //onDragOver={myDragOverHandler}
  //onDragEnter={myDragEnterHandler}
  //onDrop={myOnDropHandler}


  function myDragOverHandler(e: React.DragEvent)
  {
    if(!e.dataTransfer.types.includes(TagItemDragDataType))
      return;

    e.preventDefault();

    if(!dropZoneContainerRef.current)
      return;

    const ourYoffset = dropZoneContainerRef.current.getBoundingClientRect().y;
    const dragYoffset = e.pageY;

    const relativeOffset = dragYoffset - ourYoffset;

    //I feel like there's gotta be a cleaner way to express this but w/e
    // Bascially we want to round down to the nearest 32nd
    const layoverBaseOffset = Math.floor(relativeOffset / 32) * 32;


    //console.log("my layout base offset: " + layoverBaseOffset);

    //32 is the height of the item and is very carefully chosen because we can divide it evenly twice
    // at the least

    //Our regions are 8 + 16 + 8 (32)
    // because we want the lower region to collapse with the upper of the next region

    const ourModulus = relativeOffset % 32;
    //console.log("Modulous: " + ourModulus);
    if (ourModulus >= 24) //8 + 16 (extent of the upper and mid-region)
    {
      //console.log("below");
      setLayoverOffset(layoverBaseOffset + 16);
      setDragState(DragTargetState.Below);
    }
    else if (ourModulus >= 8) //8 is the extent of the upper region
    {
      //console.log("middle");
      setLayoverOffset(layoverBaseOffset); //is just fine as it is I think it's pretty just the way it is
      setDragState(DragTargetState.Center);
    }
    else if (ourModulus < 8)
    {
      setLayoverOffset(layoverBaseOffset - 16); 
      setDragState(DragTargetState.Above);
    }
  }

  function myDragEnterHandler(e: React.DragEvent)
  {
    e.preventDefault();
  }

  function myDragLeaveHandler(e: React.DragEvent)
  {
    e.preventDefault()
  }

  function myOnDropHandler(e: React.DragEvent)
  {
    if(!e.dataTransfer.types.includes(TagItemDragDataType)) 
      return;

    e.preventDefault();

    //Do this before trying anything funny cause otherwise we could be locked in a ghost
    // drag layout situation
    setDragState(DragTargetState.None);

    try
    {
      const data = JSON.parse(e.dataTransfer.getData(TagItemDragDataType)) as TagItemDragData;

      //console.log("Accepted!");


      for (const val of document.elementsFromPoint(e.pageX, e.pageY)) //PAGE NOT RELATIVE OFFSET!!!
      {
        const rawDragTargetData = val.getAttribute(TagDropTargetDataType);

        if(!rawDragTargetData)
          continue; //find me one that isn't fucking null, then

        const targetTag = JSON.parse(rawDragTargetData) as DragTargetData;


        if (targetTag)
        {
          switch(dragState)
          {
            case DragTargetState.Above:
              tagDb.repositionTag(data.tagId, targetTag.tagId, Position.Before);
              break;
            case DragTargetState.Center:
              tagDb.setTagParent(data.tagId, targetTag.tagId);
              break;
            case DragTargetState.Below:
              //If it's dropped below a tag that's open with children, then
              // we want it to parent ABOVE the first item in the list.
              // so, PREPEND NOT REPARENT, and the backend handles reparenting
              if(targetTag.openWithChildren) {
                //BUG: I think this just skips it rather than repositioning it if the parent is already the same...
                //Testing it, the old version works the same way and I didn't seem to care idk TODO look into it later
                tagDb.setTagParent(data.tagId, targetTag.tagId);
              } else {
                tagDb.repositionTag(data.tagId, targetTag.tagId, Position.After);
              }
              break;
          }
          break;
        }
      }

    } catch(e: any) {
      console.log("Failed to accept drop: " + e);
    }
  }

  //We need the tag list to be top level cause while it's basically
  // recursive it's managed by the root
  //Uhhh?? how does this affect what we're doing now?
  //I think this comment makes no sense now...

  let layoverStyle = {display: "none", top: 0}

  if(dragState != DragTargetState.None)
  {
    layoverStyle.display = "block";
    layoverStyle.top = layoverOffset;
  }

  return (
      <div className='tag-list-item-root-container'
          ref={dropZoneContainerRef}
          onDragOver={myDragOverHandler}
          onDragEnter={myDragEnterHandler}
          onDragLeave={myDragLeaveHandler}
          onDrop={myOnDropHandler}
      >
        <InternalTagList
            {...props}
        />
      <div className={`tag-list-item-drop-zone-layover
                       ${dragState != DragTargetState.Center ? "tag-list-item-drop-zone-layover-tween" : ""}`}
            style={layoverStyle}>
      </div>
      </div>)
}


function TagListItem(props: TagListItemProps) {
  const [isSelected, setIsSelected] = useState<Boolean>(false);
  const [isHidden, setIsHidden] = useState<Boolean>(false);
  const [showChildren, setShowChildren] = useState<Boolean>(false);
  const [hasChildren, setHasChildren] = useState<Boolean>(false);

  //TODO: The content of the tag item itself should probably be in a separate
  // component so it doesn't force a re-render of the children whenever this stuff changes...
  const [childrenHaveSomethingToSay, setChildrenHaveSomethingToSay] = useState<Boolean>(false);
  const [childIsSelected, setChildIsSelected] = useState<Boolean>(false);
  const [areObjectsSelected, setAreObjectSelected] = useState<boolean>(false);

  //Use this to track the current tag action, but NOT whether it's none or not.
  // we use areObjectsSelected to compute whether this is relevant so it needs to reset to
  // TagAction.Add
  const [explicitlySetTagAction, setExplicitlySetTagAction] = useState<TagAction>(TagAction.Add);


  const [tagName, setTagName] = useState<String>("");

  const tagListItemRef = useRef<HTMLDivElement>(null);
  const dropZoneRef = useRef<HTMLDivElement> (null);

  const tagDb = props.binderContext.db;

  useEffect(() => {
    if(!props.binderContext.db.checkLimitTagChildrenToObjectSelection(props.tagId)) {
      setHasChildren(tagDb.checkTagHasChildren(props.tagId));
    }

    setIsSelected(props.binderContext.tagSelectionSentinel.get(props.tagId) != undefined);

    const syncObjectSelectedOrNot = () => {
      //So this should only fire on first selection, and it's suppose to pump the whole of all the
      // tags, and then later the more relevant one fires. Or it's saying we're all done, so
      // that's simpol as well :3
      if (props.binderContext.objectsSelectedOrNotSignal.get()) {
        setAreObjectSelected(true);
      } else {
        setAreObjectSelected(false);
      }
    }
    const objectdSelectedOrNotId = props.binderContext.objectsSelectedOrNotSignal.register(() => {
      syncObjectSelectedOrNot();
    });

    syncObjectSelectedOrNot();


    const syncTagAction = () => {
      const tagAction = props.binderContext.tagActionObservers.get(props.tagId);
      if (tagAction != undefined) {
        setExplicitlySetTagAction(tagAction);
      } else {
        setExplicitlySetTagAction(TagAction.Add); //reset to default
      }
    };

    syncTagAction();

    const tagActionObserverId = props.binderContext.tagActionObservers.register(props.tagId, () => {
      syncTagAction();
    });

    const tagSelectionObserverId = props.binderContext.tagSelectionSentinel.register(props.tagId, () => {

      //!!
      setIsSelected(!!props.binderContext.tagSelectionSentinel.get(props.tagId));
    });

    const childSomethingSayCounter = props.binderContext.getParentsWhoseChildrenHaveSomethingToSayCounter();
    const syncChildrenSomethingToSay = () => {
      setChildrenHaveSomethingToSay(!!childSomethingSayCounter.get(props.tagId));
    };

    syncChildrenSomethingToSay();

    const childrenHaveSomethingToSayObsId = childSomethingSayCounter.register(props.tagId, () => {
      syncChildrenSomethingToSay();
    });

    const childrenSomethingToSayGreenEdition = props.binderContext.getParentsWhoseChildrenHaveSomethingToSayGreenEditionCounter();
    const syncGreenEdition = () => {
      setChildIsSelected(!!childrenSomethingToSayGreenEdition.get(props.tagId));
    };

    syncGreenEdition();

    const childrenHaveSomethingToSayGreenEditionId = childrenSomethingToSayGreenEdition.register(props.tagId, () => {
      syncGreenEdition();
    });

    const tagNameChangeObserverId = props.binderContext.db.registerTagNameChangeObserver(props.tagId, (name: string) => {
      if(!name)
        name = "";

      setTagName(parseTagName(name));
    });

    setTagName(parseTagName(tagDb.getTagName(props.tagId)));


    //TODO: filtering for the search results...
    // easiest to handle this centrally like the mapper of old...

    const childrenChangedId = props.binderContext.db.registerSpecificChildrenObserver(props.tagId, () => {
      setHasChildren(tagDb.checkTagHasChildren(props.tagId));
    });


    return () => {
      props.binderContext.objectsSelectedOrNotSignal.unregister(objectdSelectedOrNotId);
      props.binderContext.tagActionObservers.unregister(props.tagId, tagActionObserverId);
      props.binderContext.tagSelectionSentinel.unregister(props.tagId, tagSelectionObserverId);
      childSomethingSayCounter.unregister(props.tagId, childrenHaveSomethingToSayObsId);
      //console.log("Unregistering!!!!! green edition for tag: " + props.tagId);
      childrenSomethingToSayGreenEdition.unregister(props.tagId, childrenHaveSomethingToSayGreenEditionId);
      props.binderContext.db.removeTagNameChangeObserver(props.tagId, tagNameChangeObserverId);
      props.binderContext.db.removeSpecificChildrenObserver(props.tagId, childrenChangedId);
    };
  }, []);

  useEffect(() => {
    if(!props.binderContext.db.checkLimitTagChildrenToObjectSelection(props.tagId)) {
      return;
    }

    const counter = props.binderContext.getParentsWhoseChildrenHaveSomethingToSayCounter();

    const sync = () => {
      if(counter.get(props.tagId)) {
        setHasChildren(true);
      } else {
        setHasChildren(false);
      }
    }

    sync();

    const id = counter.register(props.tagId, () => {
      sync();
    });

    //This only gets returned if we register, so it's fine
    return () => {
      counter.unregister(props.tagId, id);
    }

  });


  const toggleShowChildren = (e: any): void => {
    setShowChildren(!showChildren);
  }


  //This is GROSS
  //I think it would be better to own the... well the list for the current tags or...
  // have the child tag.. idk have a counter or something so if it's the last child it goes away?
  // Ughh...

  const addTagCallback = () => {
    const selectedObjectList = props.binderContext.getSelectedObjects();

    if (!selectedObjectList) {
      return;
    }
    tagDb.addTagToObjects(props.tagId, selectedObjectList);
  }

  const removeTagCallback = () => {
    const selectedObjectList = props.binderContext.getSelectedObjects();

    if(!selectedObjectList) {
      return;
    }

    tagDb.removeTagFromObjects(props.tagId, selectedObjectList);
  };

  const tagClickedHandler = (e: React.MouseEvent) => {
    if(!e.ctrlKey && !e.shiftKey) {
      props.binderContext.setSingleTagSelection(props.tagId);
      /*
      const currentlySelected = props.binderContext.getSelectedTags();
      for(const selTag of currentlySelected) {
        if(selTag == props.tagId) {
          continue;
        }

        props.binderContext.setTagSelection(selTag, false);
      }

      props.binderContext.setTagSelection(props.tagId, true);
      */
    } else if(e.ctrlKey && !e.shiftKey) {
      props.binderContext.toggleTagSelection(props.tagId);
    } //Else we do shift and go down the sorted tags list hmmm... Only truly useful with
    // drag and drop tho, which we don't have for multiple tags yet...
  };


  const selectedTagClass = isSelected ? "tag-list-item-current" : "";



  const myDragStartHandler = (e: React.DragEvent<HTMLDivElement>) =>
  {
    const data = { dragType: "tag", tagId: props.tagId.toString() }
    const dataString = JSON.stringify(data);
    //console.log("Setting: " + dataString)


    e.dataTransfer.setData(TagItemDragDataType, dataString)

    //We don't set the dropTarget to ourselves even though that's valid
    // because it'll restyle the dragged element bubble thing and we don't want that.
  }

  const myDragEnterHandler = (e: React.DragEvent<HTMLDivElement>) =>
  {
    if(!e.dataTransfer.types.includes(TagItemDragDataType)) //lowercased here only
      return;

    e.preventDefault();
  }

  const contextMenuHandler = (e: React.MouseEvent) =>
  {
    const menu: ContextMenuData = 
    {
      spawnX: e.pageX,
      spawnY: e.pageY,
      contextMenuSections:
      [
        {
          contextMenuItems:
          [
            {
              title: "New Tag Here",
              keyBinding: "",
              action: () => {
                //I like that I can do this on the fly now, but also kinda... don't know if this
                // should be encapsulated here or not... idk...
                props.binderContext.createNewTagAndAddSelected(tagDb.getTagParent(props.tagId), props.tagId);
              }
            },
            {
              title: "New Child Tag",
              keyBinding: "",
              action: () => {
                props.binderContext.createNewTagAndAddSelected(props.tagId);

                /*
                //unpositioned tags bubble to the top anyway so w/e
                const childTagList = tagDb.getSortedTags(props.tagId);
                if(childTagList.length < 1) {
                  console.warn("Just created a tag but the child list is empty");
                  return; //Weird
                }

                const topTagId = childTagList[0];

                if(topTagId == createdTag) {
                  return; //nothing to see here
                }

                //tagDb.repositionTag(createdTag, topTagId, Position.Before);
                */
              }
            },
            {
              title: "Delete Tag",
              keyBinding: "",
              action: () => {
                props.binderContext.deleteTag(props.tagId);
              }
            },
          ],
        },
          /*
        {
          contextMenuItems:
          [
            {
              title: "Include object on child tags",
              keyBinding: "",
              action: () => { }
            },
            {
              title: "Show only children on selected",
              keyBinding: "",
              action: () => { }
            }
          ]
        }

           */
      ],
    }

    e.preventDefault();
    getContextMenuStator(props.appContext).spawn(menu);
  }



  const style = isHidden ? { display: "none" } : {};


  if(dropZoneRef.current)
  {
    const data =
    {
      tagId: props.tagId,
      openWithChildren: showChildren,
    } as DragTargetData

    const dataAsString = JSON.stringify(data);

    dropZoneRef.current.setAttribute(TagDropTargetDataType, dataAsString);
  }

  const dropdownSymbol: string = (() => {
    if(showChildren)
      return "▾";
    if(childrenHaveSomethingToSay || childIsSelected)
      return "⤥";
    return "▸";
  })();

  const childList = showChildren ? (
      <div className="tag-list-item-child-tag-container">
        <InternalTagList parentTag={props.tagId}
                         theBinderId={props.theBinderId}
                         appContext={props.appContext}
                         binderContext={props.binderContext}
        />
      </div>) : undefined;

  //It was three cause at some point I had the id or data-tag-id set on both the container and tag item
  // but it stayed that way during a hot reload and after a full refresh it's just two again

  const computedTagAction = areObjectsSelected ? explicitlySetTagAction : TagAction.None;
  const hasChildrenClass = hasChildren ? "tag-list-item-body-with-children" : "";

  const renderRelated = (computedTagAction == TagAction.Remove || computedTagAction == TagAction.AddRemove);

  const renderUntouchableRelated = renderRelated && props.binderContext.db.checkLimitTagChildrenToObjectSelection(props.binderContext.db.getTagParent(props.tagId));
  const limitedTag = props.binderContext.db.checkLimitTagChildrenToObjectSelection(props.tagId);

  return (
    <div
      ref={dropZoneRef}
      id={props.tagId.toString()}
      style={style}
    >
      <div className={`tag-list-item`}
        ref={tagListItemRef}
        onContextMenu={contextMenuHandler}
        onDragStart={myDragStartHandler}
        draggable={true}
      >
        <TagMiniButton symbol={dropdownSymbol}
          doRender={hasChildren}
          currentTag={isSelected}
          greenCauseChildSelection={childIsSelected}
          callback={toggleShowChildren} />

        <button className={`tag-list-item-button tag-list-item-body-button ${selectedTagClass} ${hasChildrenClass}`}
          onClick={tagClickedHandler}>
          {tagName}
        </button>

        <div className="tag-list-item-modifier-container">
          <TagMiniButton symbol="+"
            doRender={((computedTagAction == TagAction.Add || computedTagAction == TagAction.AddRemove) && !renderUntouchableRelated && !limitedTag)}
            currentTag={isSelected}
            greenCauseChildSelection={false}
            callback={(e: any) => addTagCallback()} />

          <TagMiniButton symbol="-"
            doRender={renderRelated && !renderUntouchableRelated && !limitedTag}
            currentTag={isSelected}
            greenCauseChildSelection={false}
            callback={(e: any) => removeTagCallback()} />

          <TagMiniButton symbol="0"
            doRender={renderUntouchableRelated && !limitedTag}
            currentTag={isSelected}
            greenCauseChildSelection={false}
            callback={(e: any) => {}} />

        </div>
      </div>
      {childList}
    </div>
  );
}

export {TagList, TagListItemUpdatePayload, TagAction};