import React, {useState, useRef} from 'react'
import {TagList} from './taglistitem'
import { BinderConnectionId, BinderTag } from '../backend'
import { StatorContext, StatorObjectBase } from '../statorobject'
import './tagsidebar.css'
import './taglistitem.css'
import './subduedactionbutton.css'
import './navbar.css'
import { BinderContextPair, getBinderListStator, useWatchBinderListStator } from './openbinders'
import { reserved } from '../tagdatabase'
import { BinderContext, ContextStatus } from './bindercontext'
import { useWatchObservableValue } from '../observermanager'
import { useWatchSidebarStator} from "./sidebarstator";

function Header(props: {context: BinderContext})
{
  const [value, setValue] = useState<string>("");
  const ref = useRef<HTMLInputElement>(null);

  const blurHandler = () => {
    if (value)
      props.context.setSearchQuery(value);

    setValue("");
  }

  const inputHandler = (e: React.ChangeEvent<HTMLInputElement>) => {
    setValue(e.target.value);
  }


  const allDone = (e: any) => {
    e.preventDefault();

    ref.current?.blur();
  }

  return (
    <div className="tag-sidebar-tab-bar">
      {

        /*
              <button className="tag-sidebar-new-tag-button subdued-action-button"
                onClick={handleNewTag}>
                +
              </button>
        */
      }
      <form onSubmit={allDone}>
        <input className="tag-sidebar-header-input"
          ref={ref}
          onChange={inputHandler}
          onBlur={blurHandler}
          value={value}
          placeholder={"Search"}
        ></input>

      </form>
    </div>
  );
}

interface TagSidebarInstanceProps
{
  binderContext: BinderContext;
  theBinderId: BinderConnectionId;
  appContext: StatorContext;
}

function TagSidebarInstance(props: TagSidebarInstanceProps): JSX.Element
{
  const binderStator = getBinderListStator(props.appContext);
  const [tagList, setTagList] = useState<Array<BinderTag>>([]);
  const [isLoading, setIsLoading] = useState<boolean>(false);

  useWatchObservableValue(props.binderContext.status, (stat: number | undefined) => {
    setIsLoading(stat == ContextStatus.LoadingTags);
  });

  const handleNewTag = () => {
    props.binderContext.createNewTagAndAddSelected();
  }

  const selected = binderStator.checkSelected(props.theBinderId);

  const style = selected ? {} : { display: "none" };

  //console.log("Show loading? " + isLoading);

  return (
    <div style={style} className="tag-sidebar">
      <Header context={props.binderContext} />
      <div className="tag-sidebar-list-container">
      { isLoading ?
          <h2>
            Loading tags...
          </h2>
      :
          <TagList parentTag={reserved.tagIds.root} // :)
                   binderContext={props.binderContext}
                   theBinderId={props.theBinderId}
                   appContext={props.appContext}
          />
      }
      </div>
      <img className="tag-sidebar-new-tag-button"
        src={"static/plus-button.svg"}
        draggable={false}
        onClick={handleNewTag} />
    </div>
  );
}

interface TagSidebarProps
{
  appContext: StatorContext;
}

function TagSidebar(props: TagSidebarProps)
{
  //Fairly innefficient to re-render this errytime a selection changes...
  const binderListStator = useWatchBinderListStator(props.appContext);
  const tagSidebarStator = useWatchSidebarStator(props.appContext); //This is just for the show/hide button thing

  if(binderListStator.getOpenBinderList().length == 0)
  {
    return null; //TODO: close it
  }

  const tagSidebars = binderListStator.getOpenBinderList().map((b: BinderContextPair) =>
  {
    return (
      <TagSidebarInstance key={b.binder.id}
        binderContext={b.binder.context}
        theBinderId={b.binder.id}
        appContext={props.appContext}
      />);
  });

  const style = tagSidebarStator.tagSidebarHidden ? {display: "none"} : {};

  return <div style={style}>{tagSidebars}</div>;
}

interface SidebarViewToggleButtonProps
{
  appContext: StatorContext;
}

function SidebarViewToggleButton(props: SidebarViewToggleButtonProps)
{
  const sidebarStator = useWatchSidebarStator(props.appContext);//Cause this is for global stuff like show/hide


  const handleClick = (e: any) =>
  {
    sidebarStator.toggleTagSidebar();
  };

  const imgSrc = sidebarStator.tagSidebarHidden ? "static/show-tag-sidebar.svg"
                                      : "static/hide-tag-sidebar.svg";

  return (<img
    className={"nav-button binder-bar-ham-burber-bubbon"}
    draggable={false}
    onClick={handleClick}
    src={imgSrc}
    ></img>)
}

export {TagSidebar, SidebarViewToggleButton};