import React, { useEffect, useState, useRef } from 'react'
//import * as PDFjs from 'pdfjs-dist'
import {BinderConnectionId, BinderObject, IndexedObject, ObjectId, generateThumbnailForObject, getObjectUri, getThumbnailUri} from '../../backend'
import { ObjectViewerProps } from "../objectviewer";
import './objectthumbnail.css'
import {getContextMenuStator} from "../contextmenu";
import { getBinderListStator } from '../openbinders';
import { StatorContext } from '../../statorobject';
import { getMetaApi } from '../../ipcbindings';
import { getObjectViewerOpenStator, getObjectViewerStator } from '../objectviewercontroller';
import { GenericThumbnail } from './genericthumbnail';
import { BinderContext } from '../bindercontext';

//const pdfworker = await import('pdfjs-dist/build/pdf.worker.entry');
//PDFjs.GlobalWorkerOptions.workerSrc = pdfworker;

export interface ObjectThumbnailProps
{
  appContext: StatorContext,
  binderContext: BinderContext;
  obj: IndexedObject;
  theBinderId: BinderConnectionId;
  x: number;
  y: number;
}


class ObjectThumbnailUpdatePayload
{
  selected: Boolean = false;
}

function CheckerboardBgImageThumbnail(props: ObjectThumbnailProps)
{
  return <></>;
  const thumbnailUri = getObjectUri(props.obj.id);

  return (
    <img className={"object-thumbnail-checkerboard"}
      draggable={false}
      src={thumbnailUri}
    />
  );
}

function EyeCandyImageThumbnail(props: ObjectThumbnailProps) {
  const [loaded, setLoaded] = useState(false);
  const [lastLoadFailed, setLastLoadFailed] = useState(false);
  const [requestedGeneration, setRequestedGeneration] = useState(false);
  const [generationFailed, setGenerationFailed] = useState(false);
  //const thumbnailUri = "file:///" + props.obj.thumbnailOnDiskPath;
  //const thumbnailUri =  "http://0.0.0.0:18080/api/binder/" + "standardobjectthumbnail" + "/" + props.theBinderId + "/" + props.obj.b2bsum;
  const [thumbnailUri, setThumbnailUri] = useState("");
  const [thumbnailJobId, setThumbnailJobId] = useState<number | undefined>(undefined);

  useEffect(() => {
    (async () => {
       const thumbp = await getBinderListStator(props.appContext).getCurrentBinder()?.context.db.getThumbnailPath(props.obj.id);

       if(!thumbp) { //this doesn't mean it doesn't exist, it means the binder stator couldn't return a proper context
        throw new Error("this should not happen");
        setLastLoadFailed(true);
        return;
       }

       setThumbnailUri(thumbp.path);
    })();
  }, []);

  useEffect(() => {
    return () => {
      if(thumbnailJobId) {
        //CANCALE JOBEEE
      }
    }
  }, []);

  const loadHandler = (e: any) => {
    setLoaded(true);
  };

  const failureHandler = async (e: any) => {
    setLastLoadFailed(true);

    if(requestedGeneration) {
      setGenerationFailed(true);
      //console.log("Already requested thumbnail generation, skipping");
      return; //no infinite looopy
    }

    setRequestedGeneration(true);

    //console.log("Requesting thumbnail generation for: " + props.obj.b2bsum);

    const currentBinder = getBinderListStator(props.appContext).getCurrentBinder();
    if(!currentBinder) {
      console.warn("Binder not ready for thumbnail generation");
      return;
    }

    const objectPaths = await currentBinder.context.db.getObjectPath(props.obj.id);

    if(!objectPaths) {
      console.warn("Object missing");
      return;
    }

    const thumbnailPaths = await currentBinder.context.db.getThumbnailPath(props.obj.id);

    if(!thumbnailPaths) {
      console.warn("thumbnail path missing");
      return;
    }

    console.log("thumbnail path: " + thumbnailPaths.path);

    await getMetaApi().createDirectories(thumbnailPaths.directory);
    const jobId = await getMetaApi().generateThumbnailSharp(objectPaths.path, thumbnailPaths.path);
    await getMetaApi().longPollThumbnailJob(jobId);
    setLastLoadFailed(false);

    /*
    const jobId = await thumbnailWorker.postJob(objectPaths.path, thumbnailPaths.path, thumbnailPaths.directory, () => {
    }, () => {});
    */

    setThumbnailJobId(jobId);
  };

  //standin is from a higher up component so we can just render nothing

  if(lastLoadFailed && thumbnailUri && requestedGeneration && !generationFailed) {
    return <GenericThumbnail bodyText='' bottomBannerText='Generating Preview...'/>
  }

  if(!thumbnailUri) {
    //Technically, it's not actually "loading" the thumbnail, it's just waiting for the thumbnail uri which
    // is actually kind of slow to get back... It would be faster to have that already in the IndexedObject
    // structure......
    //But I do like that it says loading first...
    return <></>; //text/components is apparently too slow. Might just be that it's a react component...
    return <GenericThumbnail bodyText='' bottomBannerText='Loading..' />
  }

  if (generationFailed) {
    //mime like 99.999% chance exists already cause it's uh you know
    // already requested before we got this far...
    // (except the case where it failed to generate because of no disk space and also failed to write mime...)
    //TODO: Genericize "failed to load" thumbnail and let it figure out all this shit...
    return <GenericThumbnail bodyText={props.binderContext.db.pickFilename(props.obj.id)}
      bottomBannerText={props.binderContext.db.getMimeIfExists(props.obj.id)} />
  }

  const wrappedURI = "file:///" + thumbnailUri;

  return (
      <div className="object-eyecandy-thumbnail" >
        <img
            className={"object-eyecandy-thumbnail-background"}
            draggable={false}
            src={wrappedURI}
        />
        <img className={"object-eyecandy-thumbnail-image"}
             onLoad={loadHandler}
             onError={failureHandler}
             draggable={false}
             hidden={!loaded && !lastLoadFailed}
            src={wrappedURI}
        />
      </div>
  );
}


function TextPlainThumbnail(props: ObjectThumbnailProps)
{
  const [text, setText] = useState<string>("");

  return <></>;
  const textUri = getObjectUri(props.obj.id);

  useEffect(() =>
  {
    const request = new Request(textUri);

    fetch(request).then((resp: Response) =>
    {
      resp.text().then((value: string) =>
      {
        setText(value);
      });
    });
  }, [textUri]);


  return (
    <div className="object-thumbnail-plaintext-background">
      <div className='object-thumbnail-plaintext-container'>
        <pre className="object-thumbnail-text">{text}</pre>
      </div>
    </div>
  );
}


/*
async function  renderPDFthumbnailEventually(bgCanvas: HTMLCanvasElement,
                                             fgCanvas: HTMLCanvasElement,
                                             pdfURI: string,
                                             doneCallback: () => void)
{
  const pdf = (await PDFjs.getDocument(pdfURI).promise);

  const page = await pdf.getPage(1); //No magic numbers here, silly goose

  const viewport = page.getViewport({ scale: 1 }); //why this scale tho?

  const context = fgCanvas.getContext('2d');
  if (!context)
    throw "Context was null";

  bgCanvas.width = fgCanvas.width = viewport.width;
  bgCanvas.height = fgCanvas.height = viewport.height;

  const renderContext =
  {
    canvasContext: context,
    viewport: viewport,
  };

  page.render(renderContext).promise.then(() =>
  {
    const bgContext = bgCanvas.getContext('2d');
    if (!bgContext)
      return;
    
    bgContext.drawImage(fgCanvas, 0, 0);
    doneCallback();
  });
}


function PDFThumbnail(props: ObjectThumbnailProps)
{
  const bgCanvasRef = useRef<HTMLCanvasElement>(null);
  const imageCanvasRef = useRef<HTMLCanvasElement>(null);
  const [thumbnailReady, setThumbnailReady] = useState<boolean>(false);


  useEffect(() => 
  {
    if (bgCanvasRef.current && imageCanvasRef.current)
     {
      const pdfURI = getObjectUri(props.obj);

        renderPDFthumbnailEventually(bgCanvasRef.current,
        imageCanvasRef.current,
        pdfURI,
        () => { setThumbnailReady(true) });
    }
  } ,[bgCanvasRef, imageCanvasRef])

  let style = { display: "none" };

  if(thumbnailReady)
    style = {display: "unset"}


  return (
    <div className='object-eyecandy-thumbnail'>
      <canvas style={style} ref={bgCanvasRef} className="object-eyecandy-thumbnail-background"/>
      <canvas style={style} ref={imageCanvasRef} className="object-eyecandy-thumbnail-image"/>
    </div>
  );
}
*/

//really need like a "useMime" custom hook...
//cause this is all so duplicated.....
function ObjectThumbnail(props: ObjectThumbnailProps) {
  const context = getBinderListStator(props.appContext).getBinder(props.theBinderId).context;

  //I *think* it's faster than using a promise because we can skip re-rendering the thumbnails?
  const [mime, setMime] = useState<string>(context.db.getMimeIfExists(props.obj.id));
  const [isSelected, setIsSelected] = useState<Boolean>(false);


  useEffect(() => {
    if(mime) {
      return;
    }

    (async () => {
      const queriedMime = await context.db.getMimeGenIfNotExists(props.obj.id);
      if(!queriedMime) return;

      //this is where the "can't perform react state update" "indicates memeory leak"
      // error comes from.
      setMime(queriedMime);
    })();
  });

  useEffect(() => {
    const observable = context.objectSelectionObservers;
    const syncSelection = () => {
      if (observable.get(props.obj.id)) {
        setIsSelected(true);
      } else {
        setIsSelected(false);
      }
    }

    const observerId = observable.register(props.obj.id, syncSelection);
    syncSelection();

    return () => {
      observable.unregister(props.obj.id, observerId);
    };
  }, []);


  const clickHandler = (e: React.MouseEvent) => {
      //e.stopPropagation(); needed for closeable overlay
      e.preventDefault();

      if(e.ctrlKey && !e.shiftKey) {
        context.toggleObjectSelection(props.obj.id);
      } else if(!e.ctrlKey && e.shiftKey) {
        context.shiftSelect(props.obj.index);
      } else if(!e.ctrlKey && !e.shiftKey) {
        if(e.detail == 2) {
          context.openViewer(props.obj);
        } else {
          context.handoffSingleObjectSelection(props.obj);
        }
      }
    }

  let myJsx = <div></div>;

  //Really this should maybe be the default but there should be a attribute setting
  // that allows you to choose what background you want for this particular object in thumbnail or
  // in preview.

  /*
  if(mime == "image/svg+xml") //then we render the fullfat one cause it's still lighter than  athumbnail probably
  {
    myJsx = <CheckerboardBgImageThumbnail {...props}/>;
  }
  /*
  else if (props.obj.mime == "application/pdf")
  {
    myJsx = <PDFThumbnail {...props} />
  }
  else if (mime == "text/plain")
  {
    myJsx = <TextPlainThumbnail {...props}/>
  }
  */
  if(mime) //split up the thing
  {
    const splits = mime.split("/");

    if(splits.length > 1) {
      const mainMimeType = splits[0];
      const secondMimeType = splits[1];


      if (mainMimeType == "image") {
        myJsx = <EyeCandyImageThumbnail {...props} />
      } else if (mainMimeType == "application") {
        let val = secondMimeType;
        if(secondMimeType == "octet-stream") {
          val = "unknown";
        }
        myJsx = <GenericThumbnail bodyText={context.db.pickFilename(props.obj.id)} bottomBannerText={val} />
      } else {
        myJsx = <GenericThumbnail bodyText={context.db.pickFilename(props.obj.id)} bottomBannerText={mime} />
      }
    } 
  }

  const style: React.CSSProperties = {left: props.x, top: props.y}

  const noopMouseClick = (e: React.MouseEvent) => {
    e.stopPropagation();
  };

  return (
    <div className={`object-thumbnail ${isSelected ? "object-thumbnail-selected" : ""}`}
      style={style}
      tabIndex={1}
      onMouseDown={clickHandler}
      onClick={noopMouseClick}>
      <div className='object-thumbnail-underlay'>
        <div className={"object-thumbnail-standin"} tabIndex={1} ></div>

        {
          //I try this not inside a debug build, right now it's just too slow for clean scrolling down the
          // page, it's rendering text that is the problem, apparently, it just takes a liiiittle too long
          // and it can't keep up with the scrolling like an empty div can
          //<GenericThumbnail bodyText='' bottomBannerText='Loading...' />
        }
      </div>
      <div className='object-thumbnail-overlay'>
        {myJsx}

      </div>
    </div>
  );
}

export { ObjectThumbnailUpdatePayload, ObjectThumbnail }