import React from 'react';
import { StatorContext } from '../statorobject';
import { GenericOverlayWrapper } from './genericoverlay';
import { getBinderVersion, UpdaterStatorState, useWatchUpdaterStator } from '../updaterstator';
import {  handleGotoEnesdaPage } from '../ipcbindings';
import { getActivationStator } from '../activation';


interface UpdaterOverlayProps
{
    appContext: StatorContext;
}


async function handleGotoDownloadPageAsync(context: StatorContext)
{
    handleGotoEnesdaPage((await getBinderVersion()) + "/" + getActivationStator(context).licenseKey);
}

function UpdaterOverlayImpl(props: UpdaterOverlayProps)
{
    //Updator stator listens to the activation stator for changes so don't worry about it
    const stator = useWatchUpdaterStator(props.appContext);

    const handleGotoDownloadPage = () =>
    {
        handleGotoDownloadPageAsync(props.appContext);
    };

    const GoDownload = (props: {linkTitle: string}) =>
    {
        return <a href="#" onClick={handleGotoDownloadPage}>{props.linkTitle}</a>
    };

    const message = (() =>
    {
        switch(stator.getState())
        {
            case UpdaterStatorState.FailedToCheck:
                return <p>There was an error checking for updates. Check your internet connection or try again later.</p>;
            case UpdaterStatorState.Checking:
                return <p>Checking for updates...</p>;
            case UpdaterStatorState.Unknown:
                return <p>Do you want to check for updates?</p>;
            case UpdaterStatorState.UpToDate:
                return <p>Everything is currently up to date!</p>;
            case UpdaterStatorState.UpdateAvailable:
                return <p>An update is available <GoDownload linkTitle=" click here to learn more."/></p>;
        }
    })();

    return (<div>
        <h1>Updates</h1>
        {message}
        <form onSubmit={(e: any) => {e.preventDefault(); stator.checkForUpdates()}}>
            <input type="submit" className={"generic-overlay-form-button"} value="Check for updates"/>
        </form>
    </div>);
}


function UpdaterOverlay(props: UpdaterOverlayProps)
{
    return (<GenericOverlayWrapper
        appContext={props.appContext}
        startTab={0}
        tabs={
            [
                {
                    title: "Updates",
                    nestedComponent: UpdaterOverlayImpl,
                    nestedProps: props,
                }
            ]
        }/>);
}


export { UpdaterOverlay }