import { useEffect, useRef} from 'react'

//You must set the ref of the thing you want to not be excluded from
// the close click even as such:
// <elem ref={myRef}/>
function useCloseableOverlay(callback: () => void)
{
    const myRef = useRef<HTMLDivElement>(null);
    useEffect(() =>
    {
        function handler(e: any)
        {
            if(myRef.current && !myRef.current.contains(e.target))
            {
                callback();
                e.preventDefault();
            }
        };

        //In order to stop propogation of a click event, we have to track the call to
        // the callback, and then if it's fired, we skip the mouseup event, and
        // reset that state...
        //(cause it's the proceeding "up" event that triggers a click event)

        document.addEventListener("mousedown", handler);

        return () => {
            document.removeEventListener("mousedown", handler);
        }
    });

    return myRef;
}


export { useCloseableOverlay }