
/*
class IncomparableRecords extends Error {
    constructor(message) {
        super(message);
        this.name = this.constructor.name;
    }

}
 */


//Used for loading as well as asserting construction was valid
//we are tolerant of extraneous values, assuming that future versions of nucleus
// that was to be backwards compatible might write extra data.
function assertGoodWriteRecord(obj) {
    /* this makes testing difficult, and... isn't consistent, anyway
    Object.freeze(obj);

    if(!Object.isFrozen(obj))
        throw new Error("Object must be frozen to be a valid write record");

     */

    if(!("recordType" in obj))
        throw new Error("Missing record type field");

    if(!("timestamp" in obj))
        throw new Error("Missing timestamp field");

    if(typeof obj.timestamp !== "number")
        throw new Error("Wrong type for timestamp");

    if(!("key" in obj))
        throw new Error("Missing key field");

    if(typeof obj.key !== 'string')
        throw new Error("Wrong type for key id");



    if(!("machineId" in obj))
        throw new Error("Missing machine id field");

    if(typeof obj.machineId !== 'string')
        throw new Error("Wrong type for machine id");

    if(!("machineIndex" in obj))
        throw new Error("Missing machine index");

    if(typeof obj.machineIndex !== 'number')
        throw new Error("Wrong type for machine index");

    if(!("anchors" in obj))
        throw new Error("Missing anchors");

    if(typeof obj.anchors === 'object')
    {
        for(const key in obj.anchors)
        {
            if(typeof obj.anchors[key] !== 'number')
                throw new Error("Wrong type for anchor machine index, expected number");
        }
    }
    else
        throw new Error("Wrong type for anchors object");

    if(!("value"in obj))
        throw new Error("Missing value");

    if(typeof obj.value !== 'string')
        throw new Error("Wrong type for value");
}

function constructActiveProperty(machineId, key) {
    const obj = {};

    obj.machineId = machineId;
    obj.machineIndex = 0;
    obj.key = key;
    obj.anchors = {};

    obj.topRecordByMachine = {};

    assertGoodActiveProperty(obj);

    return obj;
}

function assertGoodActiveProperty(property) {
    if(!property)
        throw new Error("Undefined property"); //lol I mean

    if(!("machineId" in property))
        throw new Error("Property missing machineId");

    if(!("machineIndex" in property))
        throw new Error("Property missing machineIndex");

    if(!("key" in property))
        throw new Error("Property missing key");

    if(!("anchors" in property))
        throw new Error("Property Missing anchors");

    if(!("topRecordByMachine" in property))
        throw new Error("Property Missing top record by machine field");
}

function digestRecordIntoProperty(property, record) {
    assertGoodActiveProperty(property);
    assertGoodWriteRecord(record);

    if(record.machineId in property.topRecordByMachine) {
        const committed = property.topRecordByMachine[record.machineId];
        const superiorAnchors = {...record.anchors};

        //merge anchors, regardless of machine index. Any witnessed higher anchor on this machine is valid :)
        for(const machineId in committed.anchors) {
            if(!(machineId in superiorAnchors)) {
                superiorAnchors[machineId] = committed.anchors[machineId];
                continue;
            }

            const comAnchor = committed.anchors[machineId];
            const recAnchor = record.anchors[machineId];

            if(comAnchor > recAnchor)
                superiorAnchors[machineId] = comAnchor;
        }


        const sofitsticated = (a, b) => {
            if(a.machineIndex > b.machineIndex)
                return 1;
            if(a.machineIndex < b.machineIndex)
                return -1;

            //Equal machine indices means we have a local conflict, which is resolved by timestamp,
            // and then... yeah... main thing is to make sure it's consistent.

            if(a.timestamp > b.timestamp)
                return 1;
            if(a.timestamp < b.timestamp)
                return -1;

            //Now we've gone and done it.
            //The following isn't meant to be pretty, or logical. It's meant to be predictable.

            if(a.recordType === "write" && b.recordType === "write") {
                //Lexicographically compare the value as a tie-breaker, if it's the same then it
                // really doesn't matter now, then does it?

                //Notice: this is the stringified object, readPropertyValue is what parses it
                return predictableStringComparison(a.value, b.value);
            }

            if(a.recordType === "delete" && b.recordType === "delete") {
                return 0; //it really doesn't matter, but they are equal
            }

            //only thing left is a write AND a delete, the write takes precedence for arbitrary reasons

            if(a.recordType === "write") {
                return 1;
            }

            if(b.recordType !== "write" || a.recordType !== "delete") {
                throw new Error("very bad logic error or something");
            }
            //:)

            return -1;
        }

        const compResult = sofitsticated(record, committed);

        //-1 is committed, so no-op, but potentially propogate the anchors
        //0 is equal, so no-op for the value (but we want to propagate the anchors)

        if(compResult === 1)
        {
            //Then record is the preferred record.
            property.topRecordByMachine[record.machineId] = record;
        }

        //Merge superior anchors regardless
        property.topRecordByMachine[record.machineId].anchors = superiorAnchors;

    } else {
        //New machine
        property.topRecordByMachine[record.machineId] = record;
    }


    //If we have witnessed a machine.index higher than this one, in the property,
    // then it must have been written to property.anchors.machineId.index, and therefore
    // we know if this record is new or not
    //If it's equal or lower, then it's a local conflict and might have different anchors which
    // need to propagate.

    //if we made it past that check, then we're authorized to bump anchors, and erase
    // any top records that have matching or lower indices in property.anchors




    for(const machineId in record.anchors) {
        //Because we delete records, and we might reference our old anchor (not supposed to but it's harmless to
        // defensive)
        if(machineId === record.machineId)
        {
            console.log("Nucleus2: Warning! record anchored to it's own machine!")
            continue;
        }

        const recordAnchorIndex = record.anchors[machineId];

        /*
        //We can only obsolete records that are anchored by the record we're digesting, right?
        //Delete obsolete records
        if(machineId in property.topRecordByMachine) {
            if(property.topRecordByMachine[machineId].machineIndex <= recordAnchorIndex) {
                delete property.topRecordByMachine[machineId];
            }
        }

         */

        //bump property wide anchors

        if(machineId in property.anchors) {
            const propertyAnchorIndex = property.anchors[machineId];

            if(propertyAnchorIndex >= recordAnchorIndex)
                continue;
        }

        property.anchors[machineId] = recordAnchorIndex;
    }

    //If it's a local record then we bump the local machine index, which we own.
    //No-op if local conflict
    if(record.machineId === property.machineId) {
        if(property.machineIndex < record.machineIndex) {
            property.machineIndex = record.machineIndex;
        }
    } else {
        //We have to set the property level anchor because records are not self-referential,
        // but only if it's not the same machineId because we use these anchors to create
        // new records and again, they should not be self referential
        const prevAnchor = (() => {
            if(record.machineId in property.anchors) {
                return property.anchors[record.machineId];
            }
            return 0;
        })();

        if(prevAnchor < record.machineIndex) {
            property.anchors[record.machineId] = record.machineIndex;
        }
    }
}

function predictableStringComparison(a, b) {
    //I don't trust compareLocale
    if(a === b) {
        return 0;
    }

    if(a < b) {
        return -1;
    }

    if(a > b) {
        return 1;
    }

    throw new Error("That's a new one for sure");
}

function pickTopPropertyRecord(property) {
    assertGoodActiveProperty(property);
    const comparator = (a, b) => {
        if(a.timestamp < b.timestamp)
            return -1;

        if(a.timestamp > b.timestamp)
            return 1;

        //equal timestamps
        //Use machine id as tie-breaker
        return predictableStringComparison(a.machineId, b.machineId);
    }

    const conflictingRecrods = (() => {
        const result = [];

        const candidates = {...property.topRecordByMachine};
        //console.log(JSON.stringify(candidates));


        for(const key in candidates) {
            const prop = candidates[key];
            for(const anchoredMachineId in prop.anchors) {
                if(!(anchoredMachineId in candidates))
                    continue;

                const candidateMachineIndex = candidates[anchoredMachineId].machineIndex;
                const ourAnchor = prop.anchors[anchoredMachineId];

                if(candidateMachineIndex <= ourAnchor) {
                    delete candidates[anchoredMachineId];
                }
            }
        }


        //Go over what's left
        for(const key in candidates) {
            result.push(property.topRecordByMachine[key]);
        }

        return result;
    })();

    if(!conflictingRecrods.length)
        return null;

    conflictingRecrods.sort(comparator);

    return conflictingRecrods.at(-1);
}

function readPropertyString(property) {
    assertGoodActiveProperty(property);
    const winner = pickTopPropertyRecord(property);

    if(winner === null)
        return undefined; //because the property record defines the value

    if(winner.recordType === "write")
        return pickTopPropertyRecord(property).value;
    else
        return undefined;//I mean... it could be just... null for undefiend and for deleted?
}

function readPropertyValue(property) {
    const propertyString = readPropertyString(property);

    if(propertyString !== undefined)
        return JSON.parse(propertyString);

    return undefined;
}


function createRecord(property, type, value = null) {
    assertGoodActiveProperty(property);

    if(type !== "write" && type !== "delete")
        throw new Error("Record type must be either write or delete");

    const obj = {};
    obj["recordType"] = type;
    obj["timestamp"] = Date.now();
    obj["key"] = property.key;
    obj["machineId"] = property.machineId;


    //How do we increment? safely in case of failed commits?
    obj["machineIndex"] = property.machineIndex + 1;

    //Absolutely has to be a deep copy!!!!!!
    //It's a smol object anyway.
    obj["anchors"] = JSON.parse(JSON.stringify(property.anchors)); // need a list of highest machine indices...

    //Shouldn't be necessary but...
    delete obj["anchors"][property.machineId];

    obj["value"] = JSON.stringify(value);

    assertGoodWriteRecord(obj);

    return obj;
}


export {createRecord,
    readPropertyValue,
    readPropertyString,
    digestRecordIntoProperty,
    constructActiveProperty,
    assertGoodWriteRecord,
    }
