import lockfile from 'proper-lockfile';
import { app, ipcMain, shell, dialog, Menu } from 'electron';
import { thumbnailWorker } from './thumbnailworkernode.js';
import * as pathUtils from './pathutils.js';


//const buildInfo = JSON.parse(fs.readFileSync(path.resolve(__dirname, 'buildInfo.json'), 'utf8'));

let lastJobId = 0;

const dbWorkerString = "dbWorker";
const thumbnailWorkerString = "thumbnailWorker";

const jobBoard = {}

function setupCommonIPC(authKey, win) {
    jobBoard[dbWorkerString] = {};
    jobBoard[thumbnailWorkerString] = {};

    const fileMonitor = new pathUtils.FileMonitor(win.webContents);

    //Was gonna use a separate file for the worker script....
    // but then I realized I'd also have to add that to the setup system
    // to copy *that* file as well because this is the electron side...
    //So, thread stuff is at the bottom
    /*
    const dbWorker = new Worker(__filename, {
        workerData: dbWorkerString
    });


    //todo REFACTOR ALL THIS DUPLICATED SHITTTT
    console.log("registering db worker");
    dbWorker.on('message', (message) => {
        //console.log("Fulfilling the promise");
        jobBoard[dbWorkerString][message.jobId].resolve(message.response);
        //console.log("Deleting the fulfiller");
        delete jobBoard[dbWorkerString][message.jobId];
    });
    console.log("done register");
    */


    ipcMain.handle('dirName', () => { return __dirname});

    ipcMain.handle('just-die', () => app.quit());
    ipcMain.handle('get-auth-key', async () =>
    {
        return authKey;
    });
    ipcMain.handle("open-url", (event, args) =>
    {
        console.log(args);
        shell.openExternal(args);

        //Not sure what this part does...
        return {action: 'deny'};
    });
    ipcMain.handle('open-select-directory', (event, args) =>
    {
        return dialog.showOpenDialogSync({properties: ["openDirectory"]}); //no options cause defaults are gud
    });
    ipcMain.handle('spawn-menu', (event, args) => {
        console.log("Spawning")
        console.log(args);
        const m = new Menu();
        m.popup();
    });
    ipcMain.handle("getConfigDirPath", async () => {
        return pathUtils.getSystemConfigDirPath();
    });
    ipcMain.handle("checkPathExists", async (event, path) => {
        return pathUtils.checkExists(path);
    });
    ipcMain.handle("joinPath", async (event, base, second) => {
        return pathUtils.joinPath(base, second);
    });
    ipcMain.handle("baseNameOfPath", async (event, path) => {
        return pathUtils.baseNameOfPath(path);
    });
    ipcMain.handle("writeFile", async (event, path, data, encoding) => {
        return pathUtils.writeFile(path, data, encoding);
    });
    ipcMain.handle("readFile", async (event, path, encoding) => {
        return pathUtils.readFile(path, encoding);
    });
    ipcMain.handle("appendUtf8File", async (event, path, data) => {
        return pathUtils.appendUtf8file(path, data);
    });
    ipcMain.handle("readUtf8File", async (event, path, offset, encoding) => {
        return pathUtils.readUtf8File(path, offset);
    });
    ipcMain.handle("readMime", async (event, path) => {
        return pathUtils.readMime(path);
    });
    ipcMain.handle("createDirectories", async (event, path) => {
        return pathUtils.createDirectories(path);
    });
    ipcMain.handle("readFileModTime", async (event, path) => {
        return pathUtils.readFileModTime(path);
    });
    ipcMain.handle("readFileStats", async (event, path) => {
        return pathUtils.readFileStats(path);
    });
    ipcMain.handle("listPaths", async (event, path) => {
        return pathUtils.listPaths(path);
    });
    ipcMain.handle("copyPath", async (event, from, to) => {
        await pathUtils.copyPath(from, to);
    });
    ipcMain.handle("renamePath", async (event, path, newPath) => {
        return pathUtils.renamePath(path, newPath);
    });
    ipcMain.handle("removePath", async (event, path) => {
        return pathUtils.removePath(path);
    });
    ipcMain.handle("acquireLockfile", async(event, path) => {
        try {
            await lockfile.lock(path);
            return true;
        } catch(e) {
            console.log("Error acquiring file: " + e);
            return false;
        }
    }),
    ipcMain.handle("releaseLockfile", async (event, path) => {
        await lockfile.unlock(path);
    }),
    ipcMain.handle("watchFile", async (event, path) => {
        fileMonitor.subscribe(path);
    }),
    ipcMain.handle("unwatchFile", async (event, path) => {
        fileMonitor.unsubscribe(path);
    }),
    ipcMain.handle("takeChecksumOfFile", async (event, path, checksumType) => {
        return pathUtils.takeChecksumOfFile(path, checksumType);
    }),
    ipcMain.handle("generateThumbnailSharp", (event, source, dest) => {
        return thumbnailWorker.postJob(source, dest);
    }),
    ipcMain.handle("longPollThumbnailJob", async (event, jobId) => { //this sounds sexual
        return await thumbnailWorker.longPollJob(jobId);
    }),
    ipcMain.handle("backend-entrypoint", (event, ...args) => {
        console.log('deprecated backend-entrypoint');
        return;
    });
}


export { setupCommonIPC }