export interface FileWithPath extends File
{
  path: string;
}

class FileTree
{
  files = new Array<FileWithPath>();
  children = new Map<string, FileTree>();


  howManyFileUgot(): number
  {
    let count = this.files.length;
    for(const [name, child] of this.children)
      count += child.howManyFileUgot();

    return count;
  }

  private async turnFileIntoFileImSoTired(item: any): Promise<File>
  {
    return new Promise<File>((resolve: any, reject: any) => {
      item.file(resolve);
    });
  }

  private async turnDirectoriesEntryIntoList(item: any, reader: any): Promise<Array<any>>
  {
    return new Promise<Array<any>>((resolve: any, reject: any) => {
      reader.readEntries(resolve, reject);
    });
  }

  async digestWebkits(webkits: Array<any>)
  {
    console.log("Digesting webkits: " + webkits.length);
    let dirCount = 0;
    let fileCount = 0;
    let skipCount = 0;

    for(const item of webkits)
    {
      if(item.isFile)
      {
        //console.log("File!");
        const result = await this.turnFileIntoFileImSoTired(item);
        //console.log("Got result");
        this.files.push(result as FileWithPath);
        //console.log("len: " + theFiles.length);
        ++fileCount;
      }
      else if(item.isDirectory)
      {
        //console.log("Directory");

        const reader = item.createReader();

        let entries = new Array<any>();

        while(true)
        {
          const batch = await this.turnDirectoriesEntryIntoList(item, reader);

          //console.log("Batch: " + batch);

          if(batch.length == 0)
            break;

          entries = [...entries, ...batch];
        }

        //console.log("Entries: " + entries);

        const child = new FileTree();

        await child.digestWebkits(entries);

        this.children.set(item.name, child);
        //console.log("Done");

        ++dirCount;
      }
      else
      {
        //console.log("Neither...");
        ++skipCount;
      }
    }

    console.log("Files digested: " + fileCount);
    console.log("Dirs digested: " + dirCount);
    console.log("Skipped: " + skipCount);
  }
}

async function fileTreeFromDataTransferList(unsanitized: DataTransferItemList): Promise<FileTree> {
    //console.log("Transfer list size: " + theFilesUnsanitized.length);

    const theWebKits = new Array<any>();

    console.log("Starting theFilesUnsanitized lenght: " + unsanitized.length)

    for(const eitem of unsanitized)
    {
      if(!eitem)
        continue;

      //console.log("Eh??");

      if(!eitem.webkitGetAsEntry)
      {
        console.log("No webkitgetasentry");
        continue;
      }

      const item : any = eitem.webkitGetAsEntry();
      if(!item)
      {
        console.log("Skipping, could not get as webkit entry");
        continue;
      }

      //Because... if we call readDirectories and stuff... For some reason
      // theFilesUnsanitized becomes empty... because of course it fucking does??
      theWebKits.push(item);
    }

    console.log("Number of files converted to webkit entries: " + theWebKits.length);

    //Important we do this after.... Cause we get bullshit DataTransferItemLists every drop.
    //Don't wanna make a new tag for each one...

    const directs = new FileTree();
    await directs.digestWebkits(theWebKits);
    return directs;
}

export {FileTree, fileTreeFromDataTransferList}