import {ContextMenuData} from "./components/contextmenu";

export interface BuildInfo {
    branch: string;
    appTitle: string;
    buildTimestamp: number;
    version: string;
    CommitSHA: string;
}

interface DirectoryListing {
    dirs: string[];
    files: string[];
}

interface DataRead {
    data: string,
    bytesRead: number;
}

interface MetaApi {
    dirName: () => Promise<string>;
    justDie: () => void;
    getAuthKey: () => Promise<string>;
    getServerEndpoint: () => Promise<string>;
    getLicensePath: () => Promise<string>;
    openUrl: (url: string) => void;
    openSelectDirectory: () => Promise<string[]|undefined>;
    getBuildInfo: () => Promise<BuildInfo>;
    spawnMenu: (data: any) => void;

    //Pathutils api
    getConfigPath: () => Promise<string>;
    checkPathExists: (path: string) => Promise<boolean>;
    joinPath: (base: string, second: string) => Promise<string>;
    baseNameOfPath: (path: string) => Promise<string>;
    writeFile: (path: string, data: any, encoding: string | null) => Promise<any>;
    readFile: (path: string) => Promise<any>;
    appendUtf8File: (path: string, data: string) => Promise<any>;
    readUtf8File: (path: string) => Promise<DataRead>;
    readMime: (path: string) => Promise<string>;
    createDirectories: (path: string) => Promise<any>;
    readFileModTime: (path: string) => Promise<string>;
    readFileStats: (path: string) => Promise<any>; //todo make typed right
    listPaths: (path: string) => Promise<DirectoryListing | undefined>;
    copyPath: (from: string, to: string) => Promise<void>;
    renamePath: (path: string, newPath: string) => Promise<any>;
    removePath: (path: string) => Promise<any>;
    takeChecksumOfFile: (path: string, checksumType: string) => Promise<string>;

    acquireLockfile: (path: string) => Promise<boolean>;
    releaseLockfile: (path: string) => Promise<void>;

    watchFile: (path: string) => Promise<any>;
    unwatchFile: (path: string) => Promise<any>;

    registerGlobalFilechangeHandler: (callback: (args: any[]) => void) => void;

    generateThumbnailSharp: (source: string, dest: string) => Promise<number>;
    longPollThumbnailJob: (jobId: number) => Promise<void>;

    backendEntrypoint: (endpoint: string, payload: string) => Promise<string>;
}

declare var metaApi: MetaApi; //filled by preload

function getMetaApi(): MetaApi {
    return metaApi;
}

function justDie()
{
    metaApi.justDie();
    //window.electron.justDie();
    /*
    const win: any = (remote as any).getCurrentWindow();
    if(!win)
    {
        console.warn("Not an electron app");
        return;
    }

    win.close();
    */
}

//Isn't all this kind of redundant? I could just... Like...
// use the metaApi directly?

let authKey: string | undefined;

async function getAuthKey()
{
    if(!authKey)
        authKey = await metaApi.getAuthKey();

    if(!authKey)
        throw "logic error";
    
    return authKey;
}

//idek where else to put this
async function getServerEndpoint()
{
    return await metaApi.getServerEndpoint();
}

function openUrl(url: string)
{
    metaApi.openUrl(url);
}

async function handleGotoEnesdaPage(slug: string)
{
    const base = await getServerEndpoint();
    openUrl(base + "/" + slug);
}

async function openSelectDirectory(): Promise<string[] | undefined>
{
    return metaApi.openSelectDirectory();
}


function getBuildInfo(): Promise<BuildInfo>
{
    return metaApi.getBuildInfo();
}

function spawnContextMenu(data: ContextMenuData)
{
    metaApi.spawnMenu(data);
}


function backendEntrypoint(endpoint: string, body: object): Promise<string>
{
    //console.log("Invoking backend entrypoint for: " + endpoint);
    //const cleanBody = body != undefined ? JSON.stringify(body) : "";
    //console.log("With body: " + cleanBody);
    return metaApi.backendEntrypoint(endpoint, JSON.stringify(body));
}

export {justDie,
    getAuthKey,
    getServerEndpoint,
    openUrl,
    openSelectDirectory,
    handleGotoEnesdaPage,
    getBuildInfo,
    spawnContextMenu,
    backendEntrypoint,
    getMetaApi,
}