import { app, BrowserWindow, ipcMain, shell } from 'electron';
import path from 'path';
import { setupCommonIPC } from './electron-common.js';

import { getBuildInfo } from './buildInfo.js';

const devServerEndpoint = "http://localhost:3007";
const prodServerEndpoint = "https://enesda.com";

// Handle creating/removing shortcuts on Windows when installing/uninstalling.
if (require('electron-squirrel-startup')) {
  console.log("quitting because squirrel startup");
  app.quit();
} else { //otherwise it fucking restarts during install

  // This method will be called when Electron has finished
  // initialization and is ready to create browser windows.
  // Some APIs can only be used after this event occurs.
  app.on('ready', initializeWindow);

  // Quit when all windows are closed, except on macOS. There, it's common
  // for applications and their menu bar to stay active until the user quits
  // explicitly with Cmd + Q.
  app.on('window-all-closed', () => {
    if (process.platform !== 'darwin') {
      app.quit();
    }
  });

  app.on('activate', () => {
    // On OS X it's common to re-create a window in the app when the
    // dock icon is clicked and there are no other windows open.
    if (BrowserWindow.getAllWindows().length === 0) {
      initializeWindow();
    }
  });
}


async function initializeWindow() {
  // Create the browser window.

  const devBuild = !!MAIN_WINDOW_VITE_DEV_SERVER_URL;

  /*
  const buildInfo = {
    "build": devBuild ? "~DEVELOPMENT BUILD NOT FOR REDISTRIBUTION~" : app.getVersion(),
    "title": devBuild ? "~BINDER - TEST - NOT FOR RELEASE~" : "Binder",
    "date": -380484000000
  }

  //*/
  const buildInfo = await getBuildInfo(devBuild);

  const iconName = (() => {
    //Using process.platform instead of os.platform because for some reason === comparison
    // fails on 'win32' but not == comparison, and... It doesn't have that problem here?
    if(process.platform === 'win32') {
      //why windows is so bad at handling png scaling in 2023 beats the shit out of me
      return 'icon.ico';
    } else {
      return 'icon.png';
    }
  })();

  const iconPath = path.join(app.getAppPath(), "icons", iconName);

  const win = new BrowserWindow({
    width: devBuild ? 1500 : 800,
    height: devBuild ? 700 : 600,
    frame: true,
    icon: iconPath,
    show: false,
    webPreferences: {
      webSecurity: !devBuild,
      preload: path.join(__dirname, 'preload.js'),
    },
  });

  win.setMenu(null);


  // and load the index.html of the app.
  if (MAIN_WINDOW_VITE_DEV_SERVER_URL) {
    win.loadURL(MAIN_WINDOW_VITE_DEV_SERVER_URL);
    win.webContents.openDevTools();

    //And now specific endpoints that vary by prod/dev

    ipcMain.handle('get-server-endpoint', () => {
      return devServerEndpoint;
    });

    ipcMain.handle('get-license-path', () => {
      return "LICENSE.txt";
    });
  } else {
    win.loadFile(path.join(__dirname, `../renderer/${MAIN_WINDOW_VITE_NAME}/index.html`));

    //And now specific endpoints that vary by prod/dev
    ipcMain.handle('get-server-endpoint', () => {
      return prodServerEndpoint;
    });

    ipcMain.handle('get-license-path', () => {
      return path.join(app.getAppPath(), "LICENSE.txt");
    });
  }

  const debugAuthKey = "fhddsfhyuhyudhyue778-3-s-qw-wedff-d-s9987xuxdh"; //Securely generated by slamming my fingers onto the keyboard
  setupCommonIPC(debugAuthKey, win);

  ipcMain.handle('get-build-info', (event) => {
    return getBuildInfo(devBuild);
  });

  win.webContents.setWindowOpenHandler((deets) => {
    //I mean, for now it's fine if we don't want like, to do anything other than open a thing in a thing
    // but in the future might want to scan the url to make sure it's from enesda or something...
    //Actually probably should do that as a safety measure anyway...
    //Although enesda.com is already compromised if we get this far... probably...
    if(devBuild) {
      shell.openExternal(deets.url);
    } else {
      if (deets.url.startsWith(prodServerEndpoint)) {
        shell.openExternal(deets.url);
      }
    }

    return {action: 'deny'};
  });

  win.once('ready-to-show', () => {
      win.show();
  });
};

// In this file you can include the rest of your app's specific main process
// code. You can also put them in separate files and import them here.
