import { getMetaApi } from "./ipcbindings";

class FileMonitor {
    private lastId = 0;
    private subscriptions = new Map<number, (event: string, id: number) => void>();

    count() {
        return this.subscriptions.size;
    }

    subscribe(callback: (event: string, id: number) => void) {
        const id = ++this.lastId;
        this.subscriptions.set(id, callback);
        return id;
    }

    unsubscribe(id: number) {
        this.subscriptions.delete(id);
    }

    trigger(event: string) {
        for(const [id, callback] of this.subscriptions) {
            callback(event, id);
        }
    }
}

const fileMonitors = new Map<string, FileMonitor>();


function registerFileChangeWatcher(path: string, callback: (event: string, id: number) => void) {
    let mon = fileMonitors.get(path);

    if(!mon) {
        mon = new FileMonitor();
        getMetaApi().watchFile(path);
    }

    return mon.subscribe(callback);
}

function unregisterFileChangeWatcher(path: string, id: number) {

    const mon = fileMonitors.get(path);
    if(!mon) {
        console.log("Warning, call to unregister file change watcher but there this path is not registered")
        return;
    }

    mon.unsubscribe(id);

    if (!mon.count()) {
        getMetaApi().unwatchFile(path);
    }
}

getMetaApi().registerGlobalFilechangeHandler((args: any[]) => {
    if(args.length != 2) {
        throw new Error("Wrong number of arguments for file change observer!");
    }

    const path = args[0];
    const event = args[1];

    const mon = fileMonitors.get(path);

    if(!mon) {
        console.warn("No file monitors registered for file");
        return;
    }

    mon.trigger(event);
});

export { registerFileChangeWatcher, unregisterFileChangeWatcher }