import {Worker} from 'node:worker_threads';
import {app} from 'electron';
import path from 'node:path';

const thumbnailWorkerString = "thumbnailWorker";

class Job {
    pair = {};
    thumbnailDir = "";
    callbacks = new Map(); //number => {resolve, reject}

    constructor(pair, thumbnailDir) {
        this.thumbnailDir = thumbnailDir;
        this.pair = pair;
    }

    register(jobId, resolve, reject) {
        this.callbacks.set(jobId, {resolve: resolve, reject: reject});
    }

    unregister(jobId) {
        let callbees = this.callbacks.get(jobId);
        if(!callbees) {
            return;
        }

        callbees.reject();

        this.callbacks.delete(jobId);
    }

    checkEmpty() {
        return this.callbacks.size == 0;
    }
}

class ThumbnailWorker {
    currentJob;

    worker;

    lastJobId = 0;


    jobs = new Map(); // path pair => Job
    reverseJobMap = new Map(); //number -> path pair


    //construction happens in the main thread
    constructor() {
        this.init();
    }

    async init() {
        //const workerUrl = new URL('./thumbnailworkercode.mjs', import.meta.url);
        //all that work and it's basically where it started??????
        //const workerUrlBase = MAIN_WINDOW_VITE_DEV_SERVER_URL ? app.getAppPath() : path.join(process.resourcesPath, "app.asar");
        const workerUrlBase = app.getAppPath();

        const workerPath = path.join(workerUrlBase, ".vite", "build", "thumbnailworkercode.mjs");
        this.worker = new Worker(workerPath);
        if (!this.worker) {
            throw new Error("worker undefined help");
        }

        ///*
        this.worker.on("message", (message) => {
            if (!("succ" in message)) {
                console.warn("Badly formed message from worker");
                this.finalizeCurrentJob({ succ: false });
            } else {
                this.finalizeCurrentJob(message);
            }
        });
        //*/


        /*
        this.thread = new Worker(new URL('./thumbnailworkercode.js', import.meta.url), {type: "module"});

        this.thread.onmessage = (e) => {

            if(!("succ" in e.data) || !("buffer" in e.data)) {
                console.warn("Badly formed message from worker");
                this.finalizeCurrentJob({succ: false, buffer: undefined});
            } else {
                this.finalizeCurrentJob(e.data);
            }
        };
        */
    }

    async finalizeCurrentJob(data /*: {succ: boolean, buffer: ArrayBuffer | undefined} */) {
        if(!this.currentJob) {
            console.warn("Current job not set in finalzie!!!!");
            return;
        }

        let succ = data.succ;

        /*
        if(data.succ) {
            if(!data.buffer) {
                console.warn("Cannot be succ and bad buffer")
            } else {
                try {
                    const view = new DataView(data.buffer);
                    console.log("CHECK TO SEE IF THIS CAN BE DONE IN THE WORKER THREAD");
                    await pathUtils.createDirectories(this.currentJob.thumbnailDir);
                    await pathUtils.writeFile(this.currentJob.pair.destPath, view, null);
                } catch(e) {
                    console.log("Failed to write file: " + e);
                    succ = false;
                }
            }
        }
        */


        for(const [jobId, callbacks] of this.currentJob.callbacks) {
            callbacks.resolve(succ);

            this.reverseJobMap.delete(jobId);
        }

        this.jobs.delete(this.currentJob.pair);
        this.currentJob = undefined;
        this.nextJobIfReady();
    }

    nextJobIfReady() {
        if(this.currentJob || !this.worker) {
            return;
        }

        //Insertion order iteration, thank god
        for(const [, val] of this.jobs) {
            this.currentJob = val;
            break;
        }

        if(!this.currentJob) {
            return;
        }

        this.worker.postMessage(this.currentJob.pair);
    }

    postJob(objectPath, thumbnailPath, thumbnailDir) {
        const jobId = ++this.lastJobId;
        const pathPair = {sourcePath: objectPath, destPath: thumbnailPath};

        let job = this.jobs.get(pathPair);

        if(!job) {
            job = new Job(pathPair, thumbnailDir);
        }

        this.jobs.set(pathPair, job);

        this.reverseJobMap.set(jobId, pathPair)

        this.nextJobIfReady();

        return jobId;
    }

    async longPollJob(jobId) {
        let pathPair = this.reverseJobMap.get(jobId);

        if(!pathPair) {
            throw new Error("Could not find job pair");
        }

        let job = this.jobs.get(pathPair);

        if(!job) {
            return; // should just resolve
            throw new Error("Job does not exist");
        }

        return new Promise((resolve, reject) => {
            job.register(jobId, () => {
                resolve();
            }, () => {
                reject();
            });
        })
    }

    cancelJob(jobId) {
        const pair = this.reverseJobMap.get(jobId);

        if(!pair) {
            console.warn("Could not find job id in cancellation method!");
            return;
        }

        this.reverseJobMap.delete(jobId);

        const job = this.jobs.get(pair);

        if(!job) {
            console.warn("Could not find job pair, already cancelled?");
            return;
        }

        job.unregister(jobId);

        if(job.checkEmpty()) {
            this.jobs.delete(pair);
        }
    }
}


const thumbnailWorker = new ThumbnailWorker()

export {thumbnailWorker}