import { getOverlayStator, OverlayStatorStates } from "./components/overlays";
import { StatorContext, StatorObjectBase } from "./statorobject";
import { getMetaApi } from "./ipcbindings";

async function getBinderVersion() {
    const buildInfo = await getMetaApi().getBuildInfo();
    return buildInfo.version;
}

enum UpdaterStatorState //I'm having a go here
{
    FailedToCheck = -2,
    Checking = -1,
    Unknown = 0,
    UpToDate = 1,
    UpdateAvailable = 2,
}

class UpdaterStator extends StatorObjectBase
{
    private updaterState = UpdaterStatorState.Unknown;

    async checkForUpdates()
    {
        if(this.updaterState == UpdaterStatorState.Checking)
            return; //skip yeh it's already doing it
        
        this.updaterState = UpdaterStatorState.Checking;
        this.publishChanges();

        //key is an optional parameter
        const serverBaseEndpoint = await getMetaApi().getServerEndpoint();
        const binderV = await getBinderVersion();
        const endpoint = serverBaseEndpoint + "/api/binder/v1/updatecheck/" + binderV;

        try
        {
            const request = new Request(endpoint, {method: "GET"});
            const data = await(await fetch(request)).json();


            if(data["updateAvailable"])
            {
                this.updaterState = UpdaterStatorState.UpdateAvailable;
            }
            else
            {
                this.updaterState = UpdaterStatorState.UpToDate;
            }

            const overlayStator = getOverlayStator(this.getContext());

            if (this.updaterState >= UpdaterStatorState.UpdateAvailable
                && overlayStator.myShowState == OverlayStatorStates.showNone)
            {
                getOverlayStator(this.getContext()).set(OverlayStatorStates.showUpdater);
            }
        }
        catch
        {
            this.updaterState = UpdaterStatorState.FailedToCheck;
        }

        this.publishChanges();
    }

    getState()
    {
        return this.updaterState;
    }

    reset()  
    {
        this.updaterState = UpdaterStatorState.Unknown;
        this.publishChanges();
    }
}

const UpdaterStatorKey = "UpdaterStator";

function useWatchUpdaterStator(appContext: StatorContext)
{
    return appContext.useWatchStatorLegacy(UpdaterStatorKey, new UpdaterStator(appContext)) as UpdaterStator;
}

function getUpdaterStator(appContext: StatorContext)
{
    return appContext.getStatorLegacy(UpdaterStatorKey, new UpdaterStator(appContext)) as UpdaterStator;
}

export {UpdaterStatorState, UpdaterStator, useWatchUpdaterStator, getUpdaterStator, getBinderVersion }